/* Matthias Ullmann, 2.12.1994				*/
/* Routines to read atoms from CRD-files.		*/
/*							*/
/* The energy parameters read from CHARMM-Parameter 	*/
/* files are changed from kcal to kJ by multipling the 	*/
/* value with CAL2J from "constant.h"			*/
/*......................................................*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "mol.h"
#include "mathematik.h"
#include "crd_atom.h"
#include "util.h"
#include "memutil.h"

/*..............................................................*/
/* The procedure reads a PARAM.PRM file				*/
/* Zeilen fortsetzung mit "-" ist nicht erlaubt			*/
void read_PARAM(char *filename, PARAM_record *param)
{
 /* Variables */
 FILE *fpparam;

 fpparam=file_open(filename,"r");
 count_records(fpparam,param);
 rewind(fpparam);
 allocate_PARAM(param);
 read_records(fpparam,param);
}
/*..............................................................*/

int StopReading(char *buff)
{
  if (0==strncmp(buff,"BOND",     4)) return 0;
  if (0==strncmp(buff,"THETA",    4)) return 0;
  if (0==strncmp(buff,"ANGLE",    4)) return 0;
  if (0==strncmp(buff,"PHI",      3)) return 0;
  if (0==strncmp(buff,"DIHE",     4)) return 0;
  if (0==strncmp(buff,"IMPHI",    4)) return 0;
  if (0==strncmp(buff,"IMPROP",   4)) return 0;
  if (0==strncmp(buff,"NONBONDED",4)) return 0;
  if (0==strncmp(buff,"NBONDED",  4)) return 0;
  if (0==strncmp(buff,"NBFIX",    4)) return 0;
  if (0==strncmp(buff,"HBOND",    4)) return 0;
  if (0==strncmp(buff,"END",      3)) return 0;
  if (0==strncmp(buff,"CMAP",     4)) return 0;

  return 1;
}

/*..............................................................*/
/* Counting the number of records 				*/
void count_records(FILE *fpparam, PARAM_record *param)
{
 char buff[LINELENGTH], buff1[LINELENGTH];
 int i, n;

 /* count bond entries */
 i=0;
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if (!strncmp(buff,"BOND",4)) 
	  {while (fgets(buff,LINELENGTH,fpparam)!=NULL)
	     {if (0==StopReading(buff)) goto endbond;
              n=sscanf(buff,"%s",buff1);
              if ((buff1[0]!='!')&&(n==1)) {i++;}
             }
	  }
      }
 endbond: /* label */
 (*param).nbonds=i;

 /* count angle entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"THETA",4))||(!strncmp(buff,"ANGLE",4))) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto endangle;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1))  {i++;}
           }
      }
 endangle: /* label */
 (*param).nangles=i=i;

 /* count dihedral entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"PHI",3))||(!strncmp(buff,"DIHE",4))) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto enddihe;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) i++;
           }
      }
 enddihe: /* label */
 (*param).ntorsions=i;

 /* count improper angles entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"IMPHI",4))||(!strncmp(buff,"IMPROP",4))) 
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto endimprop;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) i++;
           }
      }
 endimprop: /* label */
 (*param).nimprops=i;

 /* count nonbonded entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NONBONDED",4))||(!strncmp(buff,"NBONDED",4)))
         {if (buff[strlen(buff)]=='-')
	     {while (fgets(buff,LINELENGTH,fpparam)!=NULL)
	        {if (buff[strlen(buff)]!='-') break;
		}
	     }
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if (0==StopReading(buff)) goto endnonbonded;
                 n=sscanf(buff,"%s",buff1);
                 if ((buff1[0]!='!')&&(n==1)) {i++;}
                }
         }
     }
 endnonbonded: /* label */
 (*param).nnonbonded=i;

 /* count nbfix entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NBFIX",4))) 
      while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto endnbfix;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)){i++;}
           }
     }
 endnbfix: /* label */
 (*param).nnbfix=i;

 /* count hbond entries */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"HBOND",4))) 
         {while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if (buff[strlen(buff)]!='-')
                    break;
                }
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
                {if (0==StopReading(buff)) goto endhbond;
                 n=sscanf(buff,"%s",buff1);
                 if ((buff1[0]!='!')&&(n==1)) {i++;}
                }
         }
     }
 endhbond: /* label */
 (*param).nhbond=i;

}

/*..............................................................*/
/* Allocation of the structure elements				*/
void allocate_PARAM(PARAM_record *param)
{
    fprintf(stderr,"number_of_bonds:     %d\n",(*param).nbonds);
    fprintf(stderr,"number_of_angles:    %d\n",(*param).nangles);
    fprintf(stderr,"number_of_torsions:  %d\n",(*param).ntorsions);
    fprintf(stderr,"number_of_improps:   %d\n",(*param).nimprops);
    fprintf(stderr,"number_of_nonbonded: %d\n",(*param).nnonbonded);
    fprintf(stderr,"number_of_nbfix:     %d\n",(*param).nnbfix);
    fprintf(stderr,"number_of_hbond:     %d\n",(*param).nhbond);


 if ((*param).nbonds>0)
    {(*param).bond1	=charmatrix(1,(*param).nbonds,0,4); 	/*[5][N]*/
     (*param).bond2	=charmatrix(1,(*param).nbonds,0,4); 	/*[5][N]*/
     (*param).K_bond	=dvector(1,(*param).nbonds);	   
     (*param).bond_0	=dvector(1,(*param).nbonds);	   
    }

 if ((*param).nangles>0)
    {(*param).angle1	=charmatrix(1,(*param).nangles,0,4); 	/*[5][N]*/
     (*param).angle2	=charmatrix(1,(*param).nangles,0,4); 	/*[5][N]*/
     (*param).angle3	=charmatrix(1,(*param).nangles,0,4); 	/*[5][N]*/
     (*param).K_angle=dvector(1,(*param).nangles);	   
     (*param).angle_0=dvector(1,(*param).nangles);	   
     (*param).K_ub=dvector(1,(*param).nangles);	   
     (*param).dist_ub_0=dvector(1,(*param).nangles);	   
    }

 if ((*param).ntorsions>0)
    {(*param).torsion1 =charmatrix(1,(*param).ntorsions,0,4); 	/*[5][N]*/
     (*param).torsion2 =charmatrix(1,(*param).ntorsions,0,4); 	/*[5][N]*/
     (*param).torsion3 =charmatrix(1,(*param).ntorsions,0,4); 	/*[5][N]*/
     (*param).torsion4 =charmatrix(1,(*param).ntorsions,0,4); 	/*[5][N]*/
     (*param).K_torsion=dvector(1,(*param).ntorsions);	   
     (*param).delta	  =dvector(1,(*param).ntorsions);	   
     (*param).n	  =ivector(1,(*param).ntorsions);   
    }

 if ((*param).nimprops>0)
    {(*param).improp1 =charmatrix(1,(*param).nimprops,0,4); 	/*[5][N]*/
     (*param).improp2 =charmatrix(1,(*param).nimprops,0,4); 	/*[5][N]*/
     (*param).improp3 =charmatrix(1,(*param).nimprops,0,4); 	/*[5][N]*/
     (*param).improp4 =charmatrix(1,(*param).nimprops,0,4); 	/*[5][N]*/
     (*param).K_improp=dvector(1,(*param).nimprops);	   
     (*param).improp_0=dvector(1,(*param).nimprops);	   
    }

 if ((*param).nnonbonded>0)
    {(*param).atom 		=charmatrix(1,(*param).nnonbonded,0,4); /*[5][N]*/
     (*param).polarizability	=dvector(1,(*param).nnonbonded);	   
     (*param).Emin		=dvector(1,(*param).nnonbonded);	   
     (*param).Rmin		=dvector(1,(*param).nnonbonded);	   
     (*param).polarizability1_4	=dvector(1,(*param).nnonbonded);	   
     (*param).Emin1_4	=dvector(1,(*param).nnonbonded);	   
     (*param).Rmin1_4	=dvector(1,(*param).nnonbonded);	   
    }

 if ((*param).nnbfix>0)
    {(*param).atom1	  =charmatrix(1,(*param).nnbfix,0,4);
     (*param).atom2	  =charmatrix(1,(*param).nnbfix,0,4);
     (*param).EminNBfix=dvector(1,(*param).nnbfix);
     (*param).RminNBfix=dvector(1,(*param).nnbfix);
    }	

 if ((*param).nhbond>0)
    {(*param).hatom1=charmatrix(1,(*param).nhbond,0,4);
     (*param).hatom2=charmatrix(1,(*param).nhbond,0,4);
     (*param).para1=dvector(1,(*param).nhbond);		
     (*param).para2=dvector(1,(*param).nhbond);	
    }	
}
/*------------------------------------------------------*/
/* Now read the parameter file				*/
/* The read energy constants are in kJ/mol              */

void read_records(FILE *fpparam,PARAM_record *param)
{
 char buff[LINELENGTH], buff1[LINELENGTH], help[LINELENGTH], 
      help1[LINELENGTH], help2[LINELENGTH];
 int i, n, m;

 /* Read Bond Records */
 i=0;
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if (!strncmp(buff,"BOND",4))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto endbond;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
               {i++;
                if (i>(*param).nbonds) ERROR("too many bond entries");
	        sscanf(buff,"%s %s %lf %lf",(*param).bond1[i],
				            (*param).bond2[i],
				           &(*param).K_bond[i],
				           &(*param).bond_0[i]);
               }
           }
      }
 endbond: /* label */
 if (i!=(*param).nbonds) ERROR("too few bond entries");

 /* Read Angle Records */
 /* CHANGE the Urey-Bradley terms is not red */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"THETA",4))||(!strncmp(buff,"ANGLE",4)))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto endangle;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
               {i++;
                if (i>(*param).nangles) ERROR("too many angles entries");
	        m=sscanf(buff,"%s %s %s %lf %lf %s %s",(*param).angle1[i], 
					       (*param).angle2[i],
					       (*param).angle3[i],
				       	      &(*param).K_angle[i],
					      &(*param).angle_0[i],
                                              help1, help2);
	        if (m>=5)
                   {if (help1[0]!='!')
                       {(*param).K_ub[i]=atof(help1);
                        (*param).dist_ub_0[i]=atof(help2);
                       }
                   }
                else {(*param).K_ub[i]=0.0; (*param).dist_ub_0[i]=0.0;} 

               }
           }
      }
 endangle: /* label */
 if (i!=(*param).nangles) ERROR("too few angles entries");

 /* Read Torsion Records */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"PHI",3))||(!strncmp(buff,"DIHE",4)))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto enddihe;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
                {i++;
                 if (i>(*param).ntorsions) ERROR("too many torsion entries");
	         sscanf(buff,"%s %s %s %s %lf %d %lf",(*param).torsion1[i], 
					  	      (*param).torsion2[i],
					  	      (*param).torsion3[i],
					  	      (*param).torsion4[i],
				       	  	     &(*param).K_torsion[i],
						     &(*param).n[i],
					  	     &(*param).delta[i]);
                }
           }
      }
 enddihe: /* label */
 if (i!=(*param).ntorsions) ERROR("too few  torsion entries");

 /* Read Improper torsion records */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
      {if ((!strncmp(buff,"IMPHI",4))||(!strncmp(buff,"IMPROP",4)))
       while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) goto endimprop;
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
                {i++;
                 if (i>(*param).nimprops) ERROR("too many improper entries");
	         sscanf(buff,"%s %s %s %s %lf %*d %lf",(*param).improp1[i], 
					              (*param).improp2[i],
					              (*param).improp3[i],
					              (*param).improp4[i],
				       	             &(*param).K_improp[i],
					             &(*param).improp_0[i]);
                }
           }
      }
 endimprop: /* label */
 if (i!=(*param).nimprops) ERROR("too few improper entries");

 /* Read nonbonded interactions */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NONBONDED",4))||(!strncmp(buff,"NBONDED",4)))
         {/* CHANGE: the nonbonded settings need to be read somehow!!! */
          if ((*param).nnonbonded==0) {goto endnonbonded;}
          if (buff[strlen(buff)]=='-')
	     {while (fgets(buff,LINELENGTH,fpparam)!=NULL)
	        {if (buff[strlen(buff)]!='-') break;
		}
	     }
          while (fgets(buff,LINELENGTH,fpparam)!=NULL)
               {if (0==StopReading(buff)) {goto endnonbonded;}
                n=sscanf(buff,"%s",buff1);
                if ((buff1[0]!='!')&&(n==1))
                   {i++;
                    if (i>(*param).nnonbonded) 
		       {fprintf(stderr, "%d %d\n", i, (*param).nnonbonded);
                        ERROR("too many nonbonded entries");
		       }
	            if (StringPerLine(buff)>=7)
	               {sscanf(buff,"%s %*f %lf %lf %s %lf %lf",(*param).atom[i], 
					     	    &(*param).Emin[i],
					     	    &(*param).Rmin[i],
					     	    help,
					     	    &(*param).Emin1_4[i],
					     	    &(*param).Rmin1_4[i]);
                        if (help[0]=='!')
                           {(*param).Emin1_4[i]=(*param).Emin[i];
                            (*param).Rmin1_4[i]=(*param).Rmin[i];
                           }
                       }
	             else {if ((StringPerLine(buff)<7) && (StringPerLine(buff)>=4))
	                      {sscanf(buff,"%s %*f %lf %lf",(*param).atom[i], 
					     	       &(*param).Emin[i],
					     	       &(*param).Rmin[i]);
		               (*param).Emin1_4[i]=(*param).Emin[i];
		               (*param).Rmin1_4[i]=(*param).Rmin[i];
	                      }
		           else {WARN("Number of elements not correct in Line.");
		                 fprintf(stderr,"Number of words in Line: %d\n", 
                                         StringPerLine(buff));
		                 fprintf(stderr,"%s", buff);
		                }
                          }
                   }
               }
         }
     }
 endnonbonded: /* label */
 if (i!=(*param).nnonbonded) ERROR("too few nonbonded entries");

 /* Read nbfix interactions */
 i=0; rewind(fpparam);
 while (fgets(buff,LINELENGTH,fpparam)!=NULL)
     {if ((!strncmp(buff,"NBFIX",4)))
      while (fgets(buff,LINELENGTH,fpparam)!=NULL)
           {if (0==StopReading(buff)) {goto endnbfix;}
            n=sscanf(buff,"%s",buff1);
            if ((buff1[0]!='!')&&(n==1)) 
	       {i++;
                if (i>(*param).nnbfix) ERROR("too many nbfix entries");
	           {sscanf(buff,"%s %s %lf %lf",(*param).atom1[i], 
				                (*param).atom2[i],
				               &(*param).EminNBfix[i],
				               &(*param).RminNBfix[i]);
                   }
               }
           }
     }
 endnbfix: /* label */

 if (i!=(*param).nnbfix) ERROR("too few nbfix entries");

 /* CHANGE -- HBOND not read yet - to be implemented */


}

/*..............................................................*/
void fprint_BONDS(FILE *fpout, PARAM_record *param)
{
int i;

for (i=1; i<=(*param).nbonds; i++)
    fprintf(fpout,"%5s %5s %8.3f %8.3f\n",(*param).bond1[i],
				  (*param).bond2[i],
				  (*param).K_bond[i],
				  (*param).bond_0[i]);
}
/*..............................................................*/
void fprint_THETA(FILE *fpout, PARAM_record *param)
{
int i;

for (i=1; i<=(*param).nangles; i++)
    fprintf(fpout,"%5s %5s %5s %8.3f %8.3f %8.3f %8.3f\n",(*param).angle1[i], 
				     (*param).angle2[i],
				     (*param).angle3[i],
				     (*param).K_angle[i],
				     (*param).angle_0[i],
                                     (*param).K_ub[i],
                                     (*param).dist_ub_0[i]);
}
/*..............................................................*/
void fprint_PHI(FILE *fpout, PARAM_record *param)
{
int i;

for (i=1; i<=(*param).ntorsions; i++)
    fprintf(fpout,"%5s %5s %5s %5s %8.3f %5d %8.3f\n",(*param).torsion1[i], 
					    (*param).torsion2[i],
					    (*param).torsion3[i],
					    (*param).torsion4[i],
				       	    (*param).K_torsion[i],
					    (*param).n[i],
					    (*param).delta[i]);
}
/*..............................................................*/
void fprint_IMPHI(FILE *fpout, PARAM_record *param)
{
int i;

for (i=1; i<=(*param).nimprops; i++)
    fprintf(fpout,"%5s %5s %5s %5s %8.3f %8.3f\n",(*param).improp1[i], 
					(*param).improp2[i],
					(*param).improp3[i],
					(*param).improp4[i],
				       	(*param).K_improp[i],
					(*param).improp_0[i]);

}
/*..............................................................*/
void fprint_NBONDED(FILE *fpout, PARAM_record *param)
{
int i;

for (i=1; i<=(*param).nnonbonded; i++)
    fprintf(fpout,"%5s %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
				    (*param).atom[i],
				    0.0,
				    (*param).Emin[i],
				    (*param).Rmin[i],
				    0.0,
				    (*param).Emin1_4[i],
				    (*param).Rmin1_4[i]);
}
/*..............................................................*/
void fprint_NBFIX(FILE *fpout, PARAM_record *param)
{
int i;

for (i=1; i<=(*param).nnbfix; i++)
    fprintf(fpout,"%5s %5s %8.3f %8.3f\n",(*param).atom1[i], 
				    (*param).atom2[i],
				    (*param).EminNBfix[i],
				    (*param).RminNBfix[i]);
}
/*..............................................................*/
/* Print the parameter						*/

void fprint_PARAM(FILE *fpout, PARAM_record *param)
{

 fprintf(fpout,"%s !%d\n",BONDS,(*param).nbonds);
 fprint_BONDS(fpout, param);

 fprintf(fpout,"%s !%d\n",THETA,(*param).nangles);
 fprint_THETA(fpout, param);

 fprintf(fpout,"%s !%d\n",PHI,(*param).ntorsions);
 fprint_PHI(fpout, param);

 fprintf(fpout,"%s !%d\n",IMPHI,(*param).nimprops);
 fprint_IMPHI(fpout, param);

 fprintf(fpout,"%s !%d\n",NBONDED,(*param).nnonbonded);
 fprint_NBONDED(fpout, param);

 fprintf(fpout,"%s !%d\n",NBFIX,(*param).nnbfix);
 fprint_NBFIX(fpout, param);

 fprintf(fpout,"END\n");
}

/*..............................................................*/
/* Print the parameter						*/

void fprint_two_PARAM(FILE *fpout, PARAM_record *param1, PARAM_record *param2)
{
 fprintf(fpout,"%s !%d\n",BONDS,(*param1).nbonds+(*param2).nbonds);
 fprint_BONDS(fpout, param1);
 fprint_BONDS(fpout, param2);

 fprintf(fpout,"%s !%d\n",THETA,(*param1).nangles+(*param2).nangles);
 fprint_THETA(fpout, param1);
 fprint_THETA(fpout, param2);

 fprintf(fpout,"%s !%d\n",PHI,(*param1).ntorsions+(*param2).ntorsions);
 fprint_PHI(fpout, param1);
 fprint_PHI(fpout, param2);

 fprintf(fpout,"%s !%d\n",IMPHI,(*param1).nimprops+(*param2).nimprops);
 fprint_IMPHI(fpout, param1);
 fprint_IMPHI(fpout, param2);

 fprintf(fpout,"%s !%d\n",NBONDED,(*param1).nnonbonded+(*param2).nnonbonded);
 fprint_NBONDED(fpout, param1);
 fprint_NBONDED(fpout, param2);

 fprintf(fpout,"%s !%d\n",NBFIX,(*param1).nnbfix+(*param2).nnbfix);
 fprint_NBFIX(fpout, param1);
 fprint_NBFIX(fpout, param2);

 fprintf(fpout,"END\n");
}

/*..............................................................*/
/* The procedure reads a MASS RTF file				*/
/*								*/

void read_MASS(char *filename, MASS_record *mass)
{
FILE *fpmass;
char buff[LINELENGTH], help[LINELENGTH];
int i=0;

fpmass=file_open(filename,"r");
while (fgets(buff,LINELENGTH,fpmass)!=NULL)
      {if (strncmp(buff,"MASS",4)==0) i++;
      }
(*mass).natomtypes=i;
fprintf(stderr,"Number of Atom Types: %d\n",(*mass).natomtypes);
/* Allocation */
(*mass).number=ivector(1,(*mass).natomtypes);
(*mass).atom_type=charmatrix(1,(*mass).natomtypes,0,5);
(*mass).mass=dvector(1,(*mass).natomtypes);
(*mass).element=charmatrix(1,(*mass).natomtypes,0,2);

rewind(fpmass);
i=0;
while (fgets(buff,LINELENGTH,fpmass)!=NULL)
      {if (strncmp(buff,"MASS",4)==0) 
	  {i++;
	   sscanf(buff,"%s %d %s %lf %s", help, 
					  &(*mass).number[i],
					   (*mass).atom_type[i],
					  &(*mass).mass[i],
					   (*mass).element[i]);
	 }
      }
}
/*..............................................................*/
/* Procedure reads a CRD_atom from a string taken from a CRD-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/*								*/
/* Syntax: read_CRDatom(line, &record)				*/
/*								*/
void read_CRDatom_old(CRD_atom *record, int i,char *line)
{
    sscanf(line,    "%d",   &((*record).atomno[i]));
    sscanf(line+5,  "%d",   &((*record).res_no[i]));
    sscanf(line+11, "%c",    &((*record).residue[i][0]));
    sscanf(line+12, "%c",    &((*record).residue[i][1]));
    sscanf(line+13, "%c",    &((*record).residue[i][2]));
    sscanf(line+14, "%c",    &((*record).residue[i][3]));
    (*record).residue[i][4]='\0';
    sscanf(line+16, "%c",   &((*record).atomid[i][0]));
    sscanf(line+17, "%c",   &((*record).atomid[i][1]));
    sscanf(line+18, "%c",   &((*record).atomid[i][2]));
    sscanf(line+19, "%c",   &((*record).atomid[i][3]));
    (*record).atomid[i][4]='\0';
    sscanf(line+20, "%lf", &((*record).xyz[i][1]));
    sscanf(line+30, "%lf", &((*record).xyz[i][2]));
    sscanf(line+40, "%lf", &((*record).xyz[i][3]));
    sscanf(line+51, "%c",   &((*record).segid[i][0]));
    sscanf(line+52, "%c",   &((*record).segid[i][1]));
    sscanf(line+53, "%c",   &((*record).segid[i][2]));
    sscanf(line+54, "%c",   &((*record).segid[i][3]));
    (*record).segid[i][4]='\0';
    sscanf(line+56, "%d",  &((*record).res_no2[i]));
    sscanf(line+60, "%lf",&((*record).variable[i]));

}
/*..............................................................*/
/* Procedure reads a CRD_atom from a string taken from a CRD-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/*								*/
/* Syntax: read_CRDatom(line, &record)				*/
/*								*/
void read_CRDatom(CRD_atom *record, int i,char *line)
{
    sscanf(line,"%d %d %s %s %lf %lf %lf %s %d %lf",
           &(*record).atomno[i],&(*record).res_no[i],
           (*record).residue[i], (*record).atomid[i],
           &(*record).xyz[i][1],
           &(*record).xyz[i][2],
           &(*record).xyz[i][3],
           (*record).segid[i],
           &(*record).res_no2[i],&(*record).variable[i]);

}
/*..............................................................*/
/* The procedure prints a CRD-Atom to a file.			*/
/*								*/
/* Syntax: fprint_CRDatom(fptr,&record)				*/
/*								*/
void fprint_CRDatom(FILE *fptr, CRD_atom *record, int i)
{
fprintf(fptr, "%5d %4d %c%c%c%c %c%c%c%c%10.5f%10.5f%10.5f %c%c%c%c %-4d%10.5f\n", 
		(*record).atomno[i],
		(*record).res_no[i],
		(*record).residue[i][0],
		(*record).residue[i][1],
		(*record).residue[i][2],
		(*record).residue[i][3],
		(*record).atomid[i][0],
		(*record).atomid[i][1],
		(*record).atomid[i][2],
		(*record).atomid[i][3],
		(*record).xyz[i][1],
		(*record).xyz[i][2],
		(*record).xyz[i][3],
		(*record).segid[i][0],
		(*record).segid[i][1],
		(*record).segid[i][2],
		(*record).segid[i][3],
		(*record).res_no2[i],
		(*record).variable[i]);
}

/*..............................................................*/
/* allocate an crd-protein 					*/

void crd_alloc(CRD_atom *crd)
{
(*crd).atomid  = charmatrix(1,(*crd).natoms,0,4); 	/*[5][N]*/
(*crd).residue = charmatrix(1,(*crd).natoms,0,4); 	/*[5][N]*/
(*crd).segid   = charmatrix(1,(*crd).natoms,0,4); 	/*[5][N]*/

(*crd).variable	= dvector(1,(*crd).natoms);	   

(*crd).atomno   = ivector(1,(*crd).natoms);   
(*crd).res_no2  = ivector(1,(*crd).natoms);   
(*crd).res_no   = ivector(1,(*crd).natoms);   
(*crd).xyz      = dmatrix(1,(*crd).natoms,1,3);   	/*[4][N]*/
}

/*..............................................................*/
/* Procedure reads a PSF_atom from a string taken from a PSF-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/* The file is read in "free format"				*/
/*								*/
/* Die Namen haben z.T. keinen Sinn.				*/
/*								*/
/* Syntax: read_PSFatom(line, &record)				*/
/*								*/
void read_PSFatom(PSF_atom *record, int i, char *line)
{
sscanf(line,"%d %s %d %s %s %s %lf %lf %d",   
		&((*record).atomno[i]),
		((*record).segid[i]), 
		&((*record).res_no[i]),
		((*record).residue[i]), 
		((*record).atomid[i]),
		((*record).atom_type[i]),  
		&((*record).charge[i]), 
		&((*record).mass[i]), 
		&((*record).res_no2[i]));
}

/*..............................................................*/
/* Procedure reads a PSF_atom from a string taken from a PSF-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/* The file is read in "free format"				*/
/*								*/
/* Die Namen haben z.T. keinen Sinn.				*/
/*								*/
/* Syntax: read_PSFatom(line, &record)				*/
/*								*/
void read_PSFatom_old(PSF_atom *record, int i, char *line)
{
sscanf(line,"%d %s %d %s %s %d %lf %lf %d",   
		&((*record).atomno[i]),
		((*record).segid[i]), 
		&((*record).res_no[i]),
		((*record).residue[i]), 
		((*record).atomid[i]),
		&((*record).number[i]),  
		&((*record).charge[i]), 
		&((*record).mass[i]), 
		&((*record).res_no2[i]));
}

/*..............................................................*/
/* Procedure reads a PSF_atom from a string taken from a PSF-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/* The file is read in "free format"				*/
/*								*/
/* Das Format hier ist nicht im Standard-PSF-Format.		*/
/*								*/
/* Die Namen haben z.T. keinen Sinn.				*/
/*								*/
/* Syntax: read_PSFatom(line, &record)				*/
/*								*/
void fprint_PSFatom_old(FILE *fptr, PSF_atom *record, int i)
{
fprintf(fptr,"%d %s %d %s %s %d %f %f %d\n",   
		(*record).atomno[i],
		(*record).segid[i], 
		(*record).res_no[i],
		(*record).residue[i], 
		(*record).atomid[i], 
		(*record).number[i],  
		(*record).charge[i], 
		(*record).mass[i], 
		(*record).res_no2[i]);
}
/*..............................................................*/
/* Procedure reads a PSF_atom from a string taken from a PSF-	*/
/* file. In the procedure, no care is taken, if the line given 	*/
/* to the procedure is a correct coordinate line. This *must* 	*/
/* have been done in the procedure calling this procedure.	*/
/* The file is read in "free format"				*/
/*								*/
/* Das Format hier ist nicht im Standard-PSF-Format.		*/
/*								*/
/* Die Namen haben z.T. keinen Sinn.				*/
/*								*/
/* Syntax: read_PSFatom(line, &record)				*/
/*								*/
void fprint_PSFatom(FILE *fptr, PSF_atom *record, int i)
{
fprintf(fptr,"%d %s %d %s %s %s %f %f %d\n",   
		(*record).atomno[i],
		(*record).segid[i], 
		(*record).res_no[i],
		(*record).residue[i], 
		(*record).atomid[i], 
		(*record).atom_type[i],  
		(*record).charge[i], 
		(*record).mass[i], 
		(*record).res_no2[i]);
}
/*..............................................................*/
/* allocate an psf-protein 					*/

void psf_alloc(PSF_atom *psf)
{
(*psf).atomid    = charmatrix(1,(*psf).natoms,0,4); 	/*[5][N]*/
(*psf).atom_type = charmatrix(1,(*psf).natoms,0,4); 	/*[5][N]*/
(*psf).residue   = charmatrix(1,(*psf).natoms,0,4); 	/*[5][N]*/
(*psf).segid     = charmatrix(1,(*psf).natoms,0,4); 	/*[5][N]*/

(*psf).charge	 = dvector(1,(*psf).natoms);	   
(*psf).mass	 = dvector(1,(*psf).natoms);	   

(*psf).atomno    = ivector(1,(*psf).natoms);   
(*psf).res_no2   = ivector(1,(*psf).natoms);   
(*psf).res_no    = ivector(1,(*psf).natoms);   
(*psf).number    = ivector(1,(*psf).natoms);   
}
/*..............................................................*/
/* Reading the PSF-File						*/
void read_PSF(char *filename, PSF_atom *psf)
{
   FILE *fppsf;
   char buff[LINELENGTH], help1[LINELENGTH], help2[LINELENGTH];
   int i;
   
   fppsf=file_open(filename,"r");
   while (fgets(buff,LINELENGTH,fppsf)!=NULL)
         {sscanf(buff,"%d %s",&(*psf).natoms, help1);
          if (strcmp(help1,"!NATOM")==0) break;
         }
   fprintf(stderr,"Number of Atoms in PSF: %d\n",(*psf).natoms);
   psf_alloc(psf); /* Allocates not the bond etc. matrices */
   for (i=1; i<=(*psf).natoms; i++)
       {if (NULL==fgets(buff,LINELENGTH,fppsf)) 
	   muerror("Problem in read_PSF() 0");
        read_PSFatom(psf,i,buff);
       }
   /* After one blank line the number of bonds follows */
   while (fgets(buff,LINELENGTH,fppsf)!=NULL)
         {sscanf(buff,"%d %s %s",&(*psf).nbonds, help1, help2);
          if (strcmp(help2,"bonds")==0) break;
         }
   fprintf(stderr,"Number of Bonds in PSF: %d\n",(*psf).nbonds);
   (*psf).bond=imatrix(1,(*psf).nbonds,1,2);
   for (i=1; i<=(*psf).nbonds; i++)
       {if (2!=fscanf(fppsf,"%d %d", 
			 &(*psf).bond[i][1], &(*psf).bond[i][2]))
	   muerror("Problem in read_PSF() reading bonds");
       }
   /* After some blank line the number of angles follows */
   while (fgets(buff,LINELENGTH,fppsf)!=NULL)
         {sscanf(buff,"%d %s %s",&(*psf).nangles, help1, help2);
          if (strcmp(help2,"angles")==0) break;
         }
   fprintf(stderr,"Number of Angles in PSF: %d\n",(*psf).nangles);
   (*psf).angle=imatrix(1,(*psf).nangles,1,3);
   for (i=1; i<=(*psf).nangles; i++)
       {if (3!=fscanf(fppsf,"%d %d %d", &(*psf).angle[i][1], 
		                        &(*psf).angle[i][2], 
		                        &(*psf).angle[i][3]))
	   muerror("Problem in read_PSF() reading angles");
       }
   /* After some blank line the number of dihedrals follows */
   while (fgets(buff,LINELENGTH,fppsf)!=NULL)
         {sscanf(buff,"%d %s %s",&(*psf).ntorsions, help1, help2);
          if (strcmp(help2,"dihedrals")==0) break;
         }
   fprintf(stderr,"Number of Torsions in PSF: %d\n",(*psf).ntorsions);
   (*psf).torsion=imatrix(1,(*psf).ntorsions,1,4);
   for (i=1; i<=(*psf).ntorsions; i++)
       {if (4!=fscanf(fppsf,"%d %d %d %d", &(*psf).torsion[i][1], 
		                           &(*psf).torsion[i][2], 
		                           &(*psf).torsion[i][3], 
		                           &(*psf).torsion[i][4]))
	   muerror("Problem in read_PSF() reading torsion");
       }
   
   /* After some blank line the number of improper torsions follows */
   while (fgets(buff,LINELENGTH,fppsf)!=NULL)
         {sscanf(buff,"%d %s %s",&(*psf).nimprop, help1, help2);
          if (strcmp(help2,"impropers")==0) break;
         }
   fprintf(stderr,"Number of Improper Torsions in PSF: %d\n",(*psf).nimprop);
   (*psf).improp=imatrix(1,(*psf).nimprop,1,4);
   for (i=1; i<=(*psf).nimprop; i++)
       {if (4!=fscanf(fppsf,"%d %d %d %d", &(*psf).improp[i][1], 
                                           &(*psf).improp[i][2], 
                                           &(*psf).improp[i][3], 
		                           &(*psf).improp[i][4]))
	   muerror("Problem in read_PSF() reading improp");
       }

   /* After stuff, read the ngrp */
   while (fgets(buff,LINELENGTH,fppsf)!=NULL)
     {if (1!=sscanf(buff,"%d %*d %s %s",&(*psf).ngrp, help1, help2))
       if (strcmp(help1,"!NGRP")==0) break;
      }
   (*psf).grp = ivector(1,(*psf).ngrp+1);
   for (i=1; i<=(*psf).ngrp; i++)
       {if (1!=fscanf(fppsf,"%d %*d %*d",&(*psf).grp[i]))
	   muerror("Problem in read_PSF() 2");
       }
   (*psf).grp[i]=(*psf).natoms;
   fclose(fppsf);
}
/*..............................................................*/
/* Reading the CRD-Files 					*/
void read_CRD(char *filename, CRD_atom *crd)
{
    FILE *fpcrd;
    char buff[LINELENGTH];
    int i;

    fpcrd=file_open(filename,"r");
    if (NULL==fgets(buff,LINELENGTH,fpcrd))
	   muerror("Problem in read_CRD() 1");
    while (buff[0]=='*') 
       {if (NULL==fgets(buff,LINELENGTH,fpcrd))
	   muerror("Problem in read_CRD() 2");
       }

    if (1!=sscanf(buff,"%d",&(*crd).natoms))
	   muerror("Problem in read_CRD() 3");
    fprintf(stderr,"Number of Atoms in CRD: %d\n",(*crd).natoms);
    crd_alloc(crd);
    for (i=1; i<=(*crd).natoms; i++)
        {if (NULL==fgets(buff,LINELENGTH,fpcrd))
	   muerror("Problem in read_CRD() 4");
	 read_CRDatom(crd,i,buff);
	}
    fclose(fpcrd);
}

/*..............................................................*/
/* Function writes the charges of an PSF-record and the coor-   */
/* dinates of a CRD-record to a Mol-record.                     */
/* The radius is added in an extra procedure.                   */
/* The first SEGID-char is assumed to be the Chain identifier.  */

void charge_vol_assignment(Mol *m, CRD_atom *crd, PSF_atom *psf)
{
int i,len;

for (i=1; i<=(*m).natoms; i++)
    {(*m).identifier[i][0] = 'A';
     (*m).identifier[i][1] = 'T';
     (*m).identifier[i][2] = 'O';
     (*m).identifier[i][3] = 'M';
     (*m).identifier[i][4] = ' ';
     (*m).identifier[i][5] = ' ';
     (*m).atomno[i]        = (*crd).atomno[i];
     (*m).alternate[i]     = ' ';
     (*m).bval[i]          = (*crd).variable[i];
     (*m).occu[i]          = 0.0;
     (*m).atom_type[i]     = (*psf).atom_type[i];

     len = strlen((*crd).residue[i]);
     if (len==0) {ERROR("atomid of zero lenth");}
     else if (len==1)
          {(*m).residue[i][0]=(*crd).residue[i][0];
           (*m).residue[i][1]=' ';
           (*m).residue[i][2]=' ';
          }
     else if (len==2)
          {(*m).residue[i][0]=(*crd).residue[i][0];
           (*m).residue[i][1]=(*crd).residue[i][1];
           (*m).residue[i][2]=' ';
          }
     else if (len==3)
          {(*m).residue[i][0]=(*crd).residue[i][0];
           (*m).residue[i][1]=(*crd).residue[i][1];
           (*m).residue[i][2]=(*crd).residue[i][2];
          }
     else if (len==4)
          {(*m).residue[i][0]=(*crd).residue[i][0];
           (*m).residue[i][1]=(*crd).residue[i][1];
           (*m).residue[i][2]=(*crd).residue[i][2];
           if ((*m).residue[i][3]!=' ')
                    WARN("cat a residue name");
          }
     else ERROR("too long resid");
     (*m).residue[i][3]='\0';

     (*m).chain[i]=(*crd).segid[i][0]; 
     /*(*muf).chain=' ';*/
     /* residue number 2 is the sequence counter and so the one I need here */
     (*m).res_no[i]=(*crd).res_no2[i];
     (*m).insert[i]=' ';
     (*m).xyz[i][1]=(*crd).xyz[i][1];
     (*m).xyz[i][2]=(*crd).xyz[i][2];
     (*m).xyz[i][3]=(*crd).xyz[i][3];
     (*m).charge[i]=(*psf).charge[i];
     /* (*mus).radius[i]=atom_radius(psf,i); */
     /* (*mus).E_min[i] =atom_Emin(psf,i);   */
     (*m).mass[i]  =(*psf).mass[i];
     /*
     (*mus).ion[i]   =assign_ion((*mus).atomid[i], (*mus).residue[i]);
     (*mus).sigma[i] =assign_sigma((*mus).atomid[i], (*mus).ion[i]);
     (*mus).hbond[i] =assign_hbond((*mus).atomid[i], (*mus).residue[i]);
     */

     len = strlen((*crd).atomid[i]);
     if (len==0) {ERROR("atomid of zero lenth");}
     else if (len==1)
         {(*m).atomid[i][0]=' ';
          (*m).atomid[i][1]=(*crd).atomid[i][0];
          (*m).atomid[i][2]=' ';
          (*m).atomid[i][3]=' ';
         }
     else if (len==2)
         {(*m).atomid[i][0]=' ';
          (*m).atomid[i][1]=(*crd).atomid[i][0];
          (*m).atomid[i][2]=(*crd).atomid[i][1];
          (*m).atomid[i][3]=' ';
         }
     else if (len==3)
         {(*m).atomid[i][0]=' ';
          (*m).atomid[i][1]=(*crd).atomid[i][0];
          (*m).atomid[i][2]=(*crd).atomid[i][1];
          (*m).atomid[i][3]=(*crd).atomid[i][2];
         }
     else if (len==4)
         {(*m).atomid[i][0]=(*crd).atomid[i][0];
          (*m).atomid[i][1]=(*crd).atomid[i][1];
          (*m).atomid[i][2]=(*crd).atomid[i][2];
          (*m).atomid[i][3]=(*crd).atomid[i][3];
         }
     else ERROR("too long atomid");
     (*m).atomid[i][4]='\0';
    }
}


/*..............................................................*/
/* proccedure avoids to assign the same chain identifier to two */
/* segments.                                                    */
/*                                                              */
void assign_chains(Mol *mus, CRD_atom *crd)
{
int i, chains=1;
char chain='A';

i=1;
(*mus).chain[i]=chain;
for (i=2; i<=(*mus).natoms; i++)
    {if (0!=strcmp((*crd).segid[i-1],(*crd).segid[i]))
        {chain++; 
         chains++;
         (*mus).chain[i]=chain;
        }
     else {(*mus).chain[i]=chain;
          }
    }
fprintf(stderr,"I have found %d chains\n",chains);
}
/*..............................................................*/
/* Assigning topology and masses                                */ 
/* the treatment of the torsions here is not qiute correct and  */
/* will be corrected in a later subroutine                      */
/*                                                              */
/* This is a very modified version from create-mus              */
/*                                                              */
void psfcrd2mol(Mol *m, PSF_atom *psf, CRD_atom *crd)
{
    int i;

 (*m).natoms    = (*psf).natoms;
 (*m).nbonds    = (*psf).nbonds;
 (*m).nangles   = (*psf).nangles;
 (*m).nimprop   = (*psf).nimprop;
 (*m).ntorsions = (*psf).ntorsions;

 Mol_alloc(m);
 charge_vol_assignment(m,crd,psf);

 for (i=1; i<=(*psf).nbonds; i++)
     {(*m).bond[i][1]=(*psf).bond[i][1];
      (*m).bond[i][2]=(*psf).bond[i][2];
     }
 for (i=1; i<=(*psf).nangles; i++)
     {(*m).angle[i][1]=(*psf).angle[i][1];
      (*m).angle[i][2]=(*psf).angle[i][2];
      (*m).angle[i][3]=(*psf).angle[i][3];
     }
 for (i=1; i<=(*psf).nimprop; i++)
     {(*m).improp[i][1]=(*psf).improp[i][1];
      (*m).improp[i][2]=(*psf).improp[i][2];
      (*m).improp[i][3]=(*psf).improp[i][3];
      (*m).improp[i][4]=(*psf).improp[i][4];
     }
 for (i=1; i<=(*psf).ntorsions; i++)
     {(*m).torsion[i][1]=(*psf).torsion[i][1];
      (*m).torsion[i][2]=(*psf).torsion[i][2];
      (*m).torsion[i][3]=(*psf).torsion[i][3];
      (*m).torsion[i][4]=(*psf).torsion[i][4];
     }
}

