/* Matthias Ullmann, 2.12.1994				*/
/* Header to read atoms from CDR-files.			*/
/*							*/
/*......................................................*/


/* declaration of structures */
#ifndef CRD_ATOM
#define CRD_ATOM

#include "mol.h"

typedef struct {int  	natoms; 	/* N */
		int 	*atomno;
		int 	*res_no;  
	        char 	**residue;	    
	        char 	**atomid; 	  
	        double 	**xyz;		  
		char 	**segid;
		int 	*res_no2;
		double 	*variable;
	       } CRD_atom;
#endif

#ifndef PSF_ATOM
#define PSF_ATOM
typedef struct {int  	natoms; 	/* N */
		int	nbonds;
		int 	nangles;
		int	nimprop;
		int	ntorsions;
		int 	*atomno;
		char 	**segid;
		int 	*res_no;  
	        char 	**residue;	    
	        char 	**atomid; 	  
		int 	*number;
		char	**atom_type;
	        double 	*charge;		  
	        double 	*mass;		  
		int 	*res_no2;
		int	**bond;
		int 	**angle;
		int	**improp;
		int	**torsion;
		int	ngrp;
		int	*grp;
	       } PSF_atom;
#endif

#ifndef MASS
#define MASS
typedef struct {int	natomtypes;
	        int	*number;
		char	**atom_type;
		double	*mass;
		char	**element;
	       } MASS_record;
#endif


#ifndef PARAM
#define PARAM
/* Key words */
#define BONDS     "BOND"
#define THETA     "THETA"
#define PHI       "PHI"
#define IMPHI     "IMPHI"
#define NBONDED   "NBONDED"
#define NBFIX     "NBFIX"
#define HBOND     "HBOND"
#define END	  "END"
#define WILDCARD  "X"
#define WILDCHAR  '*'
typedef struct {int	nbonds;
		char	**bond1;
		char    **bond2;
		double	*K_bond;
		double	*bond_0;

		int	nangles;
		char	**angle1;
		char	**angle2;
		char	**angle3;
		double	*K_angle;
		double	*angle_0;
		double	*K_ub;
		double	*dist_ub_0;

		int	ntorsions;
		char	**torsion1;
		char	**torsion2;
		char	**torsion3;
		char	**torsion4;
		double	*K_torsion;
		double	*delta;
		int	*n;

		int	nimprops;
		char	**improp1;
		char	**improp2;
		char	**improp3;
		char	**improp4;
		double	*K_improp;
		double	*improp_0;		

		int	nnonbonded;
		char	**atom;
		double	*polarizability; /* not used */
		double	*Emin;
		double	*Rmin;
		double	*polarizability1_4; /* not used */
		double	*Emin1_4;
		double	*Rmin1_4;

		int	nnbfix;
		char	**atom1;
		char	**atom2;
		double  *EminNBfix;
		double  *RminNBfix;
		
		int     nhbond; /* not used */
		char	**hatom1;
		char	**hatom2;
		double  *para1;		
		double  *para2;		
		
	       } PARAM_record;
#endif

void read_MASS(char *filename, MASS_record *mass);

void fprint_BONDS(FILE *fpout, PARAM_record *param);
void fprint_THETA(FILE *fpout, PARAM_record *param);
void fprint_PHI(FILE *fpout, PARAM_record *param);
void fprint_IMPHI(FILE *fpout, PARAM_record *param);
void fprint_NBONDED(FILE *fpout, PARAM_record *param);
void fprint_NBFIX(FILE *fpout, PARAM_record *param);

void read_PARAM(char *filename, PARAM_record *param);
void count_records(FILE *fpparam, PARAM_record *param);
void fprint_PARAM(FILE *fpout, PARAM_record *param);
void fprint_two_PARAM(FILE *fpout, PARAM_record *param1, PARAM_record *param2);
void allocate_PARAM(PARAM_record *param);
void read_records(FILE *fpparam,PARAM_record *param);

void read_CRD(char *filename, CRD_atom *crd);
void read_CRDatom(CRD_atom *record, int i, char *line);
void fprint_CRDatom(FILE *fptr, CRD_atom *record, int i);
void crd_alloc(CRD_atom *crd);

void read_PSF(char *filename, PSF_atom *psf);
void read_PSFatom(PSF_atom *record, int i, char *line);
void fprint_PSFatom(FILE *fptr, PSF_atom *record, int i);
void psf_alloc(PSF_atom *psf);

void psfcrd2mol(Mol *m, PSF_atom *psf, CRD_atom *crd);
void assign_chains(Mol *mus, CRD_atom *crd);
