/* Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* .....................................................*/

#ifndef __energy__
#define __energy__


#include "mol.h"
#include "nb-list.h"

#ifndef NB_PARA
#define NB_PARA
typedef struct {int types;
		char **atom_type;
		double *Emin;
		double *radius;
		double *Emin1_4;
		double *radius1_4;
		double **Eab1_4;
		double **sigma1_4;
		double **Eab;
		double **sigma;
		double eps;		/* epsilon_relative		*/
		double inv_eps;		/* 1.0/epsilon_relative		*/
		double fact1_4;		/* weighting factor for 1-4 	*/
					/* electrostatic interaction 	*/
		double xon;		/* xon of the switch function 	*/
		double xoff;		/* xoff of the switch function  */
		double r_too_small;	/* if the distance is less than */
					/* r_too_small, the calcluation */
					/* will be aborted		*/
	       }NB_Para;
/* the number of the type is the running number i of atom_types[i] */
#endif

/******************************************************************/
/* definitions for the topology matrix 				  */
#define WRONG	 255			/* No value assigned	  */
#define NON 	   0			/* nonbonded interaction  */
#define TORSION	   1			/* torsion interaction	  */
#define BOND	   2			/* bond interaction	  */
#define ANGLE	   3			/* bond angle interaction */
#define SELF	   4			/* in fields [i][i]	  */
					/* only in char matrix!!!!*/

/****************************************************************/
#define MAX_ATOMTYPES 1000 /* Maximum Number of atom types */
/****************************************************************/
/* Definition of Energy components				*/
/*								*/
#define E_LENGTH 12  /* Range [1..E_LENGTH] */
#define E_BOND    1
#define E_ANGL    2
#define E_TORS    3
#define E_IMPR    4
#define E_LEJO	  5
#define E_COUL    6
#define E_SOLV    7
#define E_POTN	  8
#define E_KINT	  9
#define E_HBON	 10
#define E_USER	 11
#define E_TOTL   12
/****************************************************************/
/* also the SWITCH function is implemented as a Makro in order  */
/* to speed up the procedure					*/
/* The energy must multiplied with 'back' to use the switch	*/
/* function 							*/
#define Switch(back,x,xon,xoff) \
        {if (x <= xon) {back=1.0;}\
	 else {if (x > xoff) {back=0.0;}\
	       else {back=((xoff-x)*(xoff-x)*(xoff+2.0*x-3.0*xon))/\
		          ((xoff-xon)*(xoff-xon)*(xoff-xon));\
		    }\
	      }\
        }


/****************************************************************/
/* bonded energy term						*/
/* The bonded energy terms are defined via Makros as well as via*/
/* functions. Makros are faster!				*/
/*..............................................................*/
/* Makros calculating the bonded energy in CHARMm-energy 	*/
/* function							*/
#define BondEnergy(length, K_bond, bond_0)\
	((K_bond)*(((length)-(bond_0))*((length)-(bond_0))))

#define AngleEnergy(angle, K_angle, angle_0)\
	((K_angle)*(((angle)-(angle_0))*((angle)-(angle_0))))

#define TorsionEnergy(torsion,K_torsion,delta,n)\
	((K_torsion)*(1.0+cos(((n)*(torsion))-(delta))))

#define ImpropEnergy(improp, K_improp, improp_0)\
	((K_improp)*(((improp)-(improp_0))*((improp)-(improp_0))))

/****************************************************************/
/* Makros for calculating the nonbonded interactions		*/
/* ATTENTION - the dielectric constant is defined as distance 	*/
/* dipendent - it should be changed here			*/
/*								*/
/* distance dependent dieelctric constant */
/*#define CoulombEnergy(q1, q2,r) 	((q1)*(q2)/((r)*(r)))*/

/* correct Coulomb law */
#define CoulombEnergy(q1, q2,r) 	((q1)*(q2)/(r))
#define LennardJonesEnergy(elj,e,s,r)	{elj=(s)/(r);\
	 	 			 elj=elj*elj;\
	 	 			 elj=elj*elj*elj;\
	 	 			 elj=(e)*((elj*elj)-elj);\
					}		


/****************************************************************/
/* Topology Matrix						*/
char **topology(Mol *mus);

/****************************************************************/
/* Functions - not used!!					*/
double solvation_energy(double *sigma, double *area, int natom);

double bond_energy(double bond_length, double K_bond, double bond_0);
double angle_energy(double bond_angle, double K_angle, double angle_0);
double torsion_energy(double torsion_angle, double K_torsion, double delta, int n);
double improp_energy(double torsion_improp, double K_improp, double improp_0);

double lennard_jones_energy(double e,double s,double r);	
double switchfkt(double x, double xon, double xoff);

/****************************************************************/
double coulomb(int no_atoms, double **xyz, double *charge, double eps);
/****************************************************************/
void assign_typeno(Mol *mus, NB_Para *nb_para);
void LJ_ia_matrix(Mol *mus, NB_Para *nb_para);

double energy_MUS1(double *energy, Mol *mus, NB_Para *nb, char **topo);



/****************************************************************/
/* Functions for the energy calculations using a grid-cell based*/
/* non-bonded list.						*/
/*								*/
int energy_MUS3(double *energy, Mol *mus, NB_Para *nb, 
		   char **topo, iListGrid *grid);

int ia_atom_box(double *ener_LeJo, double *elect, int i,
	        double **x, double *q, int *type, NB_Para *nb, 
		char **topo, iList *list);
int calculate_interaction(double *ener_LJ, double *elect, int n, 
			    double **x, double *q, int *type, NB_Para *nb, 
			    char **topo, iListGrid *grid);

/****************************************************************/
int ia_energy(double *elect, double *lejo_ener, 
	      double **x1, double *charge1, int *type1, int atoms1,
	      double **x2, double *charge2, int *type2, int atoms2,
	      NB_Para *nb,
	      char *flexible, double d2, int *aa1, int *aa2);
/****************************************************************/
void fprint_energy(FILE *fpout, double *energy);
void energy_test(double *energy, char **topo, Mol *mus, NB_Para *nb,
	         iListGrid *grid, iList *list);


#endif  /* __energy__ */
