#include <stdio.h>
#include <stdlib.h>
#include "graph.h"
#include "util.h"
#include "memutil.h"

/*========================================================*/
/* Queue Operations                                       */
/*........................................................*/
void  QueueInit(Queue *q)
{

  (*q).n = 0;

  (*q).front = NULL;
  (*q).rear  = NULL;

}

/*........................................................*/
/* Put Node index at the end of the Queue                 */
/*                                                        */

void PutInQueue(int v, Queue *q)
{
    Node *t;

    (*q).n++; 

    AllocNode(t);
    (*t).v    = v; 
    (*t).next = NULL;

    if (((*q).front==NULL) && ((*q).front==NULL))
       {(*q).front = (*q).rear  = t;
       }
    else {(*q).rear->next = t;
          (*q).rear = t;
         }

}

/*........................................................*/
/* Get and Remove First Node from the beginning of        */
/* the Queue                                              */
/*                                                        */

int  GetFromQueue(Queue *q)
{
    int v;
    Node *t;

    if ((*q).front == NULL) 
       {muwarn("Queue is empty!?");
	return -1;
       }
    (*q).n--; 
    v = (*q).front->v;
    t = (*q).front;

    if   ((*q).front == (*q).rear) {(*q).front = (*q).rear = NULL;}
    else                           {(*q).front = (*q).front->next;}

    free(t);

    return v;
}
/*........................................................*/
void PrintQueue(Queue *que) 
{
  Node *temp;

  temp = (*que).front;

  printf("In Queue:  ");
  while(temp != NULL) 
       {
	 printf("%d ",(*temp).v);
	temp = (*temp).next;
       }
  printf("\n");
}
/*========================================================*/
/* Stack Operations                                       */
/*........................................................*/
/* pushes Node v to the stack                             */
/*                                                        */
void push(int v, Node *stack)
{
    Node *t;

    AllocNode(t);
    (*t).v    = v; 
    (*t).next = (*stack).next;
    (*stack).next = t;
}

/*........................................................*/
/* pops top Node from the stack and returns its index     */
/*                                                        */
int pop(Node *stack)
{
    int x;
    Node *t;

    t             = (*stack).next;
    (*stack).next = (*t).next; 
    x = (*t).v;
    free(t);
    return x;
}


/*========================================================*/
/* Adjactent List                                         */
/*........................................................*/
/* allocate a Node vector with subscript range v[nl..nh]  */
/* and initialize with NULL                               */
/*                                                        */
Node *NodeVector(long nl, long nh)
{
   Node *v;
   int i;

   v=(Node *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(Node)));
   if (!v) memerror("allocation failure in NodeVector()");
   for (i=nl; i<=nh; i++) v[i].next=NULL;
  
   return v-nl+NR_END;
}

/*......................................................*/
/* free a char vector with subscript range v[nl..nh]    */
void free_NodeVector(Node *v, long nl, long nh)
{
  if (nl>nh) fprintf(stderr,"Problem freeing NodeVector");

if (v!=NULL)    free((FREE_ARG) (v+nl-NR_END));
}
/*........................................................*/
/* Print Adjectent List                                   */
/*                                                        */
void PrintAdjList(FILE *fpout, Node *adj, int n)
{
  int i;
  Node *t;

  for (i=1; i<=n; i++)
    {
      fprintf(fpout,"Node %3d: ", i);
      t = adj[i].next;
      while (t != NULL)
	{fprintf(fpout," %3d ",(*t).v);
	  t = (*t).next;
	}
     fprintf(fpout,"\n");
    }

}

/*........................................................*/
/* Visit function for Depth-first search - recursive      */
/*                                                        */
void r_visit_DFS(Node *adj, int *val, int V, int clust, int m)
{
   Node *t;

   if (m>V) muerror("Problem m>V in visit()");
   val[m] = clust;

   for (t = adj[m].next; t != NULL; t = t->next)
	{if (val[(*t).v] == 0) 
	      {r_visit_DFS(adj, val, V, clust, t->v);
	      }
	}
}

/*........................................................*/
/* Visit function for Depth-first search - non-recursive  */
/*                                                        */
void nr_visit_DFS(Node *stack, Node *adj, int *val, int V, int clust, int m)
{
   Node *t;

   if (m>V) muerror("Problem m>V in visit()");

   push(m, stack);
   while ((*stack).next!=NULL)
     {
      m = pop(stack);
      val[m] = clust;
      for (t = adj[m].next; t != NULL; t = t->next)
	  {if (val[(*t).v] == 0) 
	      {push((*t).v, stack);
	       val[(*t).v]=-1;
	      }
	  }
     }

}

/*........................................................*/
/* Depth-first search                                     */
/*                                                        */
/* recusive -- boolean if the search should be done       */
/*             recursively (TRUE=1) or not (FALSE=0)      */
/*                                                        */
int listDFS(int *val, Node *adj, int V, int recursive)
{
  int k, cluster=0;
  Node *stack;

  val = ivector(1,V);

  /* initialize stack */
  AllocNode(stack);
  (*stack).v = 0; (*stack).next = NULL; 
    
  for (k=1; k<=V; k++) val[k] = 0;
  for (k=1; k<=V; k++) 
      {if (val[k] == 0) 
	  {cluster++; 
	    if (recursive==1)
	       r_visit_DFS(adj, val, V, cluster, k);
            else if (recursive==0)
	       nr_visit_DFS(stack, adj, val, V, cluster, k);
            else muerror("unvalid recursive variable in listDFS()");
	  }
      }
  free(stack);

  return cluster;
}


/*........................................................*/
/* Visit function for Breadth-first search                */
/*                                                        */
void visitBFS(Queue *que, Node *adj, int *val, int V, int clust, int m)
{
   Node *t;

   if (m>V) muerror("Problem m>V in visit()");

   PutInQueue(m, que);
   while ((*que).front!=NULL)
      {m = GetFromQueue(que);
       val[m] = clust;
       for (t = adj[m].next; t != NULL; t = t->next)
	   {if (val[(*t).v] == 0) 
	       {PutInQueue((*t).v, que);
	        val[(*t).v]=-1;
	       }
	   }
     }
}

/*........................................................*/
/* Depth-first search -recursive                          */
/*                                                        */
int listBFS(int *val, Node *adj, int V)
{
  int k, cluster=0;
  Queue q;

  QueueInit(&q);
  for (k=1; k<=V; k++) val[k] = 0;
  for (k=1; k<=V; k++) 
      {if (val[k] == 0) 
	  {cluster++;
           visitBFS(&q, adj, val, V, cluster, k);
	  }
      }

  if (q.front!=NULL) muerror("Queue not empty!");

  return cluster;
}

/*========================================================*/
/*........................................................*/
/* breath-first search to determine connectivity          */
/*                                                        */
/* procedure does a recursive breath search through an    */
/* adjacency list. The procedure marks in the vector      */
/* clst to which cluster the node belongs. The procedure  */
/* returns the number of clusters found.                  */
/*                                                        */
/* Return: Number of clusters                             */
/* Output:                                                */
/*   - *clust -- vector containing cluster assignment     */
/*           (was allocated [1..V] outside the procedure) */
/* Input:                                                 */
/*   - *adj --  Adjacency List [1..V]                     */
/*                                                        */
int ConnectivityListBTS(int *clust, Node *adj, int V)
{
  int k, ncluster=0;
  Queue q;

  QueueInit(&q);
  for (k=1; k<=V; k++) clust[k] = 0;
  for (k=1; k<=V; k++) 
      {if (clust[k] == 0) 
	  {ncluster++;
           visitBFS(&q, adj, clust, V, ncluster, k);
	  }
      }

  if (q.front!=NULL) muerror("Queue not empty!");

  return ncluster;

}


/*========================================================*/
/*........................................................*/
/* recursive visit function for MatrixDFS                 */
/*                                                        */
void visitDFS(int **adj, int *val, int V,  int clust, int m)
{
   int t;

   if (m>V) muerror("Problem m>V in visit()");
   val[m] = clust;

   for (t = 1; t <= V; t++)
       if (adj[m][t] != 0) 
	     {if (val[t] == 0)
	         {visitDFS(adj, val, V, clust, t);
	         }
	     }
}
/*........................................................*/
/* Depth-first search                                     */
/* using an Adjacency Matrix                              */
/*                                                        */
/* procedure does a recursive depth search through an     */
/* adjacency matrix. The procedure marks in the vector    */
/* clst to which cluster the node belongs. The procedure  */
/* returns the number of clusters found.                  */
/*                                                        */
/* Return: Number of clusters                             */
/* Output:                                                */
/*   - *clust -- vector containing cluster assignment     */
/*           (was allocated [1..V] outside the procedure) */
/* Input:                                                 */
/*   - **adj --  Adjacency Matrix [1..V][1..V]            */
/*                                                        */
int ConnectivityMatrixDFS(int *clst, int **adj, int V)
{

  int k, ncluster=0;

  for (k=1; k<=V; k++) clst[k] = 0;

  for (k=1; k<=V; k++) 
      if (clst[k] == 0) 
	 {ncluster++;
	  visitDFS(adj, clst, V, ncluster, k); 
	 }

  return ncluster;
}
