/* Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* .....................................................*/

/* dGrid = float-grid					*/
/* for mapping spatial properties on a grid		*/
#ifndef __grid__
#define __grid__

#ifndef FLOAT_GRID
/*
#define REAL 		float
#define REAL3TENSOR     f3tensor
#define REAL4TENSOR     f4tensor
*/
#define REAL 		double
#define REAL3TENSOR     d3tensor
#define REAL4TENSOR     d4tensor
typedef struct {REAL  ****grid;         /* lattice      	*/
		int    imin[5];	 	/* extension    	*/
		int    imax[5];   	/* extension 	  	*/
		double spacing;   	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }d4Grid;

typedef struct {REAL  ***grid;    	/* lattice      	*/
		int    imin[4];	  	/* extension    	*/
		int    imax[4];   	/* extension 	  	*/
		double spacing;   	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }dGrid;
#define FLOAT_GRID
#endif

/* iGrid = integer-grid 				*/
/* for instance for exclusion grids			*/
#ifndef INT_GRID
typedef struct {int    ***grid;   	/* lattice      	*/
		int    imin[4];	  	/* extension    	*/
		int    imax[4];  	/* extension 	  	*/
		double spacing;        	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }iGrid;
#define INT_GRID
#endif

/* cGrid = char-grid 					*/
/* for instance for exclusion grids			*/
#ifndef CHAR_GRID
typedef struct {char   ***grid;   	/* lattice      	*/
		int    imin[4];	  	/* extension    	*/
		int    imax[4];  	/* extension 	  	*/
		double spacing;        	/* grid spacing 	*/
		double inv_spacing;   	/* inverse grid spacing */
	       }cGrid;
#define CHAR_GRID
#endif

typedef struct {double ori[4];      /* origin */
                double delta[4][4]; /* axis and spacing*/
                int    npt[4];      /* grid points */
                double *data; 
	       }OpenDX;

typedef struct {double ori[4];      /* origin */
                double delta[4][4]; /* axis and spacing*/
                int    npt[4];      /* grid points */
                double ***data; 
	       }Cube;
/*------------------------------------------------------*/
/*	Functions					*/
/*------------------------------------------------------*/
/* IO-Routines						*/
void fprint_field_binary(FILE *fpout,dGrid *phi);
void fread_field_binary(FILE *fpin, dGrid *phi);
void fwrite_iGrid(FILE *fpout,iGrid *excl);
void fread_iGrid(FILE *fpin, iGrid *excl);


/* Grid initialization					*/
void Init_dGrid(dGrid *phi, double value);
void Init_iGrid(iGrid *phi, int value);

void Multiply_dGrid(dGrid *phi, double value);

/* Infos about grid parameters				*/
void dGrid_info(FILE *fpout, dGrid  *phi,  char *info);
void d4Grid_info(FILE *fpout,d4Grid *phi,  char *info);
void iGrid_info(FILE *fpout, iGrid  *excl, char *info);
void cGrid_info(FILE *fpout, cGrid  *excl, char *info);

/* Utilities						*/
double interpolate(dGrid *phi, double *xyz, int *warn);
double dist(int i, int j, int k, double spacing, double *co);
void min_max_coord(double *min, double *max, double **xyz, int atoms);
void copy_grid(dGrid *phi2, dGrid *phi);

/* Odding		*/
void odding_up(int *ivec, int len);
void odding_down(int *ivec, int len);

/* some OpenDX utilities */
void ReadDX(OpenDX *dx, FILE *fp);
int  DiffDX(OpenDX *ddx, OpenDX *dx1, OpenDX *dx2);
void WriteDX(FILE *fp, OpenDX *dx, char const *info);
void gzReadDX(OpenDX *dx, char *fname);
void gzWriteDX(char *fname, OpenDX *dx, char const *info);

void DX2Grid(dGrid *pot, OpenDX *dx);
void dGrd2DX(OpenDX *dx, dGrid *dens);
void iGrd2DX(OpenDX *dx, iGrid *dens);

void igrd2dx(FILE *fp, iGrid *dens);
void dens2dx(FILE *fp, iGrid *dens, long count);
void Write_dGridToDX(FILE *fp, dGrid *dens);
void Write_iGridToDX(FILE *fp, iGrid *dens);
void gzWrite_dGridToDX(char *fname, char *comment, dGrid *dens);
void gzWrite_iGridToDX(char *fname, char *comment, iGrid *dens);
void igrd2dx(FILE *fp, iGrid *dens);

/* some Gaussian cube utilities */
void ReadCube(Cube *dx, FILE *fp);
void WriteCube(FILE *fp, Cube *dx, const char *info);
int DiffCube(Cube *ddx, Cube *dx1, Cube *dx2);

#endif /* __grid__ */
