/* Matthias Ullmann, Berlin 10.10.1994					*/
/* some Makros and routines for calculations.				*/
/*									*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "util.h"
#include "memutil.h"
#include "constants.h"
#include "mathematik.h"

#define R   	1
#define PHI 	2
#define THETA	3

#define X	1
#define Y	2
#define Z	3


/*............................................................. */
/* Binomial coefficient is calculated and returned		*/
/*								*/
/* it returns a double and is therefore not precise. It has	*/
/* however the advange that has a larger value range		*/ 
/*								*/
/* I try to increase the value range by some tricks. This 	*/
/* increases also the precisson of the result.			*/
/*								*/

double binomial_coefficient3(int n, int k)
{
int i;
double res,  a=1.0, b=1.0;

if ( (n<k) || (n<0) || (k<0) )
   {ERROR("Error in binomial_coefficient(): (n<k) || (n<0) || (k<0)");
   }

if (n==k)
   {res=1.0;
   }
else {if ( (n-(k-1)) < k)
         {for (i=(k+1); i<=n; i++)
              {if ( (DBL_MAX/(double)i)>a)
	          {a=a*(double)i;
	          }
               else {WARN("Error in binomial_coefficient3(). number runs out of range");
	             a=-1.0; break;
	            }
              }
          b=factorial(n-k);
	  res=a/b;
         }
      else {for (i=(n-(k-1)); i<=n; i++)
                {if ( (DBL_MAX/(double)i)>a)
	            {a=a*(double)i;;
	            }
                 else {WARN("Error in binomial_coefficient3(). number runs out of range");
	               a=-1.0; break;
	              }
		}
            b=factorial(k);
            res=a/b;
           }
     }


if ( (a==-1.0) || (b==-1.0) ) {res=-1.0;}


return res;
}


/*............................................................. */
/* Binomial coefficient is calculated and returned		*/
/*								*/
/* it returns a double and is therefore not precise. It has	*/
/* however the advange that has a larger value range		*/ 
/*								*/
/* I try to increase the value range by some tricks		*/
/*								*/

double binomial_coefficient2(int n, int k)
{
int i;
double a, b;

if ( (n<k) || (n<0) || (k<0) )
   {ERROR("Error in binomial2(): (n<k) || (n<0) || (k<0)");
   }

a=1.0;
for (i=(n-(k-1)); i<=n; i++)
    {if ( (DBL_MAX/(double)i)>a)
	{a=a*(double)i;;
	}
     else {WARN("Error in binomial_coefficient2(). number runs out of range");
	   a=-1.0;
	   break;
	  }
    }


b=factorial(k);

if ( (a==-1.0) || (b==-1.0) )
   {return -1.0;
   }
else {return (a/b);
     }
}


/*............................................................. */
/* Binomial coefficient is calculated and returned		*/
/*								*/
/*								*/

double binomial_coefficient1(int n, int k)
{
if ( (n<k) || (n<0) || (k<0) )
   {ERROR("Error in binomial_coefficient1(): (n<k) || (n<0) || (k<0)");
   }

return (double)(factorial(n)/(factorial(k)*factorial(n-k)));
}


/*............................................................. */
/* calculates the factorial (n!) of n.				*/
/* 								*/
/* it returns a double and is therefore not precise. It has	*/
/* however the advange that has a larger value range		*/
/*								*/
/* It returns -1.0 if the number runs out of the range		*/ 
/* 								*/

double factorial(int n)
{
int i;
double dres;

if (n<0) 
   {ERROR("Error in factorial(). n<0");
   }

dres=1;
for (i=1; i<=n; i++)
    {if ( (DBL_MAX/(double)i)>dres)
	{dres=dres*(double)i;
	}
     else {WARN("Error in factorial(). number runs out of range");
	   dres=-1.0;
	   break;
	  }
    }

return dres;
}

/*............................................................. */
/* procedure caluclates the x^y and return a long integer	*/
/*								*/
long ipow(int x, int y)
{
long ihelp;
int i;

ihelp=1;
for (i=1; i<=y; i++)
    {ihelp= ihelp*x;
    }
return ihelp;
}

/*............................................................. */
/* routine transforms polar into cartesian coordinates in 3D	*/
/* polar coordinates: (r,phi,theta)				*/
/*								*/
void polar2cartesian(double *c, double *p)
{
c[X]=p[R]*sin(p[THETA])*cos(p[PHI]);
c[Y]=p[R]*sin(p[THETA])*sin(p[PHI]);
c[Z]=p[R]*cos(p[THETA]);
}
/*............................................................. */
/* routine transforms polar into cartesian coordinates in 3D	*/
/* polar coordinates: (r,phi,theta)				*/
/*								*/
void cartesian2polar(double *p, double *c)
{
#define QUAD(qua,cos_a,sin_a)   \
         {if ((cos_a >= 0.0) && (sin_a >= 0.0)) {qua=1;}\
     else if ((cos_a <= 0.0) && (sin_a >= 0.0)) {qua=2;}\
     else if ((cos_a <= 0.0) && (sin_a <= 0.0)) {qua=3;}\
     else if ((cos_a >= 0.0) && (sin_a <= 0.0)) {qua=4;}\
         }

double cos_phi, sin_phi;
double cos_theta, sin_theta;
double r, rh;
int quadrant_theta=-1, quadrant_phi=-1;

rh = sqrt(c[X]*c[X] + c[Y]*c[Y]);
r  = sqrt(c[X]*c[X] + c[Y]*c[Y] + c[Z]*c[Z]);

p[R]    = r;
p[PHI]  = atan(c[Y]/c[X]);
p[THETA]= atan(rh  /c[Z]);

/* determine the quadrant of the angles */
cos_phi   = c[X]/rh;
sin_phi   = c[Y]/rh;
cos_theta = c[Z]/r;
sin_theta = rh/r;

QUAD(quadrant_theta, cos_theta, sin_theta);
QUAD(quadrant_phi,   cos_phi,   sin_phi);

if ((quadrant_phi==2)   ||( quadrant_phi==3)  ) {p[PHI]=M_PI+p[PHI];}
if ((quadrant_theta==2) || (quadrant_theta==3)) {p[THETA]=M_PI+p[THETA];}

#undef QUAD
}
/*............................................................. */
/* Function calculaties the power of integers			*/
/* power_int=X^Y						*/
/* CHANGE: can also be handled with unsigned long		*/
/*								*/

int power_int(int x, int y)
{
int i,z=1;

if ((y==0) || (x==0)) return 0;

if ((x<0) || (y<0)) {ERROR("Can not handle (x<0) || (y<0) in power_int()");}

for (i=1; i<=y; i++)
    {if (x!=0)
        {if (INT_MAX/x < z) {ERROR("(INT_MAX/x < z) in power_int()");}
	}
     z=z*x;
    }

return z;
}
/*......................................................................*/
/* The real value given to the function becomes sqared.			*/
/* Syntax: sqr(numeral)							*/
/*									*/
double sqr(double numeral)
{
return (numeral*numeral);
}


/*............................................................. */
/* CHANGE - optimieren und verstehen!				*/
/* Function returns the bond angle in rad.			*/
double bond_angle(double *atom1, double *atom2, double *atom3)
{

double vec1[4], vec2[4], vec3[4];
double angle, length1, length2, length3;
double sinus, cosinus;

SubVector(atom2, atom1, vec1);
length1 = sqrt(ScalarProduct(vec1, vec1));

SubVector(atom2, atom3, vec2);
length2 = sqrt(ScalarProduct(vec2, vec2));

VectorProduct(vec1, vec2, vec3);
length3 = sqrt(ScalarProduct(vec3, vec3));

if ((length1*length2)!=0.0)
   {cosinus = ScalarProduct(vec1, vec2)/(length1*length2);
    sinus   = length3/(length1*length2);
    angle = acos(cosinus);
    if(sinus<=0) angle -= M_PI;
   }
else {WARN("(length1*length2)==0.0  in bond_angle(). Set angle to zero.");
      angle=0.0;
     }

return(angle);
}
/*............................................................. */
/* The function calculates the dihedral angle between the	*/
/* atoms given to the procedure. The sequence in the parameter	*/
/* is equal to the sequence in the defintion of torsion angles.	*/
/*								*/
/* It returns the dihedral angle in rad.			*/
double dihedral(double *atom1,double *atom2,double *atom3,
			double *atom4)
{
/* variables */
double dihe, vec1[4], vec2[4], vec3[4],
       norm1[4], norm2[4], norm3[4], 
       inv_length,argum;

/* code */
SubVector(atom2, atom1, vec1);
SubVector(atom3,atom2, vec2);
VectorProduct(vec1, vec2, norm1);

if (rint(100000*LENGTH(norm1))==0) 
   {WARN("LENGTH(norm1)==0.0 in dihedral");
    return 0.0;
   }

if (ScalarProduct(norm1, norm1)==0.0) 
   {WARN("ScalarProduct is zero in dihedral 1: mathematik.c\n");
    inv_length = 0.0;
   }
else {inv_length = 1.0/sqrt(ScalarProduct(norm1, norm1));
     }

ScaleVector(norm1, inv_length, norm1);

SubVector(atom4, atom3, vec3);
VectorProduct(vec2, vec3, norm2);

if (rint(100000*LENGTH(norm2))==0) 
   {WARN("LENGTH(norm2)==0.0 in dihedral");
    return 0.0;
   }

if (ScalarProduct(norm2, norm2)==0.0) 
   {WARN("ScalarProduct is zero in dihedral 2: mathematik.c\n");
    inv_length = 0.0;
   }
else {inv_length = 1./sqrt(ScalarProduct(norm2, norm2));
     }

ScaleVector(norm2, inv_length, norm2);

argum=ScalarProduct(norm1, norm2);
/* to avoid a domain error in acos */
if ((1.0-fabs(argum))<0.0000000000000000000000001)
   {/*fprintf(stderr,"%lf\n",argum);*/
    if ((1.0-argum)<0.0000000000000000000000001) /* i.e. argum == 1.0 */
       dihe=0.0;
    else dihe=M_PI;
   }
else {dihe= acos(argum) ;
      VectorProduct(norm1, norm2, norm3);
      if(ScalarProduct(norm3, vec2)<0.0)
         dihe *=(-1);
     }
return(dihe);
}
/*............................................................. */
/* The function calculates the angle between two planes each	*/
/* spaned by three atoms.					*/
/*								*/
/* No sign is given!						*/
/* atom(i) are 3D-Vectors in NR-Convention			*/
/*								*/

double plane_angle(double *plane1_1,double *plane1_2,double *plane1_3,
		   double *plane2_1,double *plane2_2,double *plane2_3)
{
/* variables */
double angle, a12[4], a23[4], b12[4], b23[4],
       cross1[4], cross2[4],  
       inv_length;

/* code */
SubVector(plane1_2, plane1_1, a12);
SubVector(plane1_3, plane1_2, a23);

SubVector(plane2_2, plane2_1, b12);
SubVector(plane2_3, plane2_3, b23);

VectorProduct(a12, a23, cross1);
VectorProduct(b12, b23, cross2);

if (ScalarProduct(cross1, cross1)!=0.0)
   {inv_length = 1.0/sqrt(ScalarProduct(cross1, cross1));
    ScaleVector(cross1, inv_length, cross1);
   }

if (ScalarProduct(cross2, cross2)!=0.0)
   {inv_length = 1.0/sqrt(ScalarProduct(cross2, cross2));
    ScaleVector(cross2, inv_length, cross2);
   }

angle = RAD2DEG * acos(ScalarProduct(cross1, cross2));

return(angle);
}


/*......................................................................*/
/* The procedure tests, wheather the given parameter is between the	*/
/* left_limit and right_limit (including the margins)			*/
/* If: left_limit <= number <= right_limit 				*/
/* the integer 1 (==TRUE) is returned, else 0 is returned.		*/

int between_or_equal(double number, double left_limit, double right_limit)
{

if ((left_limit<=number) && (right_limit>=number))
	{return 1;
	}
else {return 0;
     }
}
/*......................................................................*/
/* The procedure tests, wheather the given parameter is between the	*/
/* left_limit and right_limit (including the margins)			*/
/* If: left_limit <= number < right_limit 				*/
/* the integer 1 (==TRUE) is returned, else 0 is returned.		*/

int between_or_left_equal(double number, double left_limit, double right_limit)
{

if ((left_limit<=number) && (right_limit>number))
	{return 1;
	}
else {return 0;
     }
}

/*......................................................................*/
/* The procedure tests, wheather the given parameter is between the	*/
/* left_limit and right_limit (including the margins)			*/
/* If: left_limit < number <= right_limit 				*/
/* the integer 1 (==TRUE) is returned, else 0 is returned.		*/

int between_or_right_equal(double number, double left_limit, double right_limit)
{
if ((left_limit<number) && (right_limit>=number))
	{return 1;
	}
else {return 0;
     }
}
/*......................................................................*/
/* The procedure tests, wheather the given parameter is between the	*/
/* left_limit and right_limit (excluding the margins)			*/
/* If: left_limit < number < right_limit 				*/
/* the integer 1 (==TRUE) is returned, else 0 is returned.		*/

int between(double number, double left_limit, double right_limit)
{

if ((left_limit<number) && (right_limit>number))
	{return 1;
	}
else {return 0;
     }
}
/*......................................................................*/
/* Function returns the maximum integer of an int-Array			*/
/* The vector is defined in Numerical Recipes-Convention.		*/
/*									*/

int max_int(int *vector, int number)
{
int help, i;

help= vector[1];
for (i=2; i<=number; i++)
	{if (vector[i]>help)
		{help=vector[i];
		}
	}
return help;
}

/*8888888888888888888888888888888888888888888888888888888888888888888888*/
/*......................................................................*/
/*  transforms a vector from one coordinate system to another.		*/
/*   vori  = vector in original coordinate system			*/
/*   vnew  = resultant vector in new coordinate system			*/
/*   x,y,z = axis vectors of new coordinate system in terms of orig 	*/
/*	     coord system						*/
/*  									*/

void tr(double *vori, double *vnew, double *x, double *y, double *z)
{
vnew[1]=ScalarProduct(vori, x);
vnew[2]=ScalarProduct(vori, y);
vnew[3]=ScalarProduct(vori, z);
}
/*......................................................................*/
/* Function normalizes a vector to unity.				*/
/* The vector is defined in Numerical Recipes-Convention.		*/
/* Entspricht NORM in BDTIRM7.f						*/

void normalize(double *vector,int number)
{
double sum=0.0;
int i;

for (i=1; i<=number; i++)
	{sum=sum+(vector[i]*vector[i]);
	}
sum=sqrt(sum);
for (i=1; i<=number; i++)
	{if (sum!=0.0) vector[i]=vector[i]/sum;
	}
}

/*......................................................................*/
/* Function calculates the length of a vector.				*/
/* The vector is defined in Numerical Recipes-Convention.		*/
/* Entspricht LENGTH in BDTIRM7.f					*/

double length(double *vector,int number)
{
double sum=0.0;
int i;

for (i=1; i<=number; i++)
	{sum=sum+(vector[i]*vector[i]);
	}
return sqrt(sum);
}

/*......................................................................*/
/* Function calculates the vectorproduct of two 3D-vectors.		*/
/* The vector is defined in Numerical Recipes-Convention.		*/
/* Entspricht CROSS in BDTIRM7.f					*/
/* ACHTUNG- nur 3D vectoren.						*/
/* auch ueber Macros in vector.h loesbar				*/

void vectorproduct(double *result, double *vector1, double *vector2)
{
result[1]=(vector1[2]*vector2[3])-(vector1[3]*vector2[2]);
result[2]=(vector1[3]*vector2[1])-(vector1[1]*vector2[3]);
result[3]=(vector1[1]*vector2[2])-(vector1[2]*vector2[1]);
}

/*......................................................................*/
/* Function calculates the scalarproduct of two 3D-vectors.		*/
/* The vector is defined in Numerical Recipes-Convention.		*/
/* Entspricht DOT in BDTIRM7.f						*/
/* 3D auch ueber Macros in vector.h loesbar				*/

double scalarproduct(double *vector1, double *vector2, int number)
{
double sum=0.0;
int i;
for (i=1; i<=number; i++)
	{sum=sum+(vector1[i]*vector2[i]);
	}
return sum;
}

/*......................................................................*/
/* function returns the mimimum of 3 doubles				*/
/* Entspricht MINIM3 in BDTIRM7.f					*/

double minim3(double val1, double val2, double val3)
{
double back;

if (val1<val2)
	if (val1<val3)
		{back=val1;
		 goto end;
		}
if (val2<val3)
	{back=val2;
	 goto end;
	}
back=val3;
end: /* LABEL */
return back;
}

/*......................................................................*/
/* function returns the mimimum of 2 doubles				*/
/* Entspricht MINIM2 in BDTIRM7.f					*/

double minim2(double val1, double val2)
{
double back;

if (val1<val2)
	{back=val1;
	 goto end;
	}
back=val2;
end: /* LABEL */
return back;
}

/*......................................................................*/
/* function macht irgendwas				*/
/* wahrscheinlich aufbau einer Rotationsmatix oder so etwas		*/
/* Matrix and vector is defined in Numerical Recipes-Convention.	*/
/* Entspricht MATRIX in BDTIRM7.f					*/

void Matrix (double *om, double **rot)
{
double c[4],s[4];

c[1]=cos(om[1]);   s[1]=sin(om[1]);
c[2]=cos(om[2]);   s[2]=sin(om[2]);
c[3]=cos(om[3]);   s[3]=sin(om[3]);
rot[1][1]=(c[2]*c[3]);
rot[1][2]=(s[1]*s[2]*c[3]) - (c[1]*s[3]);
rot[1][3]=(s[1]*s[3]) + (c[1]*s[2]*c[3]);
rot[2][1]=(c[2]*s[3]);
rot[2][2]=(c[1]*c[3]) + (s[1]*s[2]*s[3]);
rot[2][3]=(c[1]*s[2]*s[3]) - (s[1]*c[3]);
rot[3][1]=-s[2];
rot[3][2]=s[1]*c[2];
rot[3][3]=c[1]*c[2];
}



/*......................................................................*/
/* function calculates the Mean-value of the vector			*/

double Mean(double *vector, int number)
{
  double sum=0.0;
  int i;
  
  i=number;
  while (i) 
	{sum=sum+vector[i];
    	 i--;
        }
  if ((double)number!=0.0)  return sum/(double)number;
  else return 0.0;
}

/*......................................................................*/
/* function calculates the squared distance between two vectors		*/
/*                                                                      */
/* Vectors in NR-Convention						*/
/*									*/
/* Synthax: distance(vector1,vector2,vectorlength)                      */
/*                                                                      */

 
double sqr_distance(double *vector1,double *vector2,int vectorlength)
{
/* variables */
double distance=0.0;
int i;

/* Code */
for (i=1; i<=vectorlength; i++)
    {distance= distance + (vector1[i]-vector2[i])*(vector1[i]-vector2[i]);
    }
return (distance);
}

/*..............................................................*/
/* Function determines the maximal element of an vector with 	*/
/* the length 'vector_length.					*/
/*								*/ 
double max_element(double *vector, int vector_length)
{
double max;
int i;

max=vector[1];

for(i=1; i<=vector_length;i++)
    {if (vector[i]>max)
	{max=vector[i];
	}
    }
return max;
} 

/*..............................................................*/
/* Function determines the minimal element of an vector with 	*/
/* the length 'vector_length.					*/
/*								*/ 
double min_element(double *vector, int vector_length)
{
double min;
int i;

min=vector[1];

for(i=1; i<=vector_length;i++)
    {if (vector[i]<min)
	{min=vector[i];
	}
    }
return min;
} 

/*..............................................................*/
/* Calculates the rms between two sets of vectors		*/

double rms(double **xyz1, double **xyz2, int n)
{
double rms=0.0;
int i;

for (i=1; i<=n; i++)
    {rms=rms+SqrDistance(xyz1[i], xyz2[i]);
    }

if (n!=0) rms=sqrt(rms/n);

return rms;
}


/*..............................................................*/
/* procedure transposes a matrix[m][n]				*/
/*								*/
void transpose_matrix(double **new2, double **old, int m, int n)
{
int i,j;

if (new2==old) 
   {ERROR("new==old in transpose_matrix()");
   }

for (i=1; i<=m; i++)
    {for (j=1; j<=n; j++)
	 {new2[j][i]=old[i][j];
         }
    }

}

