/* Matthias Ullmann, Berlin 10.10.1994					*/
/* Changed 16.10.1995							*/
/* Some Makros and procedures for calculations				*/
/*									*/

/* Comment								*/
/* All Makros begin with a capital letter. All Constants consists only	*/
/* of captial letters.							*/

#ifndef __mathematik__
#define __mathematik__

/* Makros */
#define Sqr(a)  ((a)*(a))
/* ACHTUNG */
/* kann laut Ritchie Fehler produzieren. Mit Klammern duerfte es aber	*/
/* nicht passieren.							*/

/* Calculation of the vectorproduct (3 dimensions) 			*/
/* definition of vector operations using vectors with NR-convention 	*/ 
#define VectorProduct(a, b, c) {c[1]=a[2]*b[3]-a[3]*b[2]; \
                                c[2]=a[3]*b[1]-a[1]*b[3]; \
                                c[3]=a[1]*b[2]-a[2]*b[1];}
/* Calculation of the scalarproduct  (3 dimensions) 			*/
#define ScalarProduct(a, b) (a[1]*b[1]+a[2]*b[2]+a[3]*b[3])
/* Subtraction of two vectors (3 dimensions) 				*/
#define SubVector(a, b, c)   {c[1]=a[1]-b[1];\
                              c[2]=a[2]-b[2];\
                              c[3]=a[3]-b[3];}
/* Calculation of the distance of two vectors in R3 			*/
#define Distance(a, b)   (sqrt((a[1]-b[1])*(a[1]-b[1])\
			      +(a[2]-b[2])*(a[2]-b[2])\
			      +(a[3]-b[3])*(a[3]-b[3])))
/* Calculation of the Square of a distance of two vectors in R3 	*/
#define SqrDistance(a, b)   ((a[1]-b[1])*(a[1]-b[1])\
			    +(a[2]-b[2])*(a[2]-b[2])\
		            +(a[3]-b[3])*(a[3]-b[3]))
/* Addition of two vectors (3 dimensions) */
#define AddVector(a, b, c)   {c[1]=a[1]+b[1];\
                              c[2]=a[2]+b[2];\
                              c[3]=a[3]+b[3];}
/* multiplication of a vector with a scalar (3 dimensions) */
#define ScaleVector(a, x, b) {b[1] =x*a[1];\
                              b[2] =x*a[2];\
                              b[3] =x*a[3];}
#define SqrLength(x) (x[1]*x[1] + x[2]*x[2] + x[3]*x[3])
#define LENGTH(x) (sqrt(x[1]*x[1] + x[2]*x[2] + x[3]*x[3]))
#define ABS_DIFF(a,b) ( (a)>(b) ?  (((a)-(b)) : ((b)-(a)))
#define MAXIMUM(A, B)  	(((A) > (B) ? (A) : (B)))
#define MINIMUM(A, B)  	(((A) < (B) ? (A) : (B)))
#define TORAD(A)     ((A)*0.017453293)
#define TOGRD(A)     ((A)/0.017453293)

/* Makros for rounding, that could replace the analgous functions */
/* NOTE, that the result here is a int not a double		  */
#define Rint(a)   (a>0 ? ((int)(a+0.5)) : ((int)(a-0.5)) )
#define Ceil(a)   (a>0 ? ((int)(a+1.0)) : ((int)(a))     )
#define Floor(a)  (a>0 ? ((int)(a))     : ((int)(a-1.0)) )

/* function Declaration */
int power_int(int x, int y);
double sqr(double numeral);
double bond_angle(double *atom1, double *atom2, double *atom3);
double dihedral(double *atom1,double *atom2,double *atom3, double *atom4);
int between(double number, double left_limit, double right_limit);
int between_or_equal(double number, double left_limit, double right_limit);
int between_or_left_equal(double number, double left_limit, double right_limit);
int between_or_right_equal(double number, double left_limit, double right_limit);
double Mean(double *vector, int number);
double sqr_distance(double *vector1,double *vector2,int vectorlength);
double plane_angle(double *plane1_1,double *plane1_2,double *plane1_3,
		   double *plane2_1,double *plane2_2,double *plane2_3);


int max_int(int *vector, int number);
void tr(double *vori, double *vnew, double *x, double *y, double *z);
void normalize(double *vector,int number);
void vectorproduct(double *result, double *vector1, double *vector2);
double scalarproduct(double *vector1, double *vector2, int number);
double length(double *vector,int number);
double minim3(double val1, double val2, double val3);
double minim2(double val1, double val2);
void Matrix (double *om, double **rot);
double max_element(double *vector, int vector_length);
double min_element(double *vector, int vector_length);
double rms(double **xyz1, double **xyz2, int n);

void polar2cartesian(double *c, double *p);
void cartesian2polar(double *p, double *c);

long ipow(int x, int y);

void transpose_matrix(double **new2, double **old, int m, int n);

/*********************************************************************/
/* Combinatorics                                                     */
double factorial(int n);
double binomial_coefficient1(int n, int k);
double binomial_coefficient2(int n, int k);
double binomial_coefficient3(int n, int k);

/*********************************************************************/
/* Wrapper for lapack routines                                       */
void eigenvalueDGEEV(double *eigenreal, double *eigenimag, 
                     double **eigenvec, double **c, int n);

/*********************************************************************/
/* numeric routines                                                  */
void lubksb(double **a, int n, int *indx, double *b);
void ludcmp(double **a, int n, int *indx, double *d);
void gaussj(double **a, int n, double *b);

/*********************************************************************/
/* Matrix operations */


double ScalarProd(double *a, int na, double *b, int nb);
void MultMatrix(double **C, int mc, int nc,
                double **A, int ma, int na, double **B, int mb, int nb);
void SubstractMatrix(double **C, int mc, int nc,
                     double **A, int ma, int na, double **B, int mb, int nb);
void AddMatrix(double **C, int mc, int nc,
               double **A, int ma, int na, double **B, int mb, int nb);
void ScalarMatrix(double **C, int mc, int nc,
                  double s, double **A, int ma, int na);
void TransposeMatrix(double **C, int mc, int nc,
                     double **A, int ma, int na);
void MatrixVectMult(double *c, int mc,
                    double **A, int ma, int na, double *b, int nb);
void ScalarVector(double *c, int nc, 
                  double s, double *a, int na);

void PrintMatrix(FILE *fp, double **c, int n, int m);
void PrintiMatrix(FILE *fp, int **c, int n, int m);
void PrintVector(FILE *fp, double *c, int n);
void PrintMatrixXYZ(FILE *fp, double **c, int n, int m);
double MaxElement(double **c, int n, int m);
void InvertMatrix(double **a, int n);


#endif  /*__mathematik__*/
