/* Matthias Ullmann, Bayreuth Jan 2006                          */
/* some routines for matrix calculations.                       */
/*                                                              */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "util.h"
#include "memutil.h"
#include "mathematik.h"

/* .....................................................*/
void InvertMatrix(double **a, int n)
{

    double **y,d,*col;
    int i,j,*indx;

    /* allocate memory */
    indx = ivector(1,n);
    col  = dvector(1,n);
    y    = dmatrix(1,n, 1,n);

    /* Decompose the matrix just once. */
    ludcmp(a,n,indx,&d);
    /* Find inverse by columns */
    for (j=1;j<=n;j++) 
        {for(i=1;i<=n;i++) col[i]=0.0;
         col[j]=1.0;
         lubksb(a,n,indx,col);
         for(i=1;i<=n;i++) y[i][j]=col[i];
        }

    /* now copy the inverse to a */
    for (j=1;j<=n;j++) 
        {for (i=1;i<=n;i++) 
            {a[i][j] = y[i][j];
            }
        }
    

    /* release the memory */
    free_ivector(indx, 1,n);
    free_dvector(col , 1,n);
    free_dmatrix(y   , 1,n, 1,n);

}
/* .....................................................*/
/*                                                      */
double MaxElement(double **c, int n, int m)
{
    double maxele=0.0;
    int i,j;

    for (i=1; i<=n; i++)
        for (j=1; j<=m; j++)
            if (maxele<c[i][j]) maxele=c[i][j];

    return maxele;
}
/* .....................................................*/
/*                                                      */
void PrintVector(FILE *fp, double *c, int n)
{

    int i;

    for (i=1; i<=n; i++)
        {fprintf(fp,"%10.3e  ",c[i]);
        }
    fprintf(fp,"\n\n");
}
/* .....................................................*/
/*                                                      */
void PrintMatrix(FILE *fp, double **c, int n, int m)
{

    int i,j;

   for (i=1; i<=n; i++)
       {for (j=1; j<=m; j++)
           {fprintf(fp,"%10.3e  ",c[i][j]);
           }
        fprintf(fp,"\n");
       }
   fprintf(fp,"\n");
}
/* .....................................................*/
/*                                                      */
void PrintiMatrix(FILE *fp, int **c, int n, int m)
{

    int i,j;

   for (i=1; i<=n; i++)
       {for (j=1; j<=m; j++)
           {fprintf(fp,"%3d  ",c[i][j]);
           }
        fprintf(fp,"\n");
       }
   fprintf(fp,"\n");
}
/* .....................................................*/
/*                                                      */
void PrintMatrixXYZ(FILE *fp, double **c, int n, int m)
{

    int i,j;

   for (i=1; i<=n; i++)
       {for (j=1; j<=m; j++)
           {fprintf(fp,"%d %d %10.3e  \n",i, j, c[i][j]);
           }
       }
}
/*............................................................. */
/* Scalarproduct of two vectors                                 */
/*  sum = a[na]*b[nb]                                           */
/*                                                              */
double ScalarProd(double *a, int na, double *b, int nb)
{
    int i;
    double sum=0.0;

    if (na!=nb) memerror("na!=nb in ScalarProd()");

    for (i=1; i<=na; i++)
        {sum=sum+(a[i]*b[i]);
        }
    return sum;
}

/*............................................................. */
/* multiplication of two matrices:                              */
/*         C[nc][mc] = A[ma][na]*B[mb][nb]                      */
/*                                                              */

void MultMatrix(double **C, int mc, int nc, 
                double **A, int ma, int na, double **B, int mb, int nb)
{
    int i,j, k;

    /* test if the formats are ok */
    if (ma!=mc) memerror("ma!=mc in MultMatrix()");
    if (na!=mb) memerror("na!=mb in MultMatrix()");
    if (nb!=nc) memerror("nb!=nc in MultMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = 0.0;
             for (k=1; k<=na;  k++)
                 C[i][j] = C[i][j] + A[i][k]*B[k][j];
            }
        }

}

/*............................................................. */
/* substraction of two matrices:                                */
/*         C[nc][mc] = A[ma][na]-B[mb][nb]                      */
/*                                                              */

void SubstractMatrix(double **C, int mc, int nc, 
                double **A, int ma, int na, double **B, int mb, int nb)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=mc) memerror("ma!=mc in SubstractMatrix()");
    if (ma!=mb) memerror("ma!=mb in SubstractMatrix()");
    if (na!=nc) memerror("na!=nc in SubstractMatrix()");
    if (na!=nb) memerror("na!=nb in SubstractMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = A[i][j] - B[i][j];
            }
        }

}
/*............................................................. */
/* Addition of two matrices:                                    */
/*         C[nc][mc] = A[ma][na]+B[mb][nb]                      */
/*                                                              */

void AddMatrix(double **C, int mc, int nc, 
               double **A, int ma, int na, double **B, int mb, int nb)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=mc) memerror("ma!=mc in AddMatrix()");
    if (ma!=mb) memerror("ma!=mb in AddMatrix()");
    if (na!=nc) memerror("na!=nc in AddMatrix()");
    if (na!=nb) memerror("na!=nb in AddMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = A[i][j] + B[i][j];
            }
        }
}
/*............................................................. */
/* Multiplication of a  matrix  with a scalar                   */
/*         C[nc][mc] = s*A[ma][na]                              */
/*                                                              */

void ScalarMatrix(double **C, int mc, int nc, 
                  double s, double **A, int ma, int na)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=mc) memerror("ma!=mc in ScalarMatrix()");
    if (na!=nc) memerror("na!=nc in ScalarMatrix()");
        
    for (i=1; i<=mc; i++)
        {for (j=1; j<=nc; j++)
            {C[i][j] = s*A[i][j];
            }
        }
}
/*............................................................. */
/* Multiplication of a  matrix  with a scalar                   */
/*         c[nc] = s*A[ma][na]                              */
/*                                                              */

void ScalarVector(double *c, int nc, 
                  double s, double *a, int na)
{
    int i;

    /* test if the formats are ok */
    if (na!=nc) memerror("na!=nc in ScalarVector()");
        
    for (i=1; i<=nc; i++)
        {c[i] = s*a[i];
        }
}
/*............................................................. */
/* Transpose a  matrix                                          */
/*        A[ma][na] ---> C[nc][mc]                              */
/*                                                              */

void TransposeMatrix(double **C, int mc, int nc, 
                     double **A, int ma, int na)
{
    int i,j;

    /* test if the formats are ok */
    if (ma!=nc) memerror("ma!=nc in TransposeMatrix()");
    if (na!=mc) memerror("na!=mc in TransposeMatrix()");
        
    for (i=1; i<=ma; i++)
        {for (j=1; j<=na; j++)
            {C[j][i] = A[i][j];
            }
        }
}
/*............................................................. */
/* multiplication of a matrix with a vector                     */
/*        c[mc] = A[ma][na] * b[nb]                         */
/*                                                              */

void MatrixVectMult(double *c, int mc, 
                    double **A, int ma, int na, double *b, int nb)
{
    int i, k;

    /* test if the formats are ok */
    if (ma!=mc) memerror("ma!=mc in MultMatrix()");
    if (nb!=mc) memerror("nb!=nc in MultMatrix()");
        
    for (i=1; i<=mc; i++)
        {c[i] = 0.0;
         for (k=1; k<=na;  k++)
             c[i] = c[i] + A[i][k]*b[k];
        }
}
