/* Matthias Ullmann, 2016                               */
/*                                                      */
/* Molecular Structure         			        */
/*......................................................*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mol.h"
#include "util.h"
#include "memutil.h"
#include "mathematik.h"
#include "properties.h"
#include "nb-list.h"

/* ------------------------------------------------------------	*/


/* ------------------------------------------------------------	*/
/* Procedure allocates the single vectors and matricies in a 	*/
/* Mol structure. The structure component "natoms" must         */
/* be assigned, befor calling this procedure.			*/
/*								*/
/* ------------------------------------------------------------	*/

/*
a[i] is a pointer to a[i][0]
*/

void Mol_alloc(Mol *m)
{
   /* Test */
   if ((*m).natoms==0)
	{fprintf(STDERR, "natom=0 - Abort in Mol_alloc.\n");
         exit(EXIT_FAILURE);
        }

   (*m).idstring = charmatrix(1,(*m).natoms, 0, 100);
   (*m).index    = ivector(1,(*m).natoms);  
   
   /* Allocation */	
   (*m).identifier = charmatrix(1,(*m).natoms,0,6); 	/*[7][N]*/
   (*m).atomno     = ivector(1,(*m).natoms);  
   (*m).atomid     = charmatrix(1,(*m).natoms,0,4); 	/*[5][N]*/
   (*m).alternate  = charvector(1,(*m).natoms);	    
   (*m).chain      = charvector(1,(*m).natoms);	  
   (*m).residue    = charmatrix(1,(*m).natoms,0,3);	/*[4][N]*/
   (*m).res_no     = ivector(1,(*m).natoms);   
   (*m).insert     = charvector(1,(*m).natoms);	  
   (*m).xyz        = dmatrix(1,(*m).natoms,1,3);   	/*[4][N]*/		  
   (*m).new_xyz    = dmatrix(1,(*m).natoms,1,3);   	/*[4][N]*/		  
   (*m).save_xyz   = dmatrix(1,(*m).natoms,1,3);   	/*[4][N]*/		  
   (*m).bval	   = dvector(1,(*m).natoms);	   
   (*m).occu	   = dvector(1,(*m).natoms);	   
    
   (*m).atom_type  = charmatrix(1,(*m).natoms,0,5);    /*[5][N]*/
   (*m).type_no    = ivector(1,(*m).natoms);  

   (*m).element    = charmatrix(1,(*m).natoms,0,4); 	/*[5][N]*/
   (*m).mass       = dvector(1,(*m).natoms);
   (*m).surface    = dvector(1,(*m).natoms);	/* surface accessiblility */
   (*m).charge	   = dvector(1,(*m).natoms);	   
   (*m).radius	   = dvector(1,(*m).natoms);
   (*m).covrad	   = dvector(1,(*m).natoms);
   (*m).VdWrad	   = dvector(1,(*m).natoms);
   (*m).dist_cm	   = dvector(1,(*m).natoms);

   (*m).aa_no      = ivector(1,(*m).natoms);   	/* for counting */

   if ((*m).nbonds    !=0) (*m).bond    = imatrix(1,(*m).nbonds,1,2);
   if ((*m).nangles   !=0) (*m).angle   = imatrix(1,(*m).nangles,1,3);
   if ((*m).ntorsions !=0) (*m).torsion = imatrix(1,(*m).ntorsions,1,4);
   if ((*m).nimprop   !=0) (*m).improp  = imatrix(1,(*m).nimprop,1,4);

   (*m).cm      = dvector(1,3);          /* center of mass     */
   (*m).save_cm = dvector(1,3);          /* saveing of center of mass     */
   (*m).new_cm  = dvector(1,3);          /* saveing of center of mass     */
   
}

/* ------------------------------------------------------------	*/
/* Procedure frees the single vectors and matricies in a 	*/
/* MUS structure. 						*/
/*								*/
/* ------------------------------------------------------------	*/
void Mol_free(Mol *m)
{
   /* Test */
   if ((*m).natoms==0)
	{fprintf(STDERR, "natoms=0 - Abort in Mol_free.\n");
         exit(EXIT_FAILURE);
        }

   free_charmatrix((*m).idstring  ,1,(*m).natoms, 0, 100);
   free_charmatrix((*m).identifier,1,(*m).natoms,0,6); 	/*[7][N]*/
   free_charmatrix((*m).atomid    ,1,(*m).natoms,0,4); 	/*[5][N]*/
   free_charmatrix((*m).element   ,1,(*m).natoms,0,4); 	/*[5][N]*/
   free_charmatrix((*m).residue   ,1,(*m).natoms,0,3);	/*[4][N]*/
   free_charvector((*m).alternate ,1,(*m).natoms);	    
   free_charvector((*m).chain     ,1,(*m).natoms);	  
   free_ivector   ((*m).index     ,1,(*m).natoms);  
   free_ivector   ((*m).atomno    ,1,(*m).natoms);  
   free_ivector   ((*m).res_no    ,1,(*m).natoms);
   free_charvector((*m).insert    ,1,(*m).natoms);	
   free_dmatrix   ((*m).xyz       ,1,(*m).natoms,1,3);   	/*[4][N]*/		  
   free_dvector   ((*m).bval      ,1,(*m).natoms);	   
   free_dvector   ((*m).occu      ,1,(*m).natoms);	       
   free_dvector   ((*m).mass      ,1,(*m).natoms);
   free_dvector   ((*m).surface   ,1,(*m).natoms);	/* surface accessiblility	*/
   free_dvector   ((*m).charge    ,1,(*m).natoms);	   
   free_dvector   ((*m).radius    ,1,(*m).natoms);
   free_dvector   ((*m).covrad    ,1,(*m).natoms);
   free_dvector   ((*m).VdWrad    ,1,(*m).natoms);
   free_ivector   ((*m).aa_no     ,1,(*m).natoms);   	/* for counting */

   if ((*m).aa!=NULL) free_imatrix((*m).aa, 1,(*m).aminoacids,1,2);

   free_charmatrix((*m).atom_type, 1,(*m).natoms,0,5);    /*[5][N]*/
   free_ivector   ((*m).type_no, 1,(*m).natoms);  

   if ((*m).nbonds    !=0) free_imatrix((*m).bond   , 1,(*m).nbonds,1,2);
   if ((*m).nangles   !=0) free_imatrix((*m).angle  , 1,(*m).nangles,1,3);
   if ((*m).ntorsions !=0) free_imatrix((*m).torsion, 1,(*m).ntorsions,1,4);
   if ((*m).nimprop   !=0) free_imatrix((*m).improp , 1,(*m).nimprop,1,4);

   if ((*m).n_onsurf  !=0) free_ivector((*m).onsurf , 1,(*m).n_onsurf);
}

/*--------------------------------------------------------------*/
/* Conversion of a MUS atom to a IS string                      */
/*                                                              */
void Atom2Str(char *str, Mol *p, int i)
{
    char atomid[10], res[10], chain, hlp[10];

    sprintf(hlp,"%s",(*p).atomid[i]);
    sscanf(hlp,"%s",atomid);
    sprintf(hlp,"%s",(*p).residue[i]);
    sscanf(hlp,"%s",res);

    if ((*p).chain[i]==' ') chain='-';
    else chain=(*p).chain[i];

    /* sprintf(str,"%s:%d:%c:%s", res, (*p).res_no[i], chain, atomid);*/
    sprintf(str,"%c:%d:%s:%s", chain, (*p).res_no[i], res, atomid);

}

/*--------------------------------------------------------------*/
/* assign atomid string                                         */
/*                                                              */
void AssignIDString(Mol *p)
{
    int i;

    for (i=1; i<=(*p).natoms; i++) 
        {Atom2Str((*p).idstring[i], p, i);
	}
}


/*--------------------------------------------------------------*/
/* search for the atom with the ID:  resname:resid:chainid:atomname 
   i.e. for instance MET:95:A:SD 
   and return its index value
**/
int GetAtomID(char *h,  Mol *p)
{
    int i, val=0, count=0;

    for (i=1; i<=(*p).natoms; i++)
        {if (strcmp((*p).idstring[i], h)==0)
	    {val = i; count ++;
	    }
	}

    if (count == 0) 
       {fprintf(stderr,"Atom ID String: %s\n",h);
        muerror("StringID not found!");
       }
    if (count > 1) 
       {fprintf(stderr,"Atom ID String: %s\n",h);
        muerror("StringID more than once");
       }

    return val;
}

/* ------------------------------------------------------------	*/
void assign_chain(Mol *p)
{
   int i, j, count=1;
   
   /* Counting of the aminoacids */
   j=1;
   for (i=2; i<=(*p).aminoacids; i++)
       {if ((*p).chain[(*p).aa[i][1]]!=(*p).chain[(*p).aa[i-1][1]])
	   j++;
       }
   /* allocate memory */
   (*p).nchains    = j;
   (*p).aa_inchain = imatrix(1,(*p).nchains,1,2);

   /* Assignment */
   j=1;
   (*p).aa_inchain[j][1]=1;
   for (i=2; i<=(*p).aminoacids; i++)
       {if ((*p).chain[(*p).aa[i][1]]!=(*p).chain[(*p).aa[i-1][1]])
	   {(*p).aa_inchain[j][2]=count;
	    j++;
	    (*p).aa_inchain[j][1]=i;
	    count=1;
	   }
	else {count++;
	     }
       }
   (*p).aa_inchain[j][2]=count;  
}

/* ------------------------------------------------------------	*/
/* Procedure assignes aminoacids, actually residues     	*/
/* It assumes that different residues have different residues 	*/
/* numbers. Numbering via insert is not allowed			*/
/* ------------------------------------------------------------	*/
void assign_aminoacid(Mol *p)
{
   int i, j, aa, count=0;
   
   /* Counting of the aminoacids */
   j=1;
   for (i=2; i<=(*p).natoms; i++)
       {if (  ((*p).res_no[i] != (*p).res_no[i-1])
           || ((*p).insert[i] != (*p).insert[i-1]) ) j++;
       }

   /* allocate */
   (*p).aminoacids=j;  
   (*p).aa  = imatrix(1,(*p).aminoacids,1,2);

   /* Assignment */
   j=1; aa=1;
   (*p).aa[aa][1]=1;
   for (i=2; i<=(*p).natoms; i++) 
       {if (((*p).res_no[i] != (*p).res_no[i-1])
         || ((*p).insert[i] != (*p).insert[i-1]) )
   	{(*p).aa[aa][2]=j;
   	 j=1;aa++;
   	 (*p).aa[aa][1]=i;
   	}
        else j++;
       }
   (*p).aa[aa][2]=j;
   /* assign the value to aa_no */
   for (i=1;  i<=(*p).aminoacids; i++) 
       {for (j=0; j<(*p).aa[i][2]; j++)
   	 {count++;
   	  (*p).aa_no[count]=i;
   	 }
       }
}


/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a PDB to a specified file  */
/* The procedure prints the radius in the occupancy and the 	*/
/* charge in the B-value column.				*/
/* ------------------------------------------------------------	*/
void fprint_PDBatom(FILE *fpout, Mol *m, int number)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %c%4d%c   %8.3f%8.3f%8.3f%6.2f%7.3f\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
          (*m).alternate[number],
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).chain[number],
          (*m).res_no[number],
          (*m).insert[number],
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          (*m).bval[number],
          (*m).occu[number]);
}

/* ------------------------------------------------------------	*/
/* prints an aminoacid with a given number			*/
/* Output is in PDB-Format					*/
/* ------------------------------------------------------------	*/
void fprint_aminoacid(FILE *fpout, Mol *p, int aa_no)
{
   int i;

   if (aa_no>(*p).aminoacids) ERROR("Aminoacid number not existent in fprint_aminoacid().");

   for (i=0; i<(*p).aa[aa_no][2];i++)
       {fprint_PDBatom(fpout, p, (*p).aa[aa_no][1]+i); 
       }

}

/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a MUS to a specified file  */
/* in pqr-Format (MEAD)						*/
/* ------------------------------------------------------------	*/
void fprint_PQRMatom(FILE *fpout, Mol *m, int number)
{
   fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c   %7.3f %7.3f %7.3f %10.6f %7.3f %7.3f\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
	  ' ',
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).res_no[number],
	  ' ',
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          (*m).charge[number],
          (*m).radius[number],
          (*m).mass[number]);
}
/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a MUS to a specified file  */
/* in PXX-Format, i.e. the last two columns are free		*/
/* ------------------------------------------------------------	*/
void fprint_PXXatom(FILE *fpout, Mol *m, int number, double *x1, double *x2)
{
   fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c    %7.3f %7.3f %7.3f %10.6f %7.3f\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
	  ' ',
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).res_no[number],
	  ' ',
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          x1[number],
          x2[number]);
}

/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a MUS to a specified file  */
/* in extendent pqr-Format (MEAD)				*/
/* ------------------------------------------------------------	*/
void fprint_PQRXatom(FILE *fpout, Mol *m, int number)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c    %7.3f %7.3f %7.3f %10.6f %7.3f %d %d %d %d\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
	  ' ',
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).res_no[number],
	  ' ',
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          (*m).charge[number],
          (*m).radius[number],
	  0, 
          0, 
          (*m).res_no[number], 
          0);
}

/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a MUS to a specified file  */
/* in extendent pqr-Format (MEAD)				*/
/* ------------------------------------------------------------	*/
void fprint_PQRXatom2(FILE *fpout, Mol *m, int number, int hlp)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c    %7.3f %7.3f %7.3f %10.6f %7.3f %d %d %d %d\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
	  ' ',
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).res_no[number],
	  ' ',
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          (*m).charge[number],
          (*m).radius[number],
	  0, 
          hlp, 
          (*m).res_no[number], 
          0);
}
/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a MUS to a specified file  */
/* in pqr-Format (MEAD)						*/
/* ------------------------------------------------------------	*/
void fprint_PQRatom(FILE *fpout, Mol *m, int number)
{
  fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %5d%c    %7.3f %7.3f %7.3f %10.6f %7.3f\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
          ' ',
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).res_no[number],
          ' ',
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          (*m).charge[number],
          (*m).radius[number]);
}
/* ------------------------------------------------------------	*/
/* Procedure prints a single atom of a MUS to a specified file  */
/* in pqr-Format (MEAD)						*/
/* ------------------------------------------------------------	*/
void fprint_PQRChain_atom(FILE *fpout, Mol *m, int number)
{
fprintf(fpout,
       "ATOM  %5d %c%c%c%c%c%c%c%c %c %3d    %7.3f %7.3f %7.3f %10.6f %7.3f\n",
          (*m).atomno[number],
          (*m).atomid[number][0],
          (*m).atomid[number][1],
          (*m).atomid[number][2],
          (*m).atomid[number][3],
	  ' ',
          (*m).residue[number][0],
          (*m).residue[number][1],
          (*m).residue[number][2],
          (*m).chain[number],
          (*m).res_no[number],
          (*m).xyz[number][1],
          (*m).xyz[number][2],
          (*m).xyz[number][3],
          (*m).charge[number],
          (*m).radius[number]);
}


/* ------------------------------------------------------------	*/
/* Procedure prints a Dummy atom 				*/
/* The value in bval is writen to the temperature factor	*/
/* so that is can be used to color atoms			*/
/* ------------------------------------------------------------	*/
void fprint_DUMMYatom(FILE *fpout, int number, const char *atomid, char *resid,
		      double *xyz, double occu, double bval)
{
    fprintf(fpout, "ATOM  %5d %c%c%c%c %3s %5d     %7.3f %7.3f %7.3f %8.3f %8.3f\n",
	    number, atomid[0], atomid[1], atomid[2], atomid[3], resid, 
            number, xyz[1], xyz[2], xyz[3], bval, occu);
}

/* ------------------------------------------------------------	*/
/* Procedure prints a MUS to a specified file in PQR-Format	*/
/* ------------------------------------------------------------	*/
void fprint_PQRChain(FILE *fpout, Mol *m)
{
int i;

for (i=1; i<=(*m).natoms; i++)
    {fprint_PQRChain_atom(fpout, m, i);
    }
fprintf(fpout,"TER\n");
}       

/* ------------------------------------------------------------	*/
/* Procedure prints a MUS to a specified file in PQR-Format	*/
/* ------------------------------------------------------------	*/
void fprint_PQR(FILE *fpout, Mol *m)
{
int i;

for (i=1; i<=(*m).natoms; i++)
    {fprint_PQRatom(fpout, m, i);
    }
fprintf(fpout,"TER\n");
}       

/* ------------------------------------------------------------	*/
/* Procedure prints a MUS to a specified file in PQR-Format	*/
/* ------------------------------------------------------------	*/
void fprint_PQRM(FILE *fpout, Mol *m)
{
int i;

for (i=1; i<=(*m).natoms; i++)
    {fprint_PQRMatom(fpout, m, i);
    }
fprintf(fpout,"TER\n");
}       
/* ------------------------------------------------------------	*/
/* Procedure prints a MUS to a specified file in PQRX-Format	*/
/* ------------------------------------------------------------	*/
void fprint_PQRX(FILE *fpout, Mol *m)
{
int i;

for (i=1; i<=(*m).natoms; i++)
    {fprint_PQRXatom(fpout, m, i);
    }
fprintf(fpout,"TER\n");
}       
/* ------------------------------------------------------------	*/
/* Procedure prints a MUS to a specified file in PDB-Format	*/
/* ------------------------------------------------------------	*/
void fprint_PDB(FILE *fpout, Mol *m)
{
int i;

for (i=1; i<=(*m).natoms; i++)
    {fprint_PDBatom(fpout, m, i);
    }
fprintf(fpout,"TER\n");
}       

/* ------------------------------------------------------------	*/
/* Reads the given line	- ATOM or HETATM			*/
/* in MEAD style						*/
/* .............                                                */
/* Change 6 April                                               */
/* MEAD does not real read the PDB-Format and a pattern matching*/
/* is better for conversions                                    */
/* ------------------------------------------------------------	*/
void read_PQRMatom(Mol *pdb, int i, char *line)
{
char help[6];/*printf("%s",line);*/
    (*pdb).insert[i]    = ' ';
    (*pdb).alternate[i] = ' ';
    (*pdb).chain[i]     = ' ';
    sscanf(line,
       "%s %d %s %s %d %lf %lf %lf %lf %lf %lf\n",
       (*pdb).identifier[i], &(*pdb).atomno[i],
       (*pdb).atomid[i], (*pdb).residue[i],  &(*pdb).res_no[i],
       &(*pdb).xyz[i][1], &(*pdb).xyz[i][2], &(*pdb).xyz[i][3],
       &(*pdb).charge[i], &(*pdb).radius[i], &(*pdb).mass[i]);

    help[0]=(*pdb).atomid[i][0]; 
    help[1]=(*pdb).atomid[i][1];
    help[2]=(*pdb).atomid[i][2]; 
    help[3]=(*pdb).atomid[i][3];
    if (strlen((*pdb).atomid[i])==4)
        {(*pdb).atomid[i][0]=help[0]; (*pdb).atomid[i][1]=help[1];
         (*pdb).atomid[i][2]=help[2]; (*pdb).atomid[i][3]=help[3];
        } 
    else if (strlen((*pdb).atomid[i])==3)
        {(*pdb).atomid[i][0]=' ';     (*pdb).atomid[i][1]=help[0];
         (*pdb).atomid[i][2]=help[1]; (*pdb).atomid[i][3]=help[2];
        } 
    else if (strlen((*pdb).atomid[i])==2)
       {(*pdb).atomid[i][0]=' ';     (*pdb).atomid[i][1]=help[0];
        (*pdb).atomid[i][2]=help[1]; (*pdb).atomid[i][3]=' ';
       } 
    else if (strlen((*pdb).atomid[i])==1)
       {(*pdb).atomid[i][0]=' ';     (*pdb).atomid[i][1]=help[0];
        (*pdb).atomid[i][2]=' ';     (*pdb).atomid[i][3]=' ';
       } 
    (*pdb).atomid[i][4]='\0';
    if (strlen((*pdb).residue[i])==2)
        {(*pdb).residue[i][2]=' ';
        } 
}

/* ------------------------------------------------------------	*/
/* Reads the given line	- ATOM or HETATM			*/
/* ------------------------------------------------------------	*/
void read_PDBatom(Mol *pdb, int i, char *line)
{
    /*printf("%s",line);*/
    sscanf(line   , "%c",  &((*pdb).identifier[i][0]));
    sscanf(line+1 , "%c",  &((*pdb).identifier[i][1]));
    sscanf(line+2 , "%c",  &((*pdb).identifier[i][2]));
    sscanf(line+3 , "%c",  &((*pdb).identifier[i][3]));
    sscanf(line+4 , "%c",  &((*pdb).identifier[i][4]));
    sscanf(line+5 , "%c",  &((*pdb).identifier[i][5]));
    (*pdb).identifier[i][6]='\0';
    sscanf(line+ 6 , "%5d", &((*pdb).atomno[i]));
    sscanf(line+ 12, "%c",  &((*pdb).atomid[i][0]));
    sscanf(line+ 13, "%c",  &((*pdb).atomid[i][1]));
    sscanf(line+ 14, "%c",  &((*pdb).atomid[i][2]));
    sscanf(line+ 15, "%c",  &((*pdb).atomid[i][3]));
    (*pdb).atomid[i][4]='\0';
    sscanf(line+ 16, "%c",  &((*pdb).alternate[i]));
    sscanf(line+ 17, "%c",  &((*pdb).residue[i][0]));
    sscanf(line+ 18, "%c",  &((*pdb).residue[i][1]));
    sscanf(line+ 19, "%c",  &((*pdb).residue[i][2]));
    (*pdb).residue[i][3]='\0';
    sscanf(line+ 21, "%c",  & ((*pdb).chain[i]));
    sscanf(line+ 22, "%4d", &((*pdb).res_no[i]));
    sscanf(line+ 26, "%c",  &((*pdb).insert[i]));
    sscanf(line+ 30, "%8lf",&(((*pdb).xyz[i][1])));
    sscanf(line+ 38, "%8lf",&(((*pdb).xyz[i][2])));
    sscanf(line+ 46, "%8lf",&(((*pdb).xyz[i][3])));
    
    sscanf(line+ 54, "%6lf",&((*pdb).occu[i]));
    sscanf(line+ 60, "%7lf",&((*pdb).bval[i]));
}

/* ------------------------------------------------------------	*/
/* Procedure reads in a PDB 					*/
/* A lot of things done here are not nessesarry in all 		*/
/* application however, it is not very time consuming		*/
/* and this function is only called a few times during a run	*/
/* ------------------------------------------------------------	*/

void read_PDB(char *filename, Mol *pdb)
{
FILE *fpin;
char line[LINELENGTH];
int atom=0;


    (*pdb).nbonds    = 0;
    (*pdb).nangles   = 0;
    (*pdb).ntorsions = 0;
    (*pdb).nimprop   = 0;

    fpin=file_open(filename,"r");
    if (fpin==NULL) return;
    
    while (fgets(line,LINELENGTH,fpin)!=NULL)
          {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
          }
    (*pdb).natoms=atom;
    
    Mol_alloc(pdb);
    rewind(fpin);
    
    atom=0; 
    
    while (fgets(line,LINELENGTH,fpin)!=NULL)
          {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
    	  {atom++; read_PDBatom(pdb, atom, line);}
          } 
    
    assign_aminoacid(pdb);
    assign_chain(pdb);
    fclose(fpin);
}

/*........................................................*/
void ReadXYZ(char *fname, Mol *p)
{
    FILE *fpin;
    char line[LINELENGTH], help[20];
    int n,i;

    (*p).nbonds    = 0;
    (*p).nangles   = 0;
    (*p).ntorsions = 0;
    (*p).nimprop   = 0;

    fpin=file_open(fname,"r");
    READ_LINE(line, fpin);
    if (1!=sscanf(line,"%d",&n))
        {ERROR("Can not read number of atoms.");
        }
    fprintf(stderr, "There %d atoms in the xyz file\n",n);
    READ_LINE(line, fpin);
    (*p).natoms = n;

    Mol_alloc(p);

    for (i=1; i<=n; i++)
        {READ_LINE(line, fpin);
        if (4!=sscanf(line,"%s %lf %lf %lf", help,
		      &(*p).xyz[i][1], &(*p).xyz[i][2], &(*p).xyz[i][3]))
             {ERROR("Problem while reading coordinates");
             }
	 strcpy((*p).residue[i], "DUM");
         (*p).atomno[i]  = i;
         (*p).res_no[i]  = i;
         (*p).alternate[i] = ' ';
         (*p).chain[i]     = ' ';
         (*p).insert[i]    = ' ';
         (*p).atomid[i][0] = ' ';
         (*p).atomid[i][1] = help[0];
         if (strlen(help)> 1) (*p).atomid[i][2] = help[1];
         else (*p).atomid[i][2] = ' ';
         if (strlen(help)> 2) (*p).atomid[i][3] = help[2];
         else (*p).atomid[i][3] = ' ';
         (*p).atomid[i][4] = '\0';
  
       }

    fclose(fpin);

}
/* ------------------------------------------------------------	*/
/* Reads the given line	- ATOM or HETATM			*/
/* in MEAD style						*/
/* .............                                                */
/* Change 6 April                                               */
/* MEAD does not real read the PDB-Format and a pattern matching*/
/* is better for conversions                                    */
/* ------------------------------------------------------------	*/
void read_PQRatom(Mol *pdb, int i, char *line)
{
    /*char help[6];printf("%s",line);*/
    
    sscanf(line+1 , "%c",  &((*pdb).identifier[i][1]));
    sscanf(line+2 , "%c",  &((*pdb).identifier[i][2]));
    sscanf(line+3 , "%c",  &((*pdb).identifier[i][3]));
    sscanf(line+4 , "%c",  &((*pdb).identifier[i][4]));
    sscanf(line+5 , "%c",  &((*pdb).identifier[i][5]));
    (*pdb).identifier[i][6]='\0';
    sscanf(line+ 6 , "%5d", &((*pdb).atomno[i]));
    sscanf(line+ 12, "%c",  &((*pdb).atomid[i][0]));
    sscanf(line+ 13, "%c",  &((*pdb).atomid[i][1]));
    sscanf(line+ 14, "%c",  &((*pdb).atomid[i][2]));
    sscanf(line+ 15, "%c",  &((*pdb).atomid[i][3]));
    (*pdb).atomid[i][4]='\0';
    sscanf(line+ 16, "%c",  &((*pdb).alternate[i]));
    sscanf(line+ 17, "%c",  &((*pdb).residue[i][0]));
    sscanf(line+ 18, "%c",  &((*pdb).residue[i][1]));
    sscanf(line+ 19, "%c",  &((*pdb).residue[i][2]));
    (*pdb).residue[i][3]='\0';
    sscanf(line+ 21, "%c",  & ((*pdb).chain[i]));
    sscanf(line+ 22, "%4d", &((*pdb).res_no[i]));
    sscanf(line+ 26, "%c",  &((*pdb).insert[i]));
    sscanf(line+ 30, "%8lf",&(((*pdb).xyz[i][1])));
    sscanf(line+ 38, "%8lf",&(((*pdb).xyz[i][2])));
    sscanf(line+ 46, "%8lf",&(((*pdb).xyz[i][3])));
    sscanf(line+ 53, "%lf %lf",&((*pdb).charge[i]), &((*pdb).radius[i]));

    if ((*pdb).radius[i]>10.0) 
      {muwarn("large atom radius in atom:");
       fprint_PQRMatom(stderr, pdb, i);
      }
}

/* .................................................... */
/* prints the incorrect PDB-styled CONECT-records to    */
/* a file                                               */
/*                                                      */
/* format could be                                      */
/*  - " %4d" -- to get PDB-like format with             */
/*              guarenteed spaces                       */
/*  - "%5d"  -- to get exact PDB format without         */
/*              guarenteed spaces                       */
/*                                                      */

void fprint_conect(FILE *fpout, int **conect, int len, char *format)
{
int i,j;

for (i=1; i<=len; i++)
    {fprintf(fpout,"CONECT");
     for (j=1; j<=MAX_CONECT;j++)
           {fprintf(fpout, format, conect[i][j]);
            if (0==conect[i][j+1])
               break;
           } 
     fprintf(fpout,"\n");
    }

}

/* ------------------------------------------------------------	*/
/* Procedure reads in a POR 					*/
/* A lot of things done here are not nessesarry in all 		*/
/* application however, it is not very time consuming		*/
/* and this function is only called a few times during a run	*/
/* ------------------------------------------------------------	*/

void read_PQR(char *filename, Mol *pdb)
{
    FILE *fpin;
    char line[LINELENGTH];
    int atom=0;
    
    fpin=file_open(filename,"r");
    
    (*pdb).nbonds    = 0;
    (*pdb).nangles   = 0;
    (*pdb).ntorsions = 0;
    (*pdb).nimprop   = 0;

    while (fgets(line,LINELENGTH,fpin)!=NULL)
          {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
          }
    (*pdb).natoms=atom;
    
    Mol_alloc(pdb);
    rewind(fpin);
    
    atom=0; 
    
    while (fgets(line,LINELENGTH,fpin)!=NULL)
          {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
    	      {atom++; 
               read_PQRatom(pdb, atom, line);
               /*fprint_PQRatom(stdout, pdb, atom);TEST;*/
              }
          } 
    
    assign_aminoacid(pdb);
    assign_chain(pdb);
    fclose(fpin);
}

/* ------------------------------------------------------------	*/
/* Procedure reads in a POR 					*/
/* A lot of things done here are not nessesarry in all 		*/
/* application however, it is not very time consuming		*/
/* and this function is only called a few times during a run	*/
/* ------------------------------------------------------------	*/

void read_PQRM(char *filename, Mol *pdb)
{
FILE *fpin;
char line[LINELENGTH];
int atom=0;


    (*pdb).nbonds    = 0;
    (*pdb).nangles   = 0;
    (*pdb).ntorsions = 0;
    (*pdb).nimprop   = 0;

    fpin=file_open(filename,"r");

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6))) atom++;
      }
(*pdb).natoms=atom;

Mol_alloc(pdb);
rewind(fpin);

atom=0; 

while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if ((!strncmp(line,"ATOM  ",6))||(!strncmp(line,"HETATM",6)))
	  {atom++; 
           read_PQRMatom(pdb, atom, line);
          }
      } 

assign_aminoacid(pdb);
assign_chain(pdb);
fclose(fpin);
}

/*--------------------------------------------------------------*/
/* Reads a molecule file in different formats                   */
/* The variable type should hold the type of the file.          */
/*                                                              */
void read_Molecule(char *filename, Mol *p, char *type)
{
    /* determine format */
    /* test for the last three or four characters */
    if (0==strcmp(type,"UNK"))
       {if (strlen(filename)<=4) 
	  {fprintf(stderr,"%s \n",filename);
	   muerror("Too short filename in read_Molecule()");
	  }
        if      (0==strcmp("pqr" ,filename+strlen(filename)-3))   strcpy(type,"PQR");
        else if (0==strcmp("pqrm",filename+strlen(filename)-4))   strcpy(type,"PQRM");
        else if (0==strcmp("pdb" ,filename+strlen(filename)-3))   strcpy(type,"PDB");
        else if (0==strcmp("xyz" ,filename+strlen(filename)-3))   strcpy(type,"XYZ");
        else {fprintf(stderr,"%s -- unknown molecular structure file\n",filename);
              exit(10);
             }
       }
    /* open the file */
    if (0==strcmp(type,"PDB"))       read_PDB(filename, p);
    else if (0==strcmp(type,"PQR"))  read_PQR(filename, p);
    else if (0==strcmp(type,"PQRM")) read_PQRM(filename, p);
    else if (0==strcmp(type,"XYZ"))  ReadXYZ(filename, p);
    else {fprintf(stderr,"%s -- unknown molecular structure file\n",filename);
          exit(10);
         }
    AssignIDString(p);

}

/* .................................................... */
/* recounts the aminoacids and the atoms in a MUS-file  */
/* ATTENTION						*/
/* Care must be taken, that two neightboured amino-	*/
/* acids does not have the same id number!!!!		*/
/*							*/
void recount_aminoacids(Mol *protein)
{
   int i,aa, save;
   char save_insert, save_chain;
   
   /* Assignment */
   aa=1;
   i=1;
   save=(*protein).res_no[i];
   save_insert = (*protein).insert[i];
   save_chain  = (*protein).chain[i];
   (*protein).res_no[i]=aa;
   (*protein).atomno[i]=i;
   for (i=2; i<=(*protein).natoms; i++) 
       {(*protein).atomno[i]=i;
              if ( ((*protein).res_no[i] != save) || ((*protein).insert[i]!=save_insert))
   	      {if (save_chain!=(*protein).chain[i])
   		  {aa=1;
   		   save_chain  = (*protein).chain[i];
   		  }
   		else {aa++;}
   	 save=(*protein).res_no[i];
                  save_insert = (*protein).insert[i];
   	 (*protein).res_no[i]=aa;
   	}
        else (*protein).res_no[i]=aa;
   	  (*protein).insert[i]=' ';
       }
}

/* .................................................... */
/* guess elements                                       */
/*                                                      */

void GuessElements(Mol *p)
{
   int i;
   char h1[20],h2[20],  h[20];

   for (i=1; i<=(*p).natoms; i++) 
       {sprintf(h1,"%s",(*p).atomid[i]);
        sscanf(h1,"%s",h);
        sprintf(h1,"%s",(*p).residue[i]);
        sscanf(h1,"%s",h2);
        if (strncmp(h, "FE",2)==0)
	   strcpy((*p).element[i],"Fe");
	else if (strncmp(h, "CU",2)==0)
	   strcpy((*p).element[i],"Cu");
	else if (strcmp(h, "NI")==0)
	   strcpy((*p).element[i],"Ni");
	else if (strcmp(h, "SE")==0)
	   strcpy((*p).element[i],"Se");
	else if (strncmp(h, "MG",2)==0)
	   strcpy((*p).element[i],"Mg");
	else if (strcmp(h, "ZN")==0)
	   strcpy((*p).element[i],"Zn");
	else if (strncmp(h, "MO",2)==0)
	   strcpy((*p).element[i],"Mo");
	else if (strncmp(h, "MN",2)==0)
	   strcpy((*p).element[i],"Mn");
	else if (strncmp(h, "AS",2)==0)
	   strcpy((*p).element[i],"As");
	else if (strncmp(h, "AL",2)==0)
	   strcpy((*p).element[i],"Al");
        else if (strncmp(h, "BI",2)==0)
	   strcpy((*p).element[i],"Bi");
        else if (strncmp(h, "BR",2)==0)
	   strcpy((*p).element[i],"Br");
        else if (strncmp(h, "BA",2)==0)
	   strcpy((*p).element[i],"Ba");
        else if (strncmp(h, "XE",2)==0)
	   strcpy((*p).element[i],"Xe");
        else if (strncmp(h, "RU",2)==0)
	   strcpy((*p).element[i],"Ru");
        else if (strncmp(h, "LI",2)==0)
	   strcpy((*p).element[i],"Li");
        else if (strncmp(h, "UNK",3)==0)
	   strcpy((*p).element[i],"UNK");
        else if (strncmp(h, "DUM",3)==0)
	   strcpy((*p).element[i],"UNK");
        else if (strcmp(h, "X")==0)
	   strcpy((*p).element[i],"UNK");
	/* some less clear cases */
	else if ((strcmp(h, "NA")==0)&&(0==strncmp(h2, "NA", 2)))
	   strcpy((*p).element[i],"Na");
	else if ((strcmp(h, "CA")==0)&&(0==strncmp(h2, "CA", 2)))
	   strcpy((*p).element[i],"Ca");
	else if ((strcmp(h, "CL")==0)&&(0==strncmp(h2, "CL", 2)))
	   strcpy((*p).element[i],"Cl");
 	/* some one-character elements */
        else if (h[0]=='K')
	   strcpy((*p).element[i],"K");
	else if (h[0]=='W')
	   strcpy((*p).element[i],"W");
	else if (h[0]=='F')
	   strcpy((*p).element[i],"F");
	else if (h[0]=='I')
	   strcpy((*p).element[i],"I");
	else if (h[0]=='V')
	   strcpy((*p).element[i],"V");
	/* now just C, P, N, O, S */
        else if (h[0]=='C')
	   strcpy((*p).element[i],"C");
        else if (h[0]=='P')
	   strcpy((*p).element[i],"P");
        else if (h[0]=='N')
	   strcpy((*p).element[i],"N");
        else if (h[0]=='O')
	   strcpy((*p).element[i],"O");
        else if (h[0]=='S')
	   strcpy((*p).element[i],"S");
        /* and finally H */
        else if ((*p).atomid[i][0]=='H')
	   strcpy((*p).element[i],"H");
        else if ((*p).atomid[i][1]=='H')
	   strcpy((*p).element[i],"H");
	/* or deuterium ... D */
        else if ((*p).atomid[i][0]=='D')
	   strcpy((*p).element[i],"D");
        else if ((*p).atomid[i][1]=='D')
	   strcpy((*p).element[i],"D");
        else {fprint_PQRatom(stderr, p, i);
	      muerror("Unknown element in GuessElements()");
             }
       }

}

/* .................................................... */
/* assigns the covalent and Van der Waals radii as well */
/* as masses from an element structure                  */
/*                                                      */

void AssignCovRad(Mol *p, Element *e)
{
  int i,j;
  int found;

  if ((*e).nele<=0) muerror("(*e).nele<=0");

  for (i=1; i<=(*p).natoms; i++)
      {found = 0;
       for (j=1; j<=(*e).nele; j++)
      	  {if (0==strcmp((*p).element[i], (*e).name[j]))
      	      {(*p).mass[i]   = (*e).mass[j];
      	       (*p).covrad[i] = (*e).covrad[j];
      	       (*p).VdWrad[i] = (*e).VdWrad[j];
      	       found=1;
	       break;
      	      }
      	  }
       if (found==0)
          {fprintf(stderr, "==========================================\n");
      	   fprint_PQRatom(stderr, p, i);
	   fprintf(stderr, "Element %s", (*p).element[i]);
      	   muwarn("Element could not be assigned in AssignCovRad()");
           fprintf(stderr, "==========================================\n");
      	  }
      }

}

/* --------------------------------------------------------- */

void BondGraph2CONECT(int **conect, char **BondGraph, int n)
{
  int i,j, count;

   for (i=1; i<=n; i++)
       {/* assign the index */
        conect[i][1]=i;
	count=1;
        for (j=1; j<=n; j++)
	    {if (BondGraph[i][j]==1)
		{count++;
                 if (count>MAX_CONECT)
		    {fprintf(STDERR,"%d %d %d\n",i, j, count);
                     ERROR("Array too small in BondGraph2CONECT().");
                    }
		 else conect[i][count]=j;
		}

            }
       }

}

/* -------------------------------------------------- */
/* see if two atoms are covalently bond               */
/*                                                    */
/* Bond := {Dist(i,j)<=(CovRad[i]+CovRad[j])}         */
/*                                                    */

void BondGraphFromDist(Mol *p)
{
    double dist;
    int i,j;

    /* calculate distance */
    for (i=1; i<=(*p).natoms; i++)
      {(*p).BondGraph[i][i]=0;
       for (j=1; j<i; j++)
	     {dist = Distance((*p).xyz[i], (*p).xyz[j]);
	       
	      if (dist<= ((*p).covrad[i]+(*p).covrad[j]))
		 {(*p).BondGraph[i][j]=(*p).BondGraph[j][i]=1; /* bond */
		 }
	      else (*p).BondGraph[i][j]=(*p).BondGraph[j][i]=0; /* no bond */
	     }
	 }

}


/* .................................................................... */
/* returns the atom id number in aminoacid aa_no in mus with the atom   */
/* id given as a char paameter to the function                          */
int get_no3(Mol *mus, int aa_no, char chain, const char *atom_id)
{
int i,j,s;
int back=0; 

s=(*mus).aa[aa_no][1];
for (i=0; i<(*mus).aa[aa_no][2]; i++)
  {if  ((0==strcmp(atom_id,(*mus).atomid[s+i]))
        && (chain == (*mus).chain[s+i]))
        {if (back==0)
            {back = (*mus).atomno[s+i];
            }
         else {WARN("Atom ID appears twice in get_no().");
               for (j=0;j<(*mus).aa[aa_no][2];j++)
                   {fprint_PDBatom(STDERR,mus,s+j);
                   }
               fprintf(STDERR,"Aminoacid %d Chain '%c' AtomID '%s'\n",
                       aa_no, chain, atom_id);
              }
        }
    }
if (back==0)
   {fprintf(stderr, "=======================================================\n");
    fprintf(stderr, "Atom ID: %s\n", atom_id);
    WARN("Atom ID does not appear in get_no().");
    for (j=0;j<(*mus).aa[aa_no][2];j++)
        {fprint_PDBatom(STDERR,mus,s+j);
        }
    fprintf(STDERR,"Aminoacid %d Chain '%c' AtomID '%s'\n"
            ,aa_no, chain, atom_id);
    fprintf(stderr, "=======================================================\n");
   }

return back;
}

/* ---------------------------------------------------- */
/* procedure checks which atoms lie on the surface      */
/* of the protein  and generates a list of these atoms  */
/*                                                      */
/* ATTENTION - Surface must be calculated prior calling */
/* this function.                                       */
/*                                                      */

void on_surf(Mol *protein)
{
int i, j=0;

/* looking for the number of surface atoms */
for (i = 1; i <= (*protein).natoms; i++)
        if ((*protein).surface[i]!=0.0) j++;

(*protein).n_onsurf=j;

/* allocation */
(*protein).onsurf=ivector(1,(*protein).n_onsurf);

/* assigning */
j=0;
for (i = 1; i <= (*protein).natoms; i++)
        if ((*protein).surface[i]!=0.0) 
                {j++;
                 (*protein).onsurf[j]=i;
                }
}

/* ------------------------------------------------------------ */
/* procedure recovers all xyz-coordinates from save_xyz         */
/*                                                              */

void recover_initial_coordinates(Mol *protein)
{
int i;

for (i=1; i<=(*protein).natoms; i++)
        {(*protein).xyz[i][1]=(*protein).save_xyz[i][1];
         (*protein).xyz[i][2]=(*protein).save_xyz[i][2];
         (*protein).xyz[i][3]=(*protein).save_xyz[i][3];
        }
}
         
/* ------------------------------------------------------------ */
/* procedure copies all xyz-coordinates to save_xyz             */
/* and cm to save_cm                                            */

void save_initial_coordinates(Mol *protein)
{
int i;

(*protein).save_cm[1]=(*protein).cm[1];
(*protein).save_cm[2]=(*protein).cm[2];
(*protein).save_cm[3]=(*protein).cm[3];

for (i=1; i<=(*protein).natoms; i++)
        {(*protein).save_xyz[i][1]=(*protein).xyz[i][1];
         (*protein).save_xyz[i][2]=(*protein).xyz[i][2];
         (*protein).save_xyz[i][3]=(*protein).xyz[i][3];
        }
}
/* ------------------------------------------------------------ */
/* procedure calculates the properties of molecules and assigns */
/* the rest of the structure.                                   */
/* ATTENTION                                                    */
/* diffusion coefficients are at present not assigned.          */
/* ------------------------------------------------------------ */
void properties(Mol *mus)
{
(*mus).total_mass  =center_of_mass((*mus).xyz, (*mus).mass, (*mus).natoms,
                                   (*mus).cm);
(*mus).total_charge=charge((*mus).charge, (*mus).natoms);
(*mus).dipol       =dipol((*mus).xyz, (*mus).charge, (*mus).natoms,
                          (*mus).cm, (*mus).dipolvec);
(*mus).max_cm_dist = max_dist((*mus).xyz, (*mus).natoms, (*mus).cm, (*mus).dist_cm);
}

/* ------------------------------------------------------------ */
/* procedure prints properties                                  */
/* ATTENTION                                                    */
/* diffusion coefficients are inactive and so are at present    */
/* not printed.                                                 */
/* ------------------------------------------------------------ */
void fprint_properties(FILE *fpout, Mol *mus)
{
fprintf(fpout,"\t Number of atoms:         %d\n",(*mus).natoms);
/*
fprintf(fpout,"\t Number of bonds:         %d\n",(*mus).nbonds);
fprintf(fpout,"\t Number of angles:                %d\n",(*mus).nangles);
fprintf(fpout,"\t Number of torsions:              %d\n",(*mus).ntorsions);
fprintf(fpout,"\t Number of improper torsions:     %d\n",(*mus).nimprop);
*/
 
fprintf(fpout,"\t Mass:                %8.3f\n",(*mus).total_mass);
fprintf(fpout,"\t Center of Mass:      %8.3f  %8.3f  %8.3f\n",(*mus).cm[1],(*mus).cm[2],(*mus).cm[3]);
fprintf(fpout,"\t Charge:              %8.3f\n",(*mus).total_charge);
fprintf(fpout,"\t Dipole:              %8.3f  = %8.3f Debye\n",(*mus).dipol,(*mus).dipol*4.802);
fprintf(fpout,"\t Dipole Vector-KM:    %8.3f  %8.3f  %8.3f\n",
        (*mus).dipolvec[1],(*mus).dipolvec[2],(*mus).dipolvec[3]);
fprintf(fpout,"\t Max. Dist. from CM:  %8.3f\n",(*mus).max_cm_dist);
}

/* -------------------------------------------------- */
/* see if two atoms are covalently bond               */
/*                                                    */
/* Bond := {Dist(i,j)<=(CovRad[i]+CovRad[j])}         */
/*                                                    */

void BondGraphFromDistList(Mol *p)
{
    double dist, maxrad;
    iListGrid atomgrid;
    iList *help;
    int n, i,j,k, ih,jh,kh;
    int iptatmgrd[4];

    maxrad=max_element((*p).covrad,(*p).natoms);
    atomgrid.spacing = 2*maxrad; 
    //(*atomgrid).look = (int)ceil((2*maxrad+1.0) * (*atomgrid).inv_spacing);
    atomgrid.look = 1;
    ProteinToGrid(&atomgrid, p, maxrad);

    /*
    for (i=atomgrid.imin[1]; i<=atomgrid.imax[1]; i++)
        for (j=atomgrid.imin[2]; j<=atomgrid.imax[2]; j++)
            for (k=atomgrid.imin[3]; k<=atomgrid.imax[3]; k++)
	        {help=&atomgrid.root[i][j][k];
		 fprintf(stderr,"%4d %4d %4d:",i,j,k);
		 while ((*help).next!=NULL)
		   {help=(*help).next;
		    fprintf(stderr," %d",(*help).number);
		   }
		 fprintf(stderr,"\n");
		}
    TEST;
    */

    /*zero out BondGraph */
    for (i=1; i<(*p).natoms; i++)
        {(*p).BondGraph[i][i]=0;
         for (j=1; j<i; j++)
	     {(*p).BondGraph[i][j]=(*p).BondGraph[j][i]=0; /* no bond */
	     }
	}

    /* calculate distance */
    for (n=1; n<=(*p).natoms; n++)
        {/* assign the atom to atomgridcell */
	 if (0!=pt_to_cell(iptatmgrd,(*p).xyz[n],&atomgrid))
            {muerror("atom outside the grid in pt_inside_atom()");
            }

         for (i=-atomgrid.look; i<=atomgrid.look; i++)
             {ih=iptatmgrd[1]+i;
              if(!((ih>atomgrid.imax[1])||(ih<atomgrid.imin[1])))
                {for (j=-atomgrid.look; j<=+atomgrid.look; j++)
                     {jh=iptatmgrd[2]+j;
                      if (!((jh>atomgrid.imax[2])||(jh<atomgrid.imin[2])))
                         {for (k=-atomgrid.look; k<=+atomgrid.look; k++)
                              {kh=iptatmgrd[3]+k;
                               if (!((kh>atomgrid.imax[3])||(kh<atomgrid.imin[3])))
                                  {help=&atomgrid.root[ih][jh][kh];
                                   /* check the distance */
		                   //fprintf(stderr,"%4d %4d %4d:",ih,jh,kh);
				   while ((*help).next!=NULL)
                                       {help=(*help).next;
					 //fprintf(stderr,"%d %d\n",n, (*help).number);
					 //fprint_PDBatom(stderr, p, n);
					 //fprint_PDBatom(stderr, p, (*help).number);
                                        if ((*help).number!=n)
					   {dist = Distance((*p).xyz[n], (*p).xyz[(*help).number]);
					    if (dist<= ((*p).covrad[n]+(*p).covrad[(*help).number]))
					       {(*p).BondGraph[n][(*help).number]=1; /* bond */
					        (*p).BondGraph[(*help).number][n]=1; /* bond */
					       }
					   }
				       }
				  }
			      }
			 }
		     }
		}
	     }
	}

    //CHANGE
    // release memory

}

