/* Matthias Ullmann, 2016				*/
/*                                                      */
/* Molecular Structure         			        */
/*......................................................*/

#ifndef MOL
#define MOL

#define MAX_CONECT 8

#define PSE_FILE "/home/ullmann/Parameter/General/period-table-info.txt"


/* declaration of structures */
/* The aminoacid i is assigned by the two numbers	*/
/* 1. Its starting point - aa[i][1]			*/
/* 2. Its length - aa[i][2]				*/
/* Internally, all Emin are stored as sqrt(-Emin)	*/
/* The conversion is done in the IO-Rountines		*/
typedef struct {char molname[256]; 
                int  natoms; /* N */
                
                /* info read from a PDB file */
		char   **identifier; 	/*[N][7]*/
	     	int    *atomno;  
	     	char   **atomid; 	/*[N][5]*/
                char   *alternate;	    
	     	char   **residue;      	/*[N][4]*/	    
	     	char   *chain;	  
	     	int    *res_no;   
	     	char   *insert;	  
	     	double **xyz;   	/*[N][4]*/
		double *bval;	/* temperature factor	*/
                double *occu;	/* occupancy	*/

                double *cm;        /*[4]*/
                double max_cm_dist;
		double *new_cm;
		double *dist_cm;
                
                double **save_xyz;
                double **new_xyz;
                double *save_cm;        /*[4]*/
  
                /* some more info */
                char   **element;
                double *mass;
	     	double *surface;	/* surface accessiblility		*/
		double *radius;         /* radius for PBE and surface           */
                double *covrad;         /* covalent radius */
                double *VdWrad;         /* van der Waals radius */
                double *charge;

                double total_mass;
                double total_charge;
                double dipol;
                double dipolvec[4];
  
                /* force field */
                char **atom_type;       /*[N][5]*/
                int  *type_no;

                /* topology */
                int    nbonds;
                int    **bond;          /* [N][2] */
                int    nangles;
                int    **angle;         /* [N][3] */
                int    ntorsions;
                int    **torsion;               /* [N][4] */
                int    nimprop;
                int    **improp;                /* [N][4] */

                /* derived parameter */
	     	int  *index;
                char **idstring; 	   	/*[N][5]*/
	     	 
                /* define residues */
                int  aminoacids;	/* number of aminoacids			*/
		int  **aa;		/* to find aminoacids - 		*/
					/* start atom number aminacid and length*/
		int  *aa_no;		/* aa number [1..atoms]			*/

                /* define chains */
                int nchains;            /* number of chains			*/
                int **aa_inchain;       /* start aminoacid number and length in */
                                        /* aminoacids */

                /* make a list of atoms on surface */
	     	int n_onsurf;
                int *onsurf;

		/* properties */
		/* CONECT records - only allocated if needed */
                char   **BondGraph; /* [N][N] */
                int    **conect;    /* [N][MAXCONECT] */
	       }Mol;

/*..............................................................
  Some periodic table info
  
  The columns have the following meaning
  
  1. elemental symbol
  2. order number
  3. mass
  4. covalent radus in Angstrom -- maximal
  5. VdW radius in Angstrom -- maximal, high spin
*/

typedef struct {int    nele;
                char   **name;
                int    *odnr; /* order number */
                double *mass;
                double *covrad;
                double *VdWrad;
               }Element;
/*..............................................................*/
#define MAXRES 10000  /* Maximal sequence length */
#define LNRES  50     /* residues per line in FASTA output */

typedef struct {int    nseq;
                char   **title;
                char   **seq;
                int    *nres;
               }fasta;

/*************************/
/* Allocation */
void Mol_alloc(Mol *m);
void Mol_free(Mol *m);

/* make some assignment */
void Atom2Str(char *str, Mol *p, int i);
void AssignIDString(Mol *p);
int  GetAtomID(char *h,  Mol *p);
void assign_aminoacid(Mol *p);

/* IO */
void fprint_PDBatom(FILE *fpout, Mol *m, int number);
void fprint_aminoacid(FILE *fpout, Mol *p, int aa_no);
void fprint_PQRMatom(FILE *fpout, Mol *m, int number);
void fprint_PXXatom(FILE *fpout, Mol *m, int number, double *x1, double *x2);
void fprint_PQRatom(FILE *fpout, Mol *m, int number);
void fprint_PQRXatom(FILE *fpout, Mol *m, int number);
void fprint_PQRXatom2(FILE *fpout, Mol *m, int number, int hlp);
void fprint_PQRChain_atom(FILE *fpout, Mol *m, int number);
void fprint_DUMMYatom(FILE *fpout, int number, const char *atomid, char *resid,
		      double *xyz, double occu, double bval);
void fprint_PQRChain(FILE *fpout, Mol *m);
void fprint_PQR(FILE *fpout, Mol *m);
void fprint_PQRM(FILE *fpout, Mol *m);
void fprint_PQRX(FILE *fpout, Mol *m);
void fprint_PDB(FILE *fpout, Mol *m);
void fprint_conect(FILE *fpout, int **conect, int len, char *format);

void read_PQRMatom(Mol *pdb, int i, char *line);
void read_PDBatom(Mol *pdb, int i, char *line);
void read_PDB(char *filename, Mol *pdb);
void ReadXYZ(char *fname, Mol *p);
void read_PQRatom(Mol *pdb, int i, char *line);
void read_PQR(char *filename, Mol *pdb);
void read_PQRM(char *filename, Mol *pdb);
void read_Molecule(char *filename, Mol *p, char *type);
void recount_aminoacids(Mol *protein);

void GuessElements(Mol *p);
void AssignCovRad(Mol *p, Element *e);

void BondGraph2CONECT(int **conect, char **BondGraph, int n);
void BondGraphFromDist(Mol *p);
void on_surf(Mol *protein);

int get_no3(Mol *mus, int aa_no, char chain, const char *atom_id);

void BondGraphFromDistList(Mol *p);

/* properties */
double charge(double *charge, int atomnumber);


/* PSE */
void read_PSE(char *filename, Element *e);
void FreeElements(Element *e);
void AllocateElements(Element *e);
void fprint_PSE(FILE *fp, Element *e);

/* Fasta */
void ReadFASTA(fasta *s, char const *name);
void WriteFASTA(FILE *fp, fasta *s);

void recover_initial_coordinates(Mol *pdb);
void save_initial_coordinates(Mol *pdb);

#endif /* MOL */
