/* G. Matthias Ullmann 						*/
/* Bayreuth, June 2009					        */
/*								*/
/* Some routines for reading and writing OpenDX files           */
/*								*/
/* ............................................................ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <zlib.h>
#include "util.h"
#include "memutil.h"
#include "grid.h"


/* ............................................................ */
/* read a openDX file                                           */
/*                                                              */
/* ATTENTION: This procedure may read some OpenDX files not     */
/*            correctly. In order to transfer it into a dGrid   */
/*            structure, special charaterizations need to be    */
/*            justified.                                        */
/*                                                              */
void gzReadDX(OpenDX *dx, char *fname)
{
     char line[LINELENGTH]; 
     char s1[LINELENGTH];
     int h,icount;
     int ihelp, nc;
     gzFile fp;

     fp=gzopen(fname,"r");

     while (gzgets(fp,line,LINELENGTH)!=NULL)
         {if (line[0]=='#') /*comment line */
             continue; /* just to do something */
          else break;
         }
     /* now read the file */
     h=sscanf(line, "%*s %*d %*s %s %*s %d %d %d",
              s1, &(*dx).npt[1], &(*dx).npt[2], &(*dx).npt[3]);
     if (h!=4) 
        {fprintf(stderr,"%s",line);
	 ERROR("error while reading dx-file 0");
	}
     if (0!=strcmp(s1,"gridpositions"))
         ERROR("error while reading gridpositions from opendx.");

     gzREAD_LINE(line, fp);
     h=sscanf(line, "%s %lf %lf %lf",
              s1, &(*dx).ori[1], &(*dx).ori[2], &(*dx).ori[3]);
     if (h!=4) ERROR("error while reading dx-file 1");
     if (0!=strcmp(s1,"origin"))
         ERROR("error while reading origin from opendx.");
             
     gzREAD_LINE(line, fp);
     if (h!=4) ERROR("error while reading dx-file 2");
     h=sscanf(line, "%s %lf %lf %lf",s1, 
            &(*dx).delta[1][1], &(*dx).delta[1][2], &(*dx).delta[1][3]);
     if (0!=strcmp(s1,"delta"))
         ERROR("error while reading delta 1 from opendx.");
             
     gzREAD_LINE(line, fp);
     h=sscanf(line, "%s %lf %lf %lf",s1, 
            &(*dx).delta[2][1], &(*dx).delta[2][2], &(*dx).delta[2][3]);
     if (h!=4) ERROR("error while reading dx-file 3");
     if (0!=strcmp(s1,"delta"))
         ERROR("error while reading delta 2 from opendx.");

     gzREAD_LINE(line, fp);
     h=sscanf(line, "%s %lf %lf %lf",s1, 
            &(*dx).delta[3][1], &(*dx).delta[3][2], &(*dx).delta[3][3]);
     if (h!=4) ERROR("error while reading dx-file 4");
     if (0!=strcmp(s1,"delta"))
         ERROR("error while reading delta 3 from opendx.");

     gzREAD_LINE(line, fp);
     gzREAD_LINE(line, fp);

     /* now read the potential data */
     ihelp = (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3];
     (*dx).data = dvector(1,ihelp+3);
     icount = 0;
     while (gzgets(fp,line,LINELENGTH)!=NULL)
           {nc=sscanf(line, "%lf %lf %lf", 
                      &(*dx).data[icount+1], &(*dx).data[icount+2], 
                      &(*dx).data[icount+3]);
           if (nc>3)
               {fprintf(stderr,"%s", line);
                WARN("Error while reading dx-file");
               }
            icount=icount+nc;
            if (icount>=ihelp) break;
           }
     if (icount!=ihelp) ERROR("error while reading OpenDX data 1.");

     gzclose(fp);
               
}
/* ............................................................ */
/* read a openDX file                                           */
/*                                                              */
/* ATTENTION: This procedure may read some OpenDX files not     */
/*            correctly. In order to transfer it into a dGrid   */
/*            structure, special charaterizations need to be    */
/*            justified.                                        */
/*                                                              */
void ReadDX(OpenDX *dx, FILE *fp)
{
     char line[LINELENGTH]; 
     char s1[LINELENGTH];
     int h,icount;
     int ihelp, nc;

     while (fgets(line,LINELENGTH,fp)!=NULL)
         {if (line[0]=='#') /*comment line */
             s1[0]='!'; /* just to do something */
          else break;
         }
     /* now read the file */
     h=sscanf(line, "%*s %*d %*s %s %*s %d %d %d",
              s1, &(*dx).npt[1], &(*dx).npt[2], &(*dx).npt[3]);
     if (h!=4) ERROR("error while reading dx-file 0");
     if (0!=strcmp(s1,"gridpositions"))
         ERROR("error while reading gridpositions from opendx.");

     READ_LINE(line, fp);
     h=sscanf(line, "%s %lf %lf %lf",
              s1, &(*dx).ori[1], &(*dx).ori[2], &(*dx).ori[3]);
     if (h!=4) ERROR("error while reading dx-file 1");
     if (0!=strcmp(s1,"origin"))
         ERROR("error while reading origin from opendx.");
             
     READ_LINE(line, fp);
     if (h!=4) ERROR("error while reading dx-file 2");
     h=sscanf(line, "%s %lf %lf %lf",s1, 
            &(*dx).delta[1][1], &(*dx).delta[1][2], &(*dx).delta[1][3]);
     if (0!=strcmp(s1,"delta"))
         ERROR("error while reading delta 1 from opendx.");
             
     READ_LINE(line, fp);
     h=sscanf(line, "%s %lf %lf %lf",s1, 
            &(*dx).delta[2][1], &(*dx).delta[2][2], &(*dx).delta[2][3]);
     if (h!=4) ERROR("error while reading dx-file 3");
     if (0!=strcmp(s1,"delta"))
         ERROR("error while reading delta 2 from opendx.");

     READ_LINE(line, fp);
     h=sscanf(line, "%s %lf %lf %lf",s1, 
            &(*dx).delta[3][1], &(*dx).delta[3][2], &(*dx).delta[3][3]);
     if (h!=4) ERROR("error while reading dx-file 4");
     if (0!=strcmp(s1,"delta"))
         ERROR("error while reading delta 3 from opendx.");

     READ_LINE(line, fp);
     READ_LINE(line, fp);

     /* now read the potential data */
     ihelp = (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3];
     (*dx).data = dvector(1,ihelp+3);
     icount = 0;
     while (fgets(line,LINELENGTH,fp)!=NULL)
           {nc=sscanf(line, "%lf %lf %lf", 
                      &(*dx).data[icount+1], &(*dx).data[icount+2], 
                      &(*dx).data[icount+3]);
           if (nc>3)
               {fprintf(stderr,"%s", line);
                WARN("Error while reading dx-file");
               }
            icount=icount+nc;
            if (icount>=ihelp) break;
           }
     if (icount!=ihelp) ERROR("error while reading OpenDX data 1.");
                
}
/* ............................................................ */
/* converts a openDX structure to a dGrid structure             */
/*                                                              */
/* ATTENTION: This procedure may read some OpenDX files not     */
/*            correctly. In order to transfer it into a dGrid   */
/*            structure, special charaterizations need to be    */
/*            justified.                                        */
/*                                                              */
void DX2Grid(dGrid *pot, OpenDX *dx)
{
     int i,j,k, ihh;

     /* some tests - diagonal should be the same number, 
        off diagonal should be zero */
     for (i=1; i<=3; i++)
         {for (j=1; j<=3; j++)
             {if (i!=j) 
                 {if (fabs((*dx).delta[i][j])>0.0000009)
                     {fprintf(stderr,"delta[%d][%d]=%e\n",
                              i,j,(*dx).delta[i][j]);
                      ERROR("delta[i][j]!=0 for i!=j");
                     }
                 }
              else {if (fabs((*dx).delta[i][j]-(*dx).delta[1][1])>0.0000009)
                     {fprintf(stderr,"delta[%d][%d]=%e\n",
                              i,j,(*dx).delta[i][j]);
                      ERROR("delta[i][i]!=delta[1][1]");
                     }
                   }
             }
         }

     /* calculate the info for dGrid */
     (*pot).spacing     = (*dx).delta[1][1];
     (*pot).inv_spacing = 1.0/(*dx).delta[1][1];
     (*pot).imin[1]=(int)floor((*dx).ori[1]/(*pot).spacing);
     (*pot).imin[2]=(int)floor((*dx).ori[2]/(*pot).spacing);
     (*pot).imin[3]=(int)floor((*dx).ori[3]/(*pot).spacing);
     (*pot).imax[1]=(*pot).imin[1]+(*dx).npt[1]-1;
     (*pot).imax[2]=(*pot).imin[2]+(*dx).npt[2]-1;
     (*pot).imax[3]=(*pot).imin[3]+(*dx).npt[3]-1;
                      
     /* allocate the dGrid */
     (*pot).grid=d3tensor((*pot).imin[1], (*pot).imax[1],
                          (*pot).imin[2], (*pot).imax[2],
                          (*pot).imin[3], (*pot).imax[3]);

 
     /* fill the dGrid */
     ihh = 0;
     for (i=(*pot).imin[1];i<=(*pot).imax[1]; i++)
         for (j=(*pot).imin[2];j<=(*pot).imax[2]; j++)
             for (k=(*pot).imin[3];k<=(*pot).imax[3]; k++)
                {ihh++;
                 (*pot).grid[i][j][k] = (*dx).data[ihh];
                }
     if (ihh!=(*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3]) 
         ERROR("error while reading potential data.");

}

/*..............................................................*/
void iGrd2DX(OpenDX *dx, iGrid *dens)
{
    int ih, ihelp;
    int i,j,k;

    (*dx).npt[1]=(*dens).imax[1]-(*dens).imin[1]+1;
    (*dx).npt[2]=(*dens).imax[2]-(*dens).imin[2]+1;
    (*dx).npt[3]=(*dens).imax[3]-(*dens).imin[3]+1;

    (*dx).ori[1]=(*dens).imin[1]*(*dens).spacing;
    (*dx).ori[2]=(*dens).imin[2]*(*dens).spacing;
    (*dx).ori[3]=(*dens).imin[3]*(*dens).spacing;

    for (i=1; i<=3; i++)
        for (j=1; j<=3; j++)
            (*dx).delta[i][j] = 0.0;
    (*dx).delta[1][1]= (*dens).spacing;
    (*dx).delta[2][2]= (*dens).spacing;
    (*dx).delta[3][3]= (*dens).spacing;

    ihelp = (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3];
    (*dx).data = dvector(1,ihelp+3);

    /* now the data */
    ih = 0;
    for (i=(*dens).imin[1];i<=(*dens).imax[1]; i++)
        for (j=(*dens).imin[2];j<=(*dens).imax[2]; j++)
            for (k=(*dens).imin[3];k<=(*dens).imax[3]; k++)
                {ih++;
                 (*dx).data[ih]=(double)(*dens).grid[i][j][k];
                }
}
/*..............................................................*/
void dGrd2DX(OpenDX *dx, dGrid *dens)
{
    int ih, ihelp;
    int i,j,k;

    (*dx).npt[1]=(*dens).imax[1]-(*dens).imin[1]+1;
    (*dx).npt[2]=(*dens).imax[2]-(*dens).imin[2]+1;
    (*dx).npt[3]=(*dens).imax[3]-(*dens).imin[3]+1;

    (*dx).ori[1]=(*dens).imin[1]*(*dens).spacing;
    (*dx).ori[2]=(*dens).imin[2]*(*dens).spacing;
    (*dx).ori[3]=(*dens).imin[3]*(*dens).spacing;

    for (i=1; i<=3; i++)
        for (j=1; j<=3; j++)
            (*dx).delta[i][j] = 0.0;
    (*dx).delta[1][1]= (*dens).spacing;
    (*dx).delta[2][2]= (*dens).spacing;
    (*dx).delta[3][3]= (*dens).spacing;

    ihelp = (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3];
    (*dx).data = dvector(1,ihelp+3);

    /* now the data */
    ih = 0;
    for (i=(*dens).imin[1];i<=(*dens).imax[1]; i++)
        for (j=(*dens).imin[2];j<=(*dens).imax[2]; j++)
            for (k=(*dens).imin[3];k<=(*dens).imax[3]; k++)
                {ih++;
                 (*dx).data[ih]=(double)(*dens).grid[i][j][k];
                }
}
/*..............................................................*/
void Write_iGridToDX(FILE *fp, iGrid *dens)
{
    int ih;
    int l,m,n;
    int i,j,k;

    l=(*dens).imax[1]-(*dens).imin[1]+1;
    m=(*dens).imax[2]-(*dens).imin[2]+1;
    n=(*dens).imax[3]-(*dens).imin[3]+1;

    /* the heading */
    fprintf(fp,"#\n");
    fprintf(fp,"object 1 class gridpositions counts %d %d %d\n",
           l,m,n);
    fprintf(fp,"origin  %e %e %e\n",
           (*dens).imin[1]*(*dens).spacing,
           (*dens).imin[2]*(*dens).spacing,
           (*dens).imin[3]*(*dens).spacing);
    fprintf(fp,"delta  %e %e %e\n",
           (*dens).spacing, 0.0, 0.0);
    fprintf(fp,"delta  %e %e %e\n",
            0.0, (*dens).spacing, 0.0);
    fprintf(fp,"delta  %e %e %e\n",
            0.0, 0.0, (*dens).spacing);
    fprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
           l,m,n);
    fprintf(fp,"object 3 class array type double rank 0 items %d data follows\n",
           l*m*n);

    /* now the data */
    ih = 0;
    for (i=(*dens).imin[1];i<=(*dens).imax[1]; i++)
        for (j=(*dens).imin[2];j<=(*dens).imax[2]; j++)
            for (k=(*dens).imin[3];k<=(*dens).imax[3]; k++)
                {ih++;
                 fprintf(fp,"%e ",(double)(*dens).grid[i][j][k]);
                 if (ih==3)
                    {fprintf(fp,"\n");
                     ih=0;
                    }
                }
    if (ih!=0)  fprintf(fp,"\n");

    /* now the tail */
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"object \"regular positions regular connections\" class field\n");
    fprintf(fp,"component \"positions\" value 1\n");
    fprintf(fp,"component \"connections\" value 2\n");
    fprintf(fp,"component \"data\" value 3\n");

    
}
/*..............................................................*/
void Write_dGridToDX(FILE *fp, dGrid *dens)
{
    int ih;
    int l,m,n;
    int i,j,k;

    l=(*dens).imax[1]-(*dens).imin[1]+1;
    m=(*dens).imax[2]-(*dens).imin[2]+1;
    n=(*dens).imax[3]-(*dens).imin[3]+1;

    /* the heading */
    fprintf(fp,"# Density plot -- Center of mass\n");
    fprintf(fp,"#\n");
    fprintf(fp,"object 1 class gridpositions counts %d %d %d\n",
           l,m,n);
    fprintf(fp,"origin  %e %e %e\n",
           (*dens).imin[1]*(*dens).spacing,
           (*dens).imin[2]*(*dens).spacing,
           (*dens).imin[3]*(*dens).spacing);
    fprintf(fp,"delta  %e %e %e\n",
           (*dens).spacing, 0.0, 0.0);
    fprintf(fp,"delta  %e %e %e\n",
            0.0, (*dens).spacing, 0.0);
    fprintf(fp,"delta  %e %e %e\n",
            0.0, 0.0, (*dens).spacing);
    fprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
           l,m,n);
    fprintf(fp,"object 3 class array type double rank 0 items %d data follows\n",
           l*m*n);

    /* now the data */
    ih = 0;
    for (i=(*dens).imin[1];i<=(*dens).imax[1]; i++)
        for (j=(*dens).imin[2];j<=(*dens).imax[2]; j++)
            for (k=(*dens).imin[3];k<=(*dens).imax[3]; k++)
                {ih++;
                 fprintf(fp,"%e ",(*dens).grid[i][j][k]);
                 if (ih==3)
                    {fprintf(fp,"\n");
                     ih=0;
                    }
                }

    if (ih!=0)  fprintf(fp,"\n");


    /* now the tail */
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"object \"regular positions regular connections\" class field\n");
    fprintf(fp,"component \"positions\" value 1\n");
    fprintf(fp,"component \"connections\" value 2\n");
    fprintf(fp,"component \"data\" value 3\n");

    
}
/*..............................................................*/
void gzWrite_dGridToDX(char *fname, char *comment, dGrid *dens)
{
    int ih;
    int l,m,n;
    int i,j,k;
    gzFile fp;

    fp = gzopen(fname,"w");

    l = (*dens).imax[1]-(*dens).imin[1]+1;
    m = (*dens).imax[2]-(*dens).imin[2]+1;
    n = (*dens).imax[3]-(*dens).imin[3]+1;

    /* the heading */
    gzprintf(fp,"#%s\n", comment);
    gzprintf(fp,"#\n");
    gzprintf(fp,"object 1 class gridpositions counts %d %d %d\n",
           l,m,n);
    gzprintf(fp,"origin  %e %e %e\n",
           (*dens).imin[1]*(*dens).spacing,
           (*dens).imin[2]*(*dens).spacing,
           (*dens).imin[3]*(*dens).spacing);
    gzprintf(fp,"delta  %e %e %e\n",
           (*dens).spacing, 0.0, 0.0);
    gzprintf(fp,"delta  %e %e %e\n",
            0.0, (*dens).spacing, 0.0);
    gzprintf(fp,"delta  %e %e %e\n",
            0.0, 0.0, (*dens).spacing);
    gzprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
           l,m,n);
    gzprintf(fp,"object 3 class array type double rank 0 items %d data follows\n",
           l*m*n);

    /* now the data */
    ih = 0;
    for (i=(*dens).imin[1];i<=(*dens).imax[1]; i++)
        for (j=(*dens).imin[2];j<=(*dens).imax[2]; j++)
            for (k=(*dens).imin[3];k<=(*dens).imax[3]; k++)
                {ih++;
                 gzprintf(fp,"%e ",(*dens).grid[i][j][k]);
                 if (ih==3)
                    {gzprintf(fp,"\n");
                     ih=0;
                    }
                }

    if (ih!=0)  gzprintf(fp,"\n");


    /* now the tail */
    gzprintf(fp,"attribute \"dep\" string \"positions\"\n");
    gzprintf(fp,"object \"regular positions regular connections\" class field\n");
    gzprintf(fp,"component \"positions\" value 1\n");
    gzprintf(fp,"component \"connections\" value 2\n");
    gzprintf(fp,"component \"data\" value 3\n");

    gzclose(fp);  
}
/*..............................................................*/
void gzWrite_iGridToDX(char *fname, char*comment, iGrid *dens)
{
    int ih;
    int l,m,n;
    int i,j,k;
    gzFile fp;

    fp = gzopen(fname,"w");

    l = (*dens).imax[1]-(*dens).imin[1]+1;
    m = (*dens).imax[2]-(*dens).imin[2]+1;
    n = (*dens).imax[3]-(*dens).imin[3]+1;

    /* the heading */
    gzprintf(fp,"#%s\n",comment);
    gzprintf(fp,"#\n");
    gzprintf(fp,"object 1 class gridpositions counts %d %d %d\n",
           l,m,n);
    gzprintf(fp,"origin  %e %e %e\n",
           (*dens).imin[1]*(*dens).spacing,
           (*dens).imin[2]*(*dens).spacing,
           (*dens).imin[3]*(*dens).spacing);
    gzprintf(fp,"delta  %e %e %e\n",
           (*dens).spacing, 0.0, 0.0);
    gzprintf(fp,"delta  %e %e %e\n",
            0.0, (*dens).spacing, 0.0);
    gzprintf(fp,"delta  %e %e %e\n",
            0.0, 0.0, (*dens).spacing);
    gzprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
           l,m,n);
    gzprintf(fp,"object 3 class array type double rank 0 items %d data follows\n",
           l*m*n);

    /* now the data */
    ih = 0;
    for (i=(*dens).imin[1];i<=(*dens).imax[1]; i++)
        for (j=(*dens).imin[2];j<=(*dens).imax[2]; j++)
            for (k=(*dens).imin[3];k<=(*dens).imax[3]; k++)
                {ih++;
		  gzprintf(fp,"%e ",(double)(*dens).grid[i][j][k]);
                 if (ih==3)
                    {gzprintf(fp,"\n");
                     ih=0;
                    }
                }

    if (ih!=0)  gzprintf(fp,"\n");


    /* now the tail */
    gzprintf(fp,"attribute \"dep\" string \"positions\"\n");
    gzprintf(fp,"object \"regular positions regular connections\" class field\n");
    gzprintf(fp,"component \"positions\" value 1\n");
    gzprintf(fp,"component \"connections\" value 2\n");
    gzprintf(fp,"component \"data\" value 3\n");

    gzclose(fp);  
}
/* ............................................................ */
/* calculates the difference between two cube files             */
/* it returns 0 if successful                                   */
/*           -1 if a problem occured                            */
/*                                                              */
int DiffDX(OpenDX *ddx, OpenDX *dx1, OpenDX *dx2)
{

#define EPS 0.0000000001

    int i,ihelp;

    /* check if the file are comparable */
    if ((fabs((*dx1).ori[1]     -(*dx2).ori[1])>EPS)||
        (fabs((*dx1).ori[2]     -(*dx2).ori[2])>EPS)||
        (fabs((*dx1).ori[3]     -(*dx2).ori[3])>EPS)||
        (fabs((*dx1).delta[1][1]-(*dx2).delta[1][1])>EPS)||
        (fabs((*dx1).delta[1][2]-(*dx2).delta[1][2])>EPS)||
        (fabs((*dx1).delta[1][3]-(*dx2).delta[1][3])>EPS)||
        (fabs((*dx1).delta[2][1]-(*dx2).delta[2][1])>EPS)||
        (fabs((*dx1).delta[2][2]-(*dx2).delta[2][2])>EPS)||
        (fabs((*dx1).delta[2][3]-(*dx2).delta[2][3])>EPS)||
        (fabs((*dx1).delta[3][1]-(*dx2).delta[3][1])>EPS)||
        (fabs((*dx1).delta[3][2]-(*dx2).delta[3][2])>EPS)||
        (fabs((*dx1).delta[3][3]-(*dx2).delta[3][3])>EPS)||
        ((*dx1).npt[1]!=(*dx2).npt[1])||
        ((*dx1).npt[2]!=(*dx2).npt[2])||
        ((*dx1).npt[3]!=(*dx2).npt[3]))
           {WARN("DX-files do not have the same dimensions or are shifted");
            return 1;
           }

    /* assign the dimensions */
    (*ddx).ori[1]      = (*dx2).ori[1];
    (*ddx).ori[2]      = (*dx2).ori[2];
    (*ddx).ori[3]      = (*dx2).ori[3];
    (*ddx).delta[1][1] = (*dx2).delta[1][1];
    (*ddx).delta[1][2] = (*dx2).delta[1][2];
    (*ddx).delta[1][3] = (*dx2).delta[1][3];
    (*ddx).delta[2][1] = (*dx2).delta[2][1];
    (*ddx).delta[2][2] = (*dx2).delta[2][2];
    (*ddx).delta[2][3] = (*dx2).delta[2][3];
    (*ddx).delta[3][1] = (*dx2).delta[3][1];
    (*ddx).delta[3][2] = (*dx2).delta[3][2];
    (*ddx).delta[3][3] = (*dx2).delta[3][3];
    (*ddx).npt[1]      = (*dx2).npt[1];
    (*ddx).npt[2]      = (*dx2).npt[2];
    (*ddx).npt[3]      = (*dx2).npt[3];

    /* calculate the difference */
    ihelp = (*ddx).npt[1]*(*ddx).npt[2]*(*ddx).npt[3];
    (*ddx).data = dvector(1,ihelp+3);

    for (i=1;i<=ihelp; i++)
        (*ddx).data[i] = (*dx2).data[i] - (*dx1).data[i];

    return 0;
}
/*..............................................................*/
/* write a DX to a file                                         */
/*                                                              */
void WriteDX(FILE *fp, OpenDX *dx, const char *info)
{
    int ih, ihelp,ic;

    /* the heading */
    fprintf(fp,"# %s\n", info);
    fprintf(fp,"#\n");
    fprintf(fp,"object 1 class gridpositions counts %d %d %d\n",
           (*dx).npt[1], (*dx).npt[2], (*dx).npt[3]);
    fprintf(fp,"origin  %e %e %e\n",
           (*dx).ori[1],(*dx).ori[2],(*dx).ori[3]);
    fprintf(fp,"delta  %e %e %e\n",
           (*dx).delta[1][1],(*dx).delta[1][2],(*dx).delta[1][3]);
    fprintf(fp,"delta  %e %e %e\n",
           (*dx).delta[2][1],(*dx).delta[2][2],(*dx).delta[2][3]);
    fprintf(fp,"delta  %e %e %e\n",
           (*dx).delta[3][1],(*dx).delta[3][2],(*dx).delta[3][3]);
    fprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
           (*dx).npt[1], (*dx).npt[2], (*dx).npt[3]);
    fprintf(fp,"object 3 class array type double rank 0 items %d data follows\n",
           (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3]);

    /* now the data */
    ihelp = (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3];
    ih = 0;
    ic = 0;
    while (ic<ihelp)
        {ic++; ih++;
         fprintf(fp," %13.5E",(*dx).data[ic]);
         if (ih==3)
            {fprintf(fp,"\n");
             ih=0;
            }
        }
    if (ih!=0)  fprintf(fp,"\n");

    /* now the tail */
    fprintf(fp,"attribute \"dep\" string \"positions\"\n");
    fprintf(fp,"object \"regular positions regular connections\" class field\n");
    fprintf(fp,"component \"positions\" value 1\n");
    fprintf(fp,"component \"connections\" value 2\n");
    fprintf(fp,"component \"data\" value 3\n");

    
}
/*..............................................................*/
/* write a DX to a file                                         */
/*                                                              */
void gzWriteDX(char *fname, OpenDX *dx, const char *info)
{
    int ih, ihelp,ic;
    gzFile fp;

    fp = gzopen(fname,"w");

    /* the heading */
    gzprintf(fp,"# %s\n", info);
    gzprintf(fp,"#\n");
    gzprintf(fp,"object 1 class gridpositions counts %d %d %d\n",
           (*dx).npt[1], (*dx).npt[2], (*dx).npt[3]);
    gzprintf(fp,"origin  %e %e %e\n",
           (*dx).ori[1],(*dx).ori[2],(*dx).ori[3]);
    gzprintf(fp,"delta  %e %e %e\n",
           (*dx).delta[1][1],(*dx).delta[1][2],(*dx).delta[1][3]);
    gzprintf(fp,"delta  %e %e %e\n",
           (*dx).delta[2][1],(*dx).delta[2][2],(*dx).delta[2][3]);
    gzprintf(fp,"delta  %e %e %e\n",
           (*dx).delta[3][1],(*dx).delta[3][2],(*dx).delta[3][3]);
    gzprintf(fp,"object 2 class gridconnections counts %d %d %d\n",
           (*dx).npt[1], (*dx).npt[2], (*dx).npt[3]);
    gzprintf(fp,"object 3 class array type double rank 0 items %d data follows\n",
           (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3]);

    /* now the data */
    ihelp = (*dx).npt[1]*(*dx).npt[2]*(*dx).npt[3];
    ih = 0;
    ic = 0;
    while (ic<ihelp)
        {ic++; ih++;
         gzprintf(fp," %13.5E",(*dx).data[ic]);
         if (ih==3)
            {gzprintf(fp,"\n");
             ih=0;
            }
        }
    if (ih!=0)  gzprintf(fp,"\n");

    /* now the tail */
    gzprintf(fp,"attribute \"dep\" string \"positions\"\n");
    gzprintf(fp,"object \"regular positions regular connections\" class field\n");
    gzprintf(fp,"component \"positions\" value 1\n");
    gzprintf(fp,"component \"connections\" value 2\n");
    gzprintf(fp,"component \"data\" value 3\n");

    gzclose(fp);      
}
