/* Matthias Ullmann, 2016                               */
/*                                                      */
/* PSE -- Periodic System of Elements                   */
/*......................................................*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mol.h"
#include "util.h"
#include "memutil.h"

#define NR_END 1
#define FREE_ARG void*


/****************************************************************/
/*              Memory Allocation                               */

void AllocateElements(Element *e)
{
     (*e).name   = charmatrix(1, (*e).nele, 0, 5);
     (*e).odnr   = ivector(1, (*e).nele);
     (*e).mass   = dvector(1, (*e).nele);
     (*e).covrad = dvector(1, (*e).nele);
     (*e).VdWrad = dvector(1, (*e).nele);

}

/*..............................................................*/
void FreeElements(Element *e)
{
     free_charmatrix((*e).name, 1, (*e).nele, 0, 9);
     free_ivector((*e).odnr   , 1, (*e).nele);
     free_dvector((*e).mass   , 1, (*e).nele);
     free_dvector((*e).covrad , 1, (*e).nele);
     free_dvector((*e).VdWrad , 1, (*e).nele);

}

/****************************************************************/
/* Reading data                                                 */
/*..............................................................*/

void read_PSE(char *filename, Element *e)
{
   FILE *fpin;
   char line[LINELENGTH];
   int n=0, i=0;

   fpin=file_open(filename,"r");

   /* count the elements and allocate the memory */
   n=0;
   while (fgets(line,LINELENGTH,fpin)!=NULL)
     if (strncmp(line,"Element",7)==0) n++;

   (*e).nele = n;
   AllocateElements(e);

   rewind(fpin);

   /* read the file */
   i=0;
   while (fgets(line,LINELENGTH,fpin)!=NULL)
         {if (strncmp(line,"Element",7)==0) 
	     {i++;
	      if (5!=sscanf(line ,"%*s %s %d %lf %lf %lf", 
			      (*e).name[i],
			     &(*e).odnr[i],
			     &(*e).mass[i],
			     &(*e).covrad[i],
			     &(*e).VdWrad[i]))
		muerror("Error while reading file in read_PSE()");
	     }
	 }
   fclose(fpin);

   if (i!=n) muerror("Problem in read_PSE()");

}

/* .......................................................... */
void fprint_PSE(FILE *fp, Element *e)
{

  int n;

  for (n=1; n<=(*e).nele; n++)
      {fprintf(fp ,"%s %-2s %3d %8.3f %8.3f %8.3f\n", 
	       "Element",
	       (*e).name[n],
	       (*e).odnr[n],
	       (*e).mass[n],
	       (*e).covrad[n],
	       (*e).VdWrad[n]);
      }

}
