/* Matthias Ullmann, 08.01.1996						*/
/*									*/
/* sidechain torsion							*/
/* To move the side chains of a protein with a simple MC Method.	*/
/*									*/
/* .................................................................... */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "util.h"
#include "memutil.h"
#include "sc-torsion.h"
#include "mathematik.h"

#define FREE_ARG void*
/* .................................................................... */
/* Make a torsion of the sidechain around a bond			*/
/* the result is directly written into the mus, sp if the procedure is	*/
/* used in a MC simulation, the old conformation should have been save 	*/
/* prior								*/
/*									*/
void rotate_around_bond(Torsion *tors, double angle, MUS_atom *mus)
{
double *central;
double a[4], v[4], s[4], t[4];
double scal_prod, sin_theta, cos_theta;
int i;

/* Do some initial calculations */
sin_theta=sin(angle);
cos_theta=cos(angle);
central=(*mus).xyz[(*tors).tors_def[2]];

/* a=Unit vector in the direction of the rotation axis */
SubVector((*mus).xyz[(*tors).tors_def[3]],(*mus).xyz[(*tors).tors_def[2]],a);
scal_prod=1.0/LENGTH(a);
ScaleVector(a, scal_prod, a);

/* Now do the rotation */
for (i=1; i<=(*tors).move_no; i++)
    {SubVector((*mus).xyz[(*tors).id_move[i]],central,v);
     scal_prod=ScalarProduct(a,v);
     VectorProduct(v,a,s);
     t[1]=v[1]-scal_prod*a[1];
     t[2]=v[2]-scal_prod*a[2];
     t[3]=v[3]-scal_prod*a[3];
     v[1]=scal_prod*a[1] + cos_theta*t[1] + sin_theta*s[1];
     v[2]=scal_prod*a[2] + cos_theta*t[2] + sin_theta*s[2];
     v[3]=scal_prod*a[3] + cos_theta*t[3] + sin_theta*s[3];
     (*mus).xyz[(*tors).id_move[i]][1]=central[1]+v[1];
     (*mus).xyz[(*tors).id_move[i]][2]=central[2]+v[2];
     (*mus).xyz[(*tors).id_move[i]][3]=central[3]+v[3];
    }
}
/* .................................................................... */
/* Accepting the new conformation which was created in 			*/
/* rotate_around_bond()							*/
/*									*/
void accept_new_conformation(MUS_atom *mus, Torsion *tors, double **result)
{
int i;
for (i=1; i<=(*tors).move_no; i++)
    {(*mus).xyz[(*tors).id_move[i]][1]=result[i][1]; 
     (*mus).xyz[(*tors).id_move[i]][2]=result[i][2]; 
     (*mus).xyz[(*tors).id_move[i]][3]=result[i][3];
     /*fprintf(stdout,"%d %lf  %lf  %lf\n",(*mus).atomno[(*tors).id_move[i]], 
					result[i][1],result[i][2],result[i][3]); */
    }
} 
/*......................................................................*/
/* allocate a TorsDef vector with subscript range v[nl..nh] 		*/
TorsDef *TorsDef_vector(long nl, long nh)
{
        TorsDef *v;

	v=(TorsDef *)malloc((size_t) ((nh-nl+1+1)*sizeof(TorsDef)));
	if (!v) ERROR("allocation failure in TorsDefvector()");
	return v-nl+1;
}
/*......................................................*/
/* free a char vector with subscript range v[nl..nh]    */
void free_TorsDefvector(TorsDef *v, long nl, long nh)
{
if (v!=NULL)	free((FREE_ARG) (v+nl-1));
nh=nh+1;
}
/*......................................................................*/
/* allocate a Torsion vector with subscript range v[nl..nh] 		*/
Torsion *Torsion_vector(long nl, long nh)
{
        Torsion *v;

	v=(Torsion *)malloc((size_t) ((nh-nl+1+1)*sizeof(Torsion)));
	if (!v) ERROR("allocation failure in Torsion_vector()");
	return v-nl+1;
}


/* .................................................................... */
/* reads the Torsion definition file and allocates and fills a TorsDef- */
/* structure-array, the length of the array is given back as argument of*/
/* the structure	       						*/
/*									*/
TorsDef *read_TorsDef(char *filename, int *length)
{
char *help, line[LINELENGTH];
FILE *fpin;
TorsDef *tors, *help_tors;
int count=0,len1,len2, len3,len4,i, j;

len1=(int)strlen(KEYWORD1);
len2=(int)strlen(KEYWORD2);
len3=(int)strlen(KEYWORD3);
len4=(int)strlen(KEYWORD4);
fpin=file_open(filename,"r");

/* Count the number of entries */
while (fgets(line,LINELENGTH,fpin)!=NULL)
      {if (0==strncmp(KEYWORD4,line,len1)) {break;}
       if (0==strncmp(KEYWORD1,line,len1)) {count++;}
      }

/* Now allocating the TorsDef-Vector */
*length=count; count=0;
tors=TorsDef_vector(1,*length);
rewind(fpin);
help_tors=tors;

/* Now reading the file */
while (fgets(line,LINELENGTH,fpin)!=NULL)
    {if (0==strncmp(KEYWORD4,line,len4)) {break;}
     if (0==strncmp(KEYWORD1,line,len1)) 
	{tors++;
	 sscanf(line,"%*s %s %d",(*tors).resname, &(*tors).tors);
	 replace_char((*tors).resname,' ',SPACE);
	 (*tors).tors_def=char_3tensor(1,(*tors).tors,1,4,0,5);  /* Allocation */
	 (*tors).no_move =ivector(1,(*tors).tors);
	 (*tors).move=char_3tensor(1,(*tors).tors,1,MAX_MOVE,0,5);
	 for (j=1;j<=(*tors).tors;j++)
	     {READ_LINE(line,fpin);
	      if (0!=strncmp(KEYWORD2,line,len2))
		 {WARN("Format error in torsion file 1 (read_TorsDef())");
		  fprintf(STDERR,"%s\n",line);
		 }
	      else {help=line+len2;
	            help=read_word((*tors).tors_def[j][1],help);
	            replace_char((*tors).tors_def[j][1],' ',SPACE);
	            help=read_word((*tors).tors_def[j][2],help);
	            replace_char((*tors).tors_def[j][2],' ',SPACE);
	            help=read_word((*tors).tors_def[j][3],help);
	            replace_char((*tors).tors_def[j][3],' ',SPACE);
	            help=read_word((*tors).tors_def[j][4],help);
	            replace_char((*tors).tors_def[j][4],' ',SPACE);
		   }
	      READ_LINE(line,fpin);
              if (0!=strncmp(KEYWORD3,line,len3))
		 {WARN("Format error in torsion file 2 (read_TorsDef())");
		  fprintf(STDERR,"%s\n",line);
		 }
	      else {(*tors).no_move[j]=StringPerLine(line)-1;
		    if ((*tors).no_move[j]>=MAX_MOVE) 
		       {ERROR("Set MAX_MOVE in sc-torsion.h to a higher value!! (read_TorsDef())");
		        fprintf(STDERR,"%s\n",line);
		       }
	            help=line+len3;
	            for (i=1; i<=(*tors).no_move[j];i++)
		        {help=read_word((*tors).move[j][i],help);
		         replace_char((*tors).move[j][i],' ',SPACE);
		        }
		   }
	    }
	       
	 }
    }
/* ============= CHANGE =====================*/
/*tors=help_tors;
for (i=1; i<=*length;i++)
    {tors++;
     fprintf(stdout,"RESIDUE  %s  %d\n",(*tors).resname, (*tors).tors);
     for(j=1;j<=(*tors).tors;j++)
        {fprintf(stdout,"TORSION  '%s' '%s' '%s' '%s' \t %d\n",
				(*tors).tors_def[j][1],
				(*tors).tors_def[j][2],
				(*tors).tors_def[j][3],
				(*tors).tors_def[j][4],
				(*tors).no_move[j]);
	 fprintf(stdout,"MOVE  ");
	 for (k=1;k<=(*tors).no_move[j];k++)
	     {fprintf(stdout,"'%s' ",(*tors).move[j][k]);
	     }
	 fprintf(stdout,"\n");
	}
     fprintf(stdout,"\n\n\n");
    }*/
/* ============= CHANGE =====================*/

fclose(fpin);

tors=help_tors;
return tors;
}
/* .................................................................... */
/* Routine assignes the torsion angle to the molecule of interest	*/
/* length - length of TorsDef						*/
/* tors_no - number of defined torsions in the molecule			*/
/*									*/
Torsion *assign_torsions(MUS_atom *mus, TorsDef *tors_def, int length, int *tors_no)
{
int i,j, k=0,l;
int s; /* start of the aminoacid */
TorsDef *help;
Torsion *tors, *tors_help;

/* first counting the torsions in the protein */
for (i=1; i<=(*mus).aminoacids; i++)
    {help=tors_def;
     s=(*mus).aa[i][1];
     for (j=1; j<=length; j++)
	{help++; /* tors_def start at 1 */
         if(0==strcmp((*mus).residue[s],(*help).resname))
	   {k=k+(*help).tors;
	   }
	}
    }
/* allocating */
*tors_no=k;
tors=Torsion_vector(1, *tors_no);
tors_help=tors;
/* Assignment */
tors++; /* to start at 1 */
for (i=1; i<=(*mus).aminoacids; i++)
    {help=tors_def;
     s=(*mus).aa[i][1];
     for (j=1; j<=length; j++)
	{help++; /* tors_def start at 1 */
         if(0==strcmp((*mus).residue[s],(*help).resname))
	   {for (k=1; k<=(*help).tors; k++)
		{(*tors).move_no=(*help).no_move[k];
		 (*tors).tors_def=ivector(1,4);
		 (*tors).id_move =ivector(1,(*tors).move_no);
		 for (l=1; l<=4; l++)
		     {(*tors).tors_def[l]=get_no(mus,i,(*help).tors_def[k][l]);
		     }
		 for (l=1; l<=(*help).no_move[k]; l++)
		     {(*tors).id_move[l] =get_no(mus,i,(*help).move[k][l]);
		     }
		 tors++;
		}
	   }
	}
    }

/* looking how many torsions energy terms are effected and allocate id_tors */
tors=tors_help;
for (i=1; i<=*tors_no; i++)
    {tors++; /* to start at 1 */
     k=0;
     for (j=1; j<=(*mus).number_of_torsions; j++)
	 {if (  (((*mus).torsion[j][2]==(*tors).tors_def[2])
	       &&((*mus).torsion[j][3]==(*tors).tors_def[3]))
	      ||(((*mus).torsion[j][3]==(*tors).tors_def[2])
	       &&((*mus).torsion[j][2]==(*tors).tors_def[3])))
	      {k++;
	      }
	 }
     (*tors).tors_no=k;
     (*tors).id_tors=ivector(1,(*tors).tors_no);
    }
/* looking which torsions energy terms are effected and assign id_tors */
tors=tors_help;
for (i=1; i<=*tors_no; i++)
    {tors++; /* to start at 1 */
     k=0;
     for (j=1; j<=(*mus).number_of_torsions; j++)
	 {if (  (((*mus).torsion[j][2]==(*tors).tors_def[2])
	       &&((*mus).torsion[j][3]==(*tors).tors_def[3]))
	      ||(((*mus).torsion[j][3]==(*tors).tors_def[2])
	       &&((*mus).torsion[j][2]==(*tors).tors_def[3])))
	      {k++;
	       (*tors).id_tors[k]=j;
	      }
	 }
    }

/* RETURN */
return tors_help;
}

/* .................................................................... */
/* Fills the TorsAdmin-Structure					*/
void fill_TorsAdmin(TorsAdmin *admin, MUS_atom *mus, char *filename)
{
TorsDef *def, *help;
Torsion *tors;
int i,j,length, tors_no, max,aa;

/* Read the torsion definition file */
def=read_TorsDef(filename, &length);
/* searching for the maximal number of torsions per residue */
max=0; help=def;
for (i=1; i<=length; i++)
    {help++;
     if (max < (*help).tors) {max=(*help).tors;}
    }

/* do the settings */
(*admin).max_tors_per_res=max;
(*admin).tors_in_res=imatrix(1,(*mus).aminoacids, 1,(*admin).max_tors_per_res);
(*admin).tors=assign_torsions(mus, def, length, &tors_no);
(*admin).max_tors_no=tors_no;
(*admin).tors_no=0;
(*admin).flexible=charvector(1,(*mus).aminoacids);
(*admin).flexible_test=charvector(1,(*mus).aminoacids);
(*admin).rotateable=ivector(1,(*admin).max_tors_no);

/* initialize of some vectors and matrices */
for (i=1;i<=(*mus).aminoacids; i++) 		{(*admin).flexible[i]=UNFLEX;}
for (i=1;i<=(*mus).aminoacids; i++) 		{(*admin).flexible_test[i]=UNFLEX;}
for (i=1;i<=(*admin).max_tors_per_res;i++)	{(*admin).rotateable[i]=0;}

for (i=1;i<=(*mus).aminoacids; i++)
    for (j=1;j<=(*admin).max_tors_per_res;j++)
	(*admin).tors_in_res[i][j]=0;

/* Now look which torsion is in which aa */
tors=(*admin).tors;
for (i=1; i<=(*admin).max_tors_no; i++)
    {tors++;
     if (  ((*mus).aa_no[(*tors).tors_def[1]]!=(*mus).aa_no[(*tors).tors_def[2]])
	 ||((*mus).aa_no[(*tors).tors_def[1]]!=(*mus).aa_no[(*tors).tors_def[3]])
	 ||((*mus).aa_no[(*tors).tors_def[1]]!=(*mus).aa_no[(*tors).tors_def[4]]))
	{WARN("Torsion is not within one residue. Use only tors_def[1] for assignemt. (fill_TorsAdmin())");
	 fprint_PDBatom(STDERR,mus,(*tors).tors_def[1]);
	 fprint_PDBatom(STDERR,mus,(*tors).tors_def[2]);
	 fprint_PDBatom(STDERR,mus,(*tors).tors_def[3]);
	 fprint_PDBatom(STDERR,mus,(*tors).tors_def[4]);
	}
     aa=(*mus).aa_no[(*tors).tors_def[1]];
     j=1;
     while ((*admin).tors_in_res[aa][j]!=0) j++;
     (*admin).tors_in_res[aa][j]=i;
    }
/* Looking to which aminoacids no torsion is assigned */
for (i=1; i<=(*mus).aminoacids; i++)
    {if (0==(*admin).tors_in_res[i][1])
	{fprintf(STDERR,"No torsion assigned to the following residue.\n");
	 fprint_aminoacid(STDERR, mus, i);
         fprintf(STDERR,"\n");
	}
    }
     
/* frees the allocated memory of def */
help=def;
for (i=1; i<=length; i++)
    {help++;
     free_char3tensor((*help).tors_def, 1, (long)(*help).tors, 1,4,0,5);
     free_ivector((*help).no_move, 1, (long)(*help).tors);
     free_char3tensor((*help).move, 1, (long)(*help).tors, 1,MAX_MOVE,0,5);
    }
free_TorsDefvector(def,1,(long)length);
} 
/* .................................................................... */
/* Fills the rotateable-vector						*/
/* The Torsions of the residues with a FLEX in the flexible record are	*/
/* written to the rotatable-vector					*/
/*									*/
void fill_rotateable(TorsAdmin *admin, int number_aa)
{
int i,j, count=0;

for (i=1; i<=number_aa; i++)
    {if ((*admin).flexible[i]==FLEX)
	{j=1;
	 while ((*admin).tors_in_res[i][j]!=0)
	     {if (j>(*admin).max_tors_per_res) {break;}
	      count++;
	      (*admin).rotateable[count]=(*admin).tors_in_res[i][j];
	      j++;
	     }
	}
    }
if (0==count)
   {WARN("No rotateable Torsions in Interface!");
   }
(*admin).tors_no=count;
}
/* .................................................................... */
/* Returns the maximal number of movable atoms per torsion		*/
int max_moveable_atoms(Torsion *tors, int length)
{
int i,max=0;

for (i=1; i<=length; i++)
    {tors++;
     if (max<(*tors).move_no) max=(*tors).move_no;
    }
return max;
}

/* .................................................................... */
/* Prints the numbers in a Torsion-vector of the length n		*/
/* tors[0] is not printed!!						*/
/*									*/
void fprint_Torsion(FILE *fpout, Torsion *tors, int n)
{
int i,j;
for(i=1; i<=n; i++)
   {tors++;
    fprintf(fpout,"TORSION  '%d' '%d' '%d' '%d'\n",
				(*tors).tors_def[1],
				(*tors).tors_def[2],
				(*tors).tors_def[3],
				(*tors).tors_def[4]);
    fprintf(stdout,"MOVE  %d: ",(*tors).move_no);
    for (j=1;j<=(*tors).move_no;j++)
	     {fprintf(stdout,"'%d' ",(*tors).id_move[j]);
	     }
     fprintf(stdout,"\n\n");
    }
}
