/* Matthias Ullmann, 08.01.1996						*/
/*									*/
/* sidechain torsion							*/
/* To move the side chains of a protein with a simple MC Method.	*/
/*									*/
/* .................................................................... */
#ifndef __sc_torsion__
#define __sc_torsion__
#include "mol.h"
//#include "properties.h"



/****************************************************************/
/* Internal representation of torsions				*/
typedef struct {int resno;  	/* id number of the residue 	*/
		int *tors_def;	/* ID number of the atoms, which
				   define the torsion [1..4]	*/
		int move_no;	/* number of atoms to move	*/
		int *id_move;	/* id number of the atoms to	
				   move [1...move_no]		*/
		int tors_no;	/* number of the effected torsion
				   energy terms			*/
		int *id_tors;	/* id number of the effected
				   torsion energy terms		*/
	       }Torsion;

/****************************************************************/
/* Structure for reading the residues from a input file and	*/
/* to create the internal representation of the torsion		*/
/*								*/
#define SPACE '-'
#define MAX_MOVE 20 	/* Maximum number of atoms to move */
typedef struct {char 	resname[4]; 	/* name of the residue 		*/
		int	tors;		/* number of torsions in this	*/
					/* residue			*/
		char	***tors_def;	/* name of the atoms which 	*/
					/* define the torsion  		*/
		char	***move;	/* name of the atoms which must	*/
					/* be moved			*/
		int	*no_move;	/* number of atoms to move	*/
	       }TorsDef;
#define SPACE '-'
#define KEYWORD1 "RESIDUE"
#define KEYWORD2 "TORSION"
#define KEYWORD3 "MOVE"
#define KEYWORD4 "END"
/****************************************************************/
/* Administration of the torsion				*/
#define UNFLEX  0 
#define FLEX	1 
typedef struct {int 	**tors_in_res; 	/* which residue includes which torsions*/
					/* [1..n-res][1..max_tors_per_res]	*/
		int 	max_tors_per_res;/* mamimal number of per residue	*/
		int 	max_tors_no;	/* maximal number of torsions		*/
		int 	tors_no;	/* actual number of flexible torsions 	*/
		char 	*flexible;	/* flag vector [1..n-res]		*/
		char	*flexible_test; /* for MC testing			*/
		Torsion *tors;		/* definition of the torsions		*/
		int	*rotateable;	/* includes the id number of flexible 	*/
					/* torsions [1..maxtors_no], used only	*/
					/* to tors_no - sometimes updated	*/
	       }TorsAdmin;
/****************************************************************/
/* Function declaration						*/
void rotate_around_bond(Torsion *tors, double angle, MUS_atom *mus);
void accept_new_conformation(MUS_atom *mus, Torsion *tors, double **result);

TorsDef *TorsDef_vector(long nl, long nh);
Torsion *Torsion_vector(long nl, long nh);

TorsDef *read_TorsDef(char *filename, int *length);

Torsion *assign_torsions(MUS_atom *mus, TorsDef *tors_def, int length, int *tors_no);
void fill_TorsAdmin(TorsAdmin *admin, MUS_atom *mus, char *filename);
void fill_rotateable(TorsAdmin *admin, int number_aa);
int max_moveable_atoms(Torsion *tors, int length);
void fprint_Torsion(FILE *fpout, Torsion *tors, int n);
#endif /* __sc_torsion__ */
 
