/* Matthias Ullmann, Berlin 15.01.1995					*/
/* 									*/
/* routines for surface and volume calculation 				*/
/* 									*/
/* method by F. Eisenhaber:						*/
/* F.Eisenhaber, P.Lijnzaad, P.Argos, M.Scharf				*/
/*    "The Double Cubic Lattice Method: Efficient Approaches to		*/
/*    Numerical Integration of Surface Area and Volume and to Dot	*/
/*    Surface Contouring of Molecular Assemblies"			*/
/*    Journal of Computational Chemistry (1995) v.16, N3, pp.273-284	*/
/*									*/
/* .................................................................... */
#ifndef __surface__
#define __surface__

#include "nb-list.h"
#include "graph.h"

#define DP_TOL     0.00001

#define UNSP_ICO_DOD      1	/* i.e. class I  tessellation */
#define UNSP_ICO_ARC      2	/* i.e. class II tessellation */
#define UNSP_RANDOM       3	/* i.e. random   tessellation */

/* does the dot-counting surface calculation */
#define Area(acc,dots,r)	(4.0*M_PI*(r)*(r)*(((double)(acc))/((double)dots)))

/* Dots for surface calculation */
#define OUTSIDE 0	/* dot on the surface 		      */
#define INSIDE  1	/* dots inside the protein	      */
typedef struct {int    n_dot;
		double **xus;
		int    cubes;	/* odd integer	(3 or 5)      */
		double spacing;
		double inv_spacing;
		int    *imax;
		int    *imin;
		int    nmax;	/* max no of dots per cube    */
		int    ****dot_no;
	       }Surf;

typedef struct {int    ndots;
		double **dots;
		double *area;
	        char **idstring;
               }DotSurf;

/* The array dot_no is a 3D grid. Each grid point is the center of a	*/
/* box on a 3D grid. To each grid point a vector of counter numbers 	*/
/* (pointers to the array) is assigned					*/
/* In the field dot_no[i][j][k][0], the length of the vector saved at	*/
/* this grid point is given.						*/
/*									*/
 
/* .................................................................... */
/* Function declaration							*/
/*									*/
/* .................................................................... */
/* tesselation of a unit sphere						*/
/*int  tessellation_type(int densit);*/
void ico_dot_arc(int densit, Surf *surf); 
void ico_dot_dod(int densit, Surf *surf); 
void random_tess(int densit, Surf *surf, long *seed); 

/* .................................................................... */
/* sorting the dots into a grid						*/
void unitsphere_to_grid(Surf *surf);


/* .................................................................... */
/* do the surface calculation and the surface updates			*/
/*
double calculate_surface(double *area, char **flag, double **xyz, 
			 double *radius, int natom, Surf *surf, 
			 iListGrid *grid, iList *list);
double calculate_area(int no, char **flag, double **xyz, double *radius,
		      Surf *surf, iListGrid *grid);
void calculate_dotsurface(DotSurf *dotsurf, char **flag, Surf *surf,
			  double **xyz, double *radius, int natoms);
*/
double surface(double *atomsurf, double **xyz, double *radius, 
               int natoms, int densit, double prob, DotSurf *dotsurf,
               int surfflag, char **idstring);
double DotSurface(double *atomsurf, double **xyz, double *radius, 
                  int natoms, int densit, double prob, DotSurf *dotsurf,
		  int surfflag, int itest, long *seed, FILE *log, char **idstring);

void ReadSrfPts(DotSurf *dotsurf, char *fname);
void ConnectPointsMatrix(int **graph, DotSurf *dotsurf, double distlimit);
void ConnectPointsList(Node *adj, DotSurf *dotsurf, double distlimit);
#endif  /* __surface __ */



