/* G. Matthias Ullmann                                          */
/* Bayreuth, 3.10.2010                                          */
/*                                                              */
/* calculated energy:                                           */
/* it reads a dx-file and a pqr-file and multiplies the charges */
/* with the potential it is assumed that the potential is given */
/* in kcal/mol/e                                                */
/*                                                              */
/* ............................................................ */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <zlib.h>
#include "mol.h"
#include "memutil.h"
#include "util.h"
#include "surface.h"
#include "random.h"
#include "exclude.h"
#include "grid.h"
#include "dock-help.h"

#define USAGE \
"usage: calc-energy [pot-receptor.dx] [ligand.pqr] [excl-grid] \n\n\
Program calculates the energy and checks for exclusion (optionally).\n\
The potential should be in kcal/mol/e.\n"

/**********************************************************/
int main(int argc, char *argv[])
{
    /* Varialbles */
    Mol protein;
    dGrid phi;
    OpenDX dxpot;
    FILE *fp;
    cGrid excl;

    double ener;
    double prob=1.4;
    int    densit=200; 
    DotSurf dotsurf;
 
    /* Usage */
    if (argc<=2)
       {fprintf(stderr,USAGE);
        exit(10);
       }

    /* read the data */
    fp = file_open(argv[1],"r");
    ReadDX(&dxpot,fp);
    DX2Grid(&phi, &dxpot);
    dGrid_info(stdout, &phi,"Potential Grid");
    fclose(fp);     

    read_PQRM(argv[2], &protein);
    surface(protein.surface, protein.xyz, 
                 protein.radius, protein.natoms, 
                 densit, prob, &dotsurf, 1, protein.idstring);
    on_surf(&protein);

    if (argc==4)
       {fp = file_open(argv[3],"r");
        fread_excl_binary(fp,&excl);     
        fclose(fp);     
        cGrid_info(stdout, &excl,"Exclusion Grid");
	if (exclude(&excl, protein.xyz,
                    protein.onsurf, protein.n_onsurf)!=0)
	  {fprintf(stdout,"\nLigand within the exclusion grid\n");
	  }
	else {fprintf(stdout,"\nLigand not within the exclusion grid\n");
	     }
       }                     

    /* calculate the energy */
    ener=electrostatic_grid_energy(&phi, protein.xyz,
                       protein.charge, protein.natoms);
    fprintf(stdout, "\nEnergy: %e kcal/mol\n",ener);

    return 0;
}

