/* G. Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* Large changes 31.10.1996				*/
/* .....................................................*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include "zlib.h"
#include "my-kabsch.h"
#include "random.h"
#include "util.h"
#include "memutil.h"
#include "mol.h"
#include "constants.h"
#include "mathematik.h"
#include "properties.h"
#include "energy.h"
#include "dock-help.h"
#include "exclude.h"
#include "surface.h"

/* .....................................................*/
/*                                                      */
void GetProtein(FILE *fp, char *name, Mol *p)
{
  int i;
  double prob=1.4;
  int    densit=255; 
  DotSurf dotsurf;
  
  read_PQRM(name, p);
  fprintf(fp,"Read Protein File: %s\n", name);
  AssignIDString(p);
  surface((*p).surface, (*p).xyz, (*p).radius, (*p).natoms, 
          densit, prob, &dotsurf, 1, (*p).idstring);
  on_surf(p);
  save_initial_coordinates(p);
  properties(p); 
  fprint_properties(fp,p);
  fprintf(fp,"\n");
  //fprintf(stdout,"I set the temperature factor vector to 1.0\n");
  for (i=1; i<=(*p).natoms; i++)
      {(*p).bval[i]=1.0;
      }

}
/*..............................................................*/
/* Calculates the transformation and prints it to a file. The	*/
/* 'temperature' vector of mus is set to one and used as weight	*/ 
/*								*/
void WriteOrient(gzFile fpout, double energy, Mol *mus)
{
    double center[4], trans[4];
    double **rot_matrix;
    double rms_value;

    rot_matrix=dmatrix(1,3,1,3);

    //rms_value=kabsch(rot_matrix, center, trans, (*mus).save_xyz, (*mus).xyz,
    //		 (*mus).number_of_atoms, (*mus).temperature);
    /* to save CPU time use only a few atoms for "kabsching" */
    rms_value=kabsch(rot_matrix, center, trans, (*mus).save_xyz, (*mus).xyz,
                     10, (*mus).bval);

    if (0.001 < fabs(rms_value))
        {muwarn("Something wrong in fprint_trans()");
         fprintf(stdout,"RMS value: %f\n",rms_value);
        }

    gzprintf(fpout,"ORIENTATION %E %E %E %E %E %E %E %E %E %E %E %E %E %E %E %E\n",
	       energy,
	       trans[1],trans[2],trans[3], 
	       center[1],center[2],center[3], 
	       rot_matrix[1][1],rot_matrix[1][2],rot_matrix[1][3], 
	       rot_matrix[2][1],rot_matrix[2][2],rot_matrix[2][3], 
	       rot_matrix[3][1],rot_matrix[3][2],rot_matrix[3][3]);


    free_dmatrix(rot_matrix,1,3,1,3);
}
/* .....................................................*/
/* Procedure makes a complete MC run including the	*/
/* annealing procedure					*/
/*							*/
double mc_run(dGrid *phi_out, dGrid *phi_in, cGrid *excl, 
              Mol *protein, Run_para *para, iGrid *dens)
{
    int back,is=0, iprint = 0, iprintdist=0, icountback=0, iprintdens=0;
    int i,j,k;
    double ener, len;
    BookKeep glob, book;

    glob.tried     =0; glob.unsuccess =0; glob.accepted=0;
    glob.rejected  =0; glob.excluded  =0; glob.reinit  =0;

    /* make an inital orientation */
    ener=init_orientation(protein, (*para).rad_in, 
                          phi_out, &(*para).seed, (*para).start);

    book.tried     =0; book.unsuccess =0; book.accepted=0;
    book.rejected  =0; book.excluded  =0; book.reinit  =0;
    while (book.tried<(*para).steps)
             {/* make a single MC move including a Metropolis test */
              back=mc_move(&ener, phi_out, phi_in, excl, protein, 
                           (*para).separation, (*para).dev_rot, 
                           (*para).dev_trans, 
                           (*para).temp,&(*para).seed);
              /* avoid that the simulation gets stuck in a minimum */
              if ((back == 2)||(back == 1)) {icountback++;}
              else {icountback=0;}
              /* do some book keeping */
              keep_book(&book,back);
              len = LENGTH((*protein).cm);
              /* print trajectory */
              iprint++; /* write trajectory, if nprint not negative!*/
              if (iprint == (*para).nprint)
                  {WriteOrient((*para).traj, ener, protein);
                   iprint = 0;
                  }
              /* print coordinate if the distance of the CM is smaller 
                 than a certain value */
              if (len <= (*para).cmdist)
                  {iprintdist++;
                   if (iprintdist == (*para).nprintdist)
                      {WriteOrient((*para).structdist, ener, protein);
                       iprintdist=0;
                      }
                   iprintdens++;
                   if (iprintdens == (*para).nprintdens)
                       {iprintdens = 0;
                        i=(int)rint((*protein).cm[1]*(*dens).inv_spacing);
                        j=(int)rint((*protein).cm[2]*(*dens).inv_spacing);
                        k=(int)rint((*protein).cm[3]*(*dens).inv_spacing);
                        if (  (i<(*dens).imin[1]) || (i>(*dens).imax[1])
                            ||(j<(*dens).imin[2]) || (j>(*dens).imax[2])
                            ||(k<(*dens).imin[3]) || (k>(*dens).imax[3]))
                            {muwarn("CM outside the box");
                            }
                        else {if ((*dens).grid[i][j][k]<INT_MAX) 
                                  (*dens).grid[i][j][k]++;
                              else muerror("too many structures, counter problem");
                             }
                       }
                  }
              /* print coordinate if the energy is smaller 
                 than a certain value */
              if (ener <= (*para).low_energy)
                 {WriteOrient((*para).structene, ener, protein);
                 }
              /* test if the protein is still within the outer sphere */
              /* or got stuck. Reset in case       */
              if ((len >(*para).rad_out)||((*para).istuck==icountback))
                  {if ((*para).istuck==icountback) 
                      fprintf((*para).output,"reset molecule - got stuck\n");
                   icountback=0;
                   book.reinit++;
                   fprintf((*para).output,"reset molecule for the %ld time at %3.1f.\n",
                           book.reinit,(*para).temp);
                   ener=init_orientation(protein, (*para).rad_in, 
				         phi_out, &(*para).seed, (*para).start);
                  }
              if (book.unsuccess>(*para).max_unsuccess)
                  {fprintf((*para).output,"Too many unsuccessful steps\n");
                   glob.tried   =glob.tried   +book.tried; 
                   glob.accepted=glob.accepted+book.accepted;
                   glob.rejected=glob.rejected+book.rejected; 
                   glob.excluded=glob.excluded+book.excluded; 
                   glob.reinit  =glob.reinit  +book.reinit;
                   /*fprint_history((*para).output, &book);*/
                   if   (is!=1) /* stop this run */
                       {fprintf((*para).output,"I stop this run\n");
                        goto stop;
                       } 
                   else {fprintf((*para).output,"ia=1 - I go to the next step.\n");
                         break;
                        } /* continue this run at the next temperature */
                  }
             }
         /*fprint_history((*para).output, &book);*/
         glob.tried   =glob.tried   +book.tried; 
         glob.accepted=glob.accepted+book.accepted;
         glob.rejected=glob.rejected+book.rejected; 
         glob.excluded=glob.excluded+book.excluded; 
         glob.reinit  =glob.reinit  +book.reinit;
stop: /* LABEL */
    fprint_history((*para).output, &glob);
    return ener;
}
/****************************************************************/
/* Metropolis algorithm						*/
/* .....................................................*/
/* The procedure performs a Monte Carlo docking.	*/
/* Protein 2 is the moving protein			*/
/* The procedure returns                             
     0 if the move was accepted, 	
     1 if the move was rejected, or 
     2 if the move was excluded				*/
/*							*/
/* create two random vectors, one for the rotation and	*/
/* an other for the translation				*/
/* do the translation and rotation			*/
/* Now look for the energy and accept or reject the new */
/* orientaion 						*/
/* 1. Test if the exclusion grid must be tested.	*/
/* 2. If the two proteins overlap, the step is rejected */
/*    and the counter 'excluded' is increased by one.	*/
/*    Otherwise, the energy is calculated and valued	*/
/*    by the Metropolis criterion.			*/
/*							*/
/* The exclusion is not tested, as long as the 		*/
/* separation distance is too large. 			*/

int mc_move(double *ener, dGrid *phi_out, dGrid *phi_in, cGrid *excl, 
	    Mol *protein, double separation, double dev_rot, 
	    double dev_trans, double temp, long *seed)
{
    double random_rot[4];
    double random_trans[4];
    double delta;
    double new_distance;
    double new_energy;
    int back =5;

#define METROPOLIS(phi) \
	{new_energy=electrostatic_grid_energy(phi, (*protein).new_xyz,\
		       (*protein).charge, (*protein).natoms);\
         delta=(new_energy  - *ener)/(R_GAS*temp);\
         if (Metropolis(delta, seed)==1) \
	    {accept(protein);  *ener=new_energy; back=0;}\
         else {back=1;}\
	}

    /* Code */
    /* Metropolis */
    new_distance=LENGTH((*protein).new_cm);
    /* Random vectors */
    if (new_distance>=separation)
       {/* make large steps */
        random_vector(random_trans, dev_trans, seed);
        random_vector(random_rot  , dev_rot,   seed);
       }
    else {/* make small steps */
          random_vector(random_trans, 0.1*dev_trans, seed);
          random_vector(random_rot  , 0.1*dev_rot,   seed);
         }
    /* move */
    move((*protein).xyz, (*protein).natoms, 
         random_trans, (*protein).cm, random_rot, 
         (*protein).new_xyz, (*protein).new_cm); 
    /* Metropolis */
    new_distance=LENGTH((*protein).new_cm);
    if (new_distance>=separation)
        {METROPOLIS(phi_out);
        }
    else /* The exclusion is tested */
        {if (exclude(excl, (*protein).new_xyz,
                     (*protein).onsurf, (*protein).n_onsurf)==0)
            {METROPOLIS(phi_in);
            }
         else {back=2;
              }
	}

#undef METROPOLIS
return back;
}

/* .....................................................*/
/* accepts the new configuration 			*/
void accept(Mol *protein)
{
    double **help;
    double *help2;

    /* Code */
    help=(*protein).xyz;
    (*protein).xyz=(*protein).new_xyz; 
    (*protein).new_xyz=help;

    help2=(*protein).cm;
    (*protein).cm=(*protein).new_cm;
    (*protein).new_cm=help2;
}	        

/* ............................................................ */
/* The procedure creates an initial orientation of the 		*/
/* molecule. The molecule is shifed, so that the center of mass */
/* lies on a sphere with the radius 'radius'. Its relative 	*/
/* orientation is also changed, i.e. the molecule is rotated.   */
/*								*/
/* The initial orientation is simply done by creating a random 	*/
/* translation vector with components between -1 and +1 and 	*/
/* multiply this normalized vector by the sphere radius.	*/
/* 								*/

double init_orientation(Mol *protein, double rad, 
		        dGrid *phi, long *seed, int start)
{
    int i;
    double random_rot[4];	/* random rotation vector 	*/
    double random_trans[4];	/* random translation vector 	*/
    double energy, sum;

    /* restore the initial coordinates */
    for (i=1; i<=(*protein).natoms; i++)
	{(*protein).xyz[i][1]=(*protein).save_xyz[i][1];
	 (*protein).xyz[i][2]=(*protein).save_xyz[i][2];
	 (*protein).xyz[i][3]=(*protein).save_xyz[i][3];
	}
	 
    /* place molecule on the surface of a unit sphere, assuming that
       initially the center of mass was at the origin */
    if (start == 1)
       {/* creating a random vectors 		  */
        random_rot[1]=M_PIt2*Ran(seed);
        random_rot[2]=M_PIt2*Ran(seed);
        random_rot[3]=M_PIt2*Ran(seed);
        
        random_trans[1]=2*Ran(seed)-1.0;
        random_trans[2]=2*Ran(seed)-1.0;
        random_trans[3]=2*Ran(seed)-1.0;
        
        /* Steps: 1. Rotation around the cm = (0.0, 0.0, 0.0) */
        /*	      2. Translation  				  */
        ori_rotation((*protein).xyz, (*protein).natoms, random_rot);  
        
        sum = (random_trans[1]*random_trans[1])
             +(random_trans[2]*random_trans[2])
             +(random_trans[3]*random_trans[3]);
        sum=rad/sqrt(sum);
        
        (*protein).cm[1]=random_trans[1]*sum; 
        (*protein).cm[2]=random_trans[2]*sum;
        (*protein).cm[3]=random_trans[3]*sum;
        
        translation((*protein).xyz,(*protein).natoms,(*protein).cm);
       }

    energy=electrostatic_grid_energy(phi, (*protein).xyz, 
	       (*protein).charge, (*protein).natoms);
    return energy;
}

/****************************************************************/
/* Somethings for the book keeping				*/
/*..............................................................*/
/* The counting, i.e. the book keeping is done in this routinie */
/*								*/
void keep_book (BookKeep *book, int back)
{
    if (0==back) 
        {(*book).unsuccess = 0;
         (*book).accepted++;
         (*book).tried++;
        }
    else if (1==back)
        {(*book).unsuccess++;
         (*book).rejected++;
         (*book).tried++;
        }
    else if (back==2) 
        {(*book).excluded++;
         //(*book).tried++; -- not considered as a trial!!
        }
    else {muwarn("I have got something wrong");
	  fprintf(stderr,"Back is %d.\n",back);
         }
}
/****************************************************************/
/* Energy calculation						*/
/*..............................................................*/
/* function calculates the electrostatic interaction energy	*/
/* between two molecules using a precalculated grid for molecule*/
/* one.								*/

double electrostatic_grid_energy(dGrid *phi,
				 double **xyz, double *charge, int atoms)

{
    int n;
    double energy   = 0.0;
    int warn=0;

    for (n=1; n<=atoms; n++)
        {energy=energy+charge[n]*interpolate(phi, xyz[n], &warn);
        }

    if (warn!=0)
        {muwarn("Some Atoms outside the grid!");
         fprintf(stderr,"Number of atoms: %d\n", warn);
        }

    return (energy);
}

/****************************************************************/
/* IO Stuff							*/
/*..............................................................*/
/* Print the history informations to the file			*/
/*								*/

void fprint_history(FILE *fpout, BookKeep *book)
{
    fprintf(fpout,"-------------- History Informations ----------------------\n");
    fprintf(fpout,"Tried:\t\t%10ld\t",(*book).tried);
    fprintf(fpout,"Check:\t\t%10ld\n",(*book).tried
				-(*book).accepted-(*book).rejected);
    fprintf(fpout,"Accepted:\t%10ld\t",(*book).accepted);
    fprintf(fpout,"Rejected:\t%10ld\n",(*book).rejected);
    fprintf(fpout,"Excluded:\t%10ld\t",(*book).excluded);
    fprintf(fpout,"Reinit.:\t%10ld\n",(*book).reinit);

}
/*..............................................................*/
/* Calculates the transformation and prints it to a file. The	*/
/* 'temperature' vector of mus is set to one and used as weight	*/ 
/*								*/
void fprint_trans(FILE *fpout, double energy, Mol *mus)
{
    static int flag=0;
    int i;
    double center[4], trans[4];
    double **rot_matrix;
    double rms_value;

    rot_matrix=dmatrix(1,3,1,3);

    if (flag==0)
        {flag=1;
         fprintf(stdout,"I set the temperature vector to 1.0\n");
         for (i=1; i<=(*mus).natoms; i++)
             {(*mus).bval[i]=1.0;
             }
        }
    rms_value=kabsch(rot_matrix, center, trans, (*mus).save_xyz, (*mus).xyz,
		 (*mus).natoms, (*mus).bval);

    if (0.001 < fabs(rms_value))
        {muwarn("Something wrong in fprint_trans()");
         fprintf(stdout,"RMS value: %f\n",rms_value);
        }

    fprintf(fpout,"ORIENTATION %E %E %E %E %E %E %E %E %E %E %E %E %E %E %E %E\n",
	       energy,
	       trans[1],trans[2],trans[3], 
	       center[1],center[2],center[3], 
	       rot_matrix[1][1],rot_matrix[1][2],rot_matrix[1][3], 
	       rot_matrix[2][1],rot_matrix[2][2],rot_matrix[2][3], 
	       rot_matrix[3][1],rot_matrix[3][2],rot_matrix[3][3]);


    free_dmatrix(rot_matrix,1,3,1,3);
}

