/* G. Matthias Ullmann 					*/
/* Berlin, 04.10.1995					*/
/* Large changes 31.10.1996				*/
/* .....................................................*/

#ifndef __dock_help__
#define __dock_help__

#include <zlib.h>
#include "mol.h"
#include "constants.h"
#include "grid.h"


#define MAX_REINIT 1000
#define RUN "************************ Run %5d ***********************\n"


typedef struct {int nruns;
		FILE *output;		/* Output file					*/
		FILE *pot_out;		/* outer potential file				*/
		FILE *pot_in;		/* inner potential file				*/
		FILE *excl;		/* exclusion grid file				*/
		FILE *dockdens;		/* docking density file				*/
		gzFile traj;		/* trajectory file				*/
		gzFile structene;	/* low energy structures file			*/
                gzFile structdist;	/* low distance structures file		       	*/
		long seed;		/* Seed for the random number generator 	*/
		double rad_in; 		/* initial cm separation distance		*/
		double rad_out; 	/* maxial cm separation distance 		*/
					/* maximum distance for deviation calculation 	*/
		double separation;	/* distance at which the testing    		*/
					/* of the exclusion grid is started 		*/
					/* = minimum distance for deviation calculation */
		int istuck;
		int max_unsuccess;
		double low_energy;
		double **low_struct;
		double dev_trans;	/* trans_deviation		 		*/
		double dev_rot;	        /* rot_deviation		 		*/
		double temp;	        /* temeratures at annealing steps		*/
		long steps;		/* number of steps at this temerature		*/
		int nprint;     	/* for writing trajectories */
                int nprintdist; 	/* for writing dist file */
                int nprintdens; 	/* for writing dend file */
                double cmdist; 		/* distance from which on to save */
                int start;              /* where to start the simulation: 
					        0 -- use the input coordinates 
				                1 -- start on the inner sphere with rad_in
					*/
	       } Run_para;

typedef struct {long excluded;
		long accepted;
		long tried;
		long reinit;
		long rejected;
		long unsuccess;
	       } BookKeep;
/*------------------------------------------------------*/
/*-------- Function declaration ------------------------*/
/*------------------------------------------------------*/

/* IO */
void fprint_history(FILE *fpout, BookKeep *book);
void fprint_trans(FILE *fpout, double energy, Mol *mus);

void keep_book (BookKeep *book, int back);
void GetProtein(FILE *fp, char *name, Mol *p);

/* MC stuff */
double init_orientation(Mol *protein, double rad, 
		        dGrid *phi, long *seed, int start);
void accept(Mol *protein);
double mc_run(dGrid *phi_out, dGrid *phi_in, cGrid *excl, 
	    Mol *protein, Run_para *para, iGrid *dens);
int mc_move(double *ener, dGrid *phi_out, dGrid *phi_in, cGrid *excl, 
	    Mol *protein, double separation, double dev_rot, 
	    double dev_trans, double temp, long *seed);

/* Coulomb energy via gird interpolation */
double electrostatic_grid_energy(dGrid *phi,
				 double **xyz, double *charge, int atoms);


void alloc_excl(cGrid *excl, double *min, double *max);

#endif  /* __dock_help__ */
