/* Matthias Ullmann 					*/
/* Berlin, 23.01.1996					*/
/* Exclusion Grid					*/
/* CHANGE not tested in the char-Version		*/
/* .....................................................*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>
#include "memutil.h"
#include "util.h"
#include "random.h"
#include "constants.h"
#include "mathematik.h"
#include "grid.h"
#include "exclude.h"

/*--------------------------------------------------------------*/
/* exclusion grid stuff						*/
/* ....................................................	*/
/* NOT! like Northrup					*/
/* Testing, if a protein penetrates an exclusion grid   */
/* it returns 1, if the protein penetrates the grid 	*/
/*  							*/
/* exl - exclusion grid (array of 1 and 0)		*/
/* exl_dim - extension of the exclusion grid (xyz)	*/
/* xyz - coordinates of the test protein		*/
/* surf - atom numbers of surface atoms			*/
/* surf_atoms - number of surface atoms			*/

int exclude(cGrid *excl, double **xyz, int *surf, int surf_atoms)
{
int bool=0;
int i,j,k, n;

if (surf_atoms<=0) muerror("surf_atoms<=0");
for (n=1; n<=surf_atoms; n++)
    {i=rint(xyz[surf[n]][1]*(*excl).inv_spacing); 
     j=rint(xyz[surf[n]][2]*(*excl).inv_spacing); 
     k=rint(xyz[surf[n]][3]*(*excl).inv_spacing); 
     if (!(  (i<(*excl).imin[1]) || (i>(*excl).imax[1]) 
	   ||(j<(*excl).imin[2]) || (j>(*excl).imax[2])
	   ||(k<(*excl).imin[3]) || (k>(*excl).imax[3])))
         {if ((*excl).grid[i][j][k]==1)
	     {bool=1;
	      break;
	     }
         }
    }
return bool;
}

/* ....................................................	*/
/* NOT! like Northrup					*/
/* procedure constructs an exclusion grid.		*/
/*							*/
/* ix holds the dimension of the grid and becomes 	*/
/* assigned during the procedure.			*/
/*							*/
/* probe_radius is the averaged radius of the surface 	*/
/* atoms of protein2. It can be set to lower values to  */
/* soft the interaction.				*/
void exlmake(cGrid *excl, double probe,
	     double **xyz, int atoms, double *radius)

{

double max[4], min[4];
double rexlmax;


/* Code */
/* looking for the atoms with the largest and the smallest coordinates */
min_max_coord(min, max, xyz, atoms);

/* to make a symmetric cubus with respect to the origin */
if (max[1]>-min[1]) min[1]=-max[1]; else max[1]=-min[1];
if (max[2]>-min[2]) min[2]=-max[2]; else max[2]=-min[2];
if (max[3]>-min[3]) min[3]=-max[3]; else max[3]=-min[3];

/* the grid should be a little bit larger than the extention of	*/
/* protein. It must be larger about the same extention as 	*/
/* rexlmax: Atom radius of protein1 + atomradius of protein2	*/
/* (for protein2 averaged radius)				*/
/* Here is the place, where the docking can be softed		*/

rexlmax=max_element(radius, atoms) + probe;

max[1]=max[1]+rexlmax; max[2]=max[2]+rexlmax; max[3]=max[3]+rexlmax; 
min[1]=max[1]-rexlmax; min[2]=max[2]-rexlmax; min[3]=min[3]-rexlmax; 

excl_to_grid(excl, probe, min, max, xyz, radius, atoms);
}


/*..............................................................*/
/* CHANGE: 1. Warning when i,j,k outside the grid.		*/
/*	   2. trennen - allocation and assing			*/

/* not tested */
void excl_to_grid(cGrid *excl, double probe, double *min, double *max,
	          double **xyz, double *radius, int atoms)
{
    int i,j,k,n;
    int ii, jj, kk, ih, jh, kh;
    double temp[4];
    int klim;
    double rexlmax, rexl, rexl2, half_spacing, dist;

    /* Code */
    if ((*excl).spacing<=0.0) 
        muerror("Grid spacing <= zero. excl_to_grid()");
    (*excl).inv_spacing=1.0/(*excl).spacing;

    (*excl).imax[1]=ceil(max[1]*(*excl).inv_spacing); 
    (*excl).imax[2]=ceil(max[2]*(*excl).inv_spacing); 
    (*excl).imax[3]=ceil(max[3]*(*excl).inv_spacing); 

    (*excl).imin[1]=floor(min[1]*(*excl).inv_spacing); 
    (*excl).imin[2]=floor(min[2]*(*excl).inv_spacing); 
    (*excl).imin[3]=floor(min[3]*(*excl).inv_spacing); 

    /* make the grid spacing to odd integer, 
       if they are not odd integer */
    odding_down((*excl).imin,3);
    odding_up((*excl).imax,3);

    (*excl).grid=char_3tensor((*excl).imin[1], (*excl).imax[1], 
                              (*excl).imin[2], (*excl).imax[2], 
                              (*excl).imin[3], (*excl).imax[3]);

    /* initializing */
    for (i=(*excl).imin[1];i<=(*excl).imax[1]; i++)
        for (j=(*excl).imin[2];j<=(*excl).imax[2]; j++)
            for (k=(*excl).imin[3];k<=(*excl).imax[3]; k++)
                (*excl).grid[i][j][k]=0;

    /* assigning the exclusion grid */
    /* klim is half the number minus 1 of grid points in one dimension, */
    /* which can maximal lay within a atom and the proberadius  */

    rexlmax=max_element(radius, atoms) + probe;
    klim=ceil((rexlmax+(1.5*(*excl).spacing))*(*excl).inv_spacing);
    half_spacing=(*excl).spacing/2.0;
 
    for (n=1; n<=atoms; n++)
        {rexl=radius[n]+probe;
        rexl2 = rexl * rexl;
         /* find the home grid point */
         ih=ceil((xyz[n][1]+half_spacing)*(*excl).inv_spacing); 
         if (xyz[n][1]<0.0) ih=ih-1;
         jh=ceil((xyz[n][2]+half_spacing)*(*excl).inv_spacing); 
         if (xyz[n][2]<0.0) jh=jh-1;
         kh=ceil((xyz[n][3]+half_spacing)*(*excl).inv_spacing); 
         if (xyz[n][3]<0.0) kh=kh-1;
         /* looping through all neighbors */
         for (kk=-klim; kk<=klim; kk++)  /* z */
             {k=kk+kh;
              if ((k<=(*excl).imax[3])||(k>=(*excl).imin[3]))
                 {for (jj=-klim; jj<=klim; jj++)  /* y */
                      {j=jj+jh;
                       if ((j<=(*excl).imax[2])||(j>=(*excl).imin[2]))
                          {for (ii=-klim; ii<=klim; ii++)  /* x */
                               {i=ii+ih;
                                if ((i<=(*excl).imax[1]) ||(i>=(*excl).imin[1]))
                                   {temp[1] = i*(*excl).spacing-xyz[n][1];
                                    temp[2] = j*(*excl).spacing-xyz[n][2];
                                    temp[3] = k*(*excl).spacing-xyz[n][3];
                                    dist= (temp[1]*temp[1]) + 
                                          (temp[2]*temp[2]) + 
                                          (temp[3]*temp[3]);
                                    if (dist<(rexl*rexl)) 
                                        {(*excl).grid[i][j][k]=1;
                                        }
                                   }
                               }
                          }
                      }
                 }
             }	     
        }
 }
/*..............................................................*/
/* prints the precaculated exclusion grid to a file		*/
void fprint_excl(FILE *fpout,cGrid *excl)
{
    int i,j,k;

    PRINT_iVEC(fpout,(*excl).imin);
    PRINT_iVEC(fpout,(*excl).imax);
    fprintf(fpout,"%8.3lf\n", (*excl).spacing);

    for (i=(*excl).imin[1];i<=(*excl).imax[1]; i++)
        for (j=(*excl).imin[2];j<=(*excl).imax[2]; j++)
            for (k=(*excl).imin[3];k<=(*excl).imax[3]; k++)
                fprintf(fpout,"%d  ",(*excl).grid[i][j][k]);
}

/*..............................................................*/
/* prints the precaculated exclusion grid to a file in binary mode*/
void fprint_excl_binary(FILE *fpout,cGrid *excl)
{
    int i,j,k;
    double probe=0.0;

    SAVE_fwrite(&((*excl).imax[1]), sizeof(int), 3, fpout);
    SAVE_fwrite(&((*excl).imin[1]), sizeof(int), 3, fpout);
    SAVE_fwrite(&((*excl).spacing), sizeof(double), 1, fpout);
    SAVE_fwrite(&(probe), sizeof(double), 1, fpout);

    for (i=(*excl).imin[1];i<=(*excl).imax[1]; i++)
        for (j=(*excl).imin[2];j<=(*excl).imax[2]; j++)
            for (k=(*excl).imin[3];k<=(*excl).imax[3]; k++)
                SAVE_fwrite(&((*excl).grid[i][j][k]), sizeof(char), 1, fpout);
}

/*..............................................................*/
/* reads the precaculated exclusion grid from a file in binary mode*/
void fread_excl_binary(FILE *fpin, cGrid *excl)
{
    int i,j,k;
    double probe;

    SAVE_fread(&((*excl).imax[1]), sizeof(int), 3, fpin);
    SAVE_fread(&((*excl).imin[1]), sizeof(int), 3, fpin);
    SAVE_fread(&((*excl).spacing), sizeof(double), 1, fpin);
    SAVE_fread(&(probe), sizeof(double), 1, fpin);
    (*excl).inv_spacing=1.0/(*excl).spacing;

    /* allocation of the tensor */
    (*excl).grid=char_3tensor((*excl).imin[1], (*excl).imax[1], 
                              (*excl).imin[2], (*excl).imax[2], 
                              (*excl).imin[3], (*excl).imax[3]);

    for (i=(*excl).imin[1];i<=(*excl).imax[1]; i++)
        for (j=(*excl).imin[2];j<=(*excl).imax[2]; j++)
            for (k=(*excl).imin[3];k<=(*excl).imax[3]; k++)
                SAVE_fread(&((*excl).grid[i][j][k]), sizeof(char), 1, fpin);
}

