/* G. Matthias Ullmann                                          */
/* Bayreuth, 3.10.2010                                          */
/*                                                              */
/* calculated energy:                                           */
/* it reads a dx-file and a pqr-file and multiplies the charges */
/* with the potential it is assumed that the potential is given */
/* in kcal/mol/e                                                */
/*                                                              */
/* ............................................................ */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <zlib.h>
#include "mol.h"
#include "memutil.h"
#include "util.h"
#include "surface.h"
#include "random.h"
#include "exclude.h"
#include "grid.h"
#include "dock-help.h"

#define USAGE \
"usage: get-potential [pot-receptor.dx] [ligand.pqr] \n\n\
Program calculates the potential at given points.\n"

/**********************************************************/
int main(int argc, char *argv[])
{
    /* Varialbles */
    Mol p;
    dGrid phi;
    OpenDX dxpot;
    FILE *fp;
    int n, warn=0;

    /* Usage */
    usage(USAGE, argc, 3);

    /* read the data */
    fp = file_open(argv[1],"r");
    ReadDX(&dxpot,fp);
    DX2Grid(&phi, &dxpot);
    //    fprintf(stdout,"\nPotential Grid:\n");
    //    dGrid_info(stdout, &phi);
    fclose(fp);     

    read_PQRM(argv[2], &p);

    /* calculate the energy */
    fprintf(stdout,"# Potential\n");
    for (n=1; n<=p.natoms; n++)
      {fprintf(stdout, "%e %e %e %e\n",
	       p.xyz[n][1], p.xyz[n][2], p.xyz[n][3],
	       interpolate(&phi, p.xyz[n], &warn));
      }

    if (warn!=0)
        {muwarn("Some Points outside the grid!");
         fprintf(stderr,"Number of atoms: %d\n", warn);
        }

    return 0;
}

