/* ...................................................................*/
/* M. Ullmann, Bayreuth March 2004                                    */
/*                                                                    */
/* Program generates a 2D histogram from column data.                 */
/* The values are between 0.0 and 1.0, i.e. normalized on the maximum */
/*                                                                    */
/* ...................................................................*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mathematik.h"
#include "util.h"
#include "memutil.h"



/**********************************************************/
int main(int argc, char *argv[])
{
    /* Varialbles */
    FILE *fpin;
    double *val1, *val2;
    int **histo;
    double step1, step2, test, min1, max1, min2, max2;
    int n, i,j, imin1, imax1, ih, jh, imin2, imax2, max;
    int norm;
    char line[LINELENGTH];

#define USAGE \
"usage: histogram2d [file] [step1] [step2] [norm-flag]\n\
    norm-flag = 0 -- no normalization\n\
    norm-flag = 1 -- on the highest value\n\
    norm-flag = 2 -- on total number (probability)\n"
      
    /* Code */
    usage(USAGE,argc,5);
    
    fpin=file_open(argv[1],"r");
    step1 = atof(argv[2]);
    step2 = atof(argv[3]);
    norm = atoi(argv[4]);

    /* count the lines in the file and allocate memory */
    n = 0;
    while (fgets(line, LINELENGTH, fpin) != NULL)
        {if (1==sscanf(line,"%lf",&test)) n++;
        }
    fprintf(stdout,"# %d lines\n",n);
    rewind(fpin);
    val1 = dvector(1,n);
    val2 = dvector(1,n);

    /* read the file */
    i = 0;
    while (fgets(line, LINELENGTH, fpin) != NULL)
        {if (1==sscanf(line,"%lf",&test)) 
            {i++;
             sscanf(line,"%lf %lf",&val1[i],&val2[i]);
            }
        }
    fclose(fpin);

    /* now generate the histogram */
    min1 = min_element(val1, n);
    max1 = max_element(val1, n);
    min2 = min_element(val2, n);
    max2 = max_element(val2, n);

    fprintf(stdout,"# Val1: Min: %e  Max: %e\n",min1,max1);
    fprintf(stdout,"# Val2: Min: %e  Max: %e\n",min2,max2);

    imin1 = (int)floor(min1/step1);
    imax1 = (int) ceil(max1/step1);
    fprintf(stdout,"# Histogram D1: %d ... %d\n",imin1, imax1);

    imin2 = (int)floor(min2/step2);
    imax2 = (int) ceil(max2/step2);
    fprintf(stdout,"# Histogram D2: %d ... %d\n",imin2, imax2);

    histo = imatrix(imin1, imax1, imin2, imax2);
    for (i=imin1; i<=imax1; i++) 
        for (j=imin2; j<=imax2; j++) 
            {histo[i][j]=0;}


    for (i=1; i<=n; i++) 
        {ih=(int)rint(val1[i]/step1);
	 if (ih<imin1) {ERROR("Error 1");}
	 if (ih>imax1) {ERROR("Error 2");}
         jh=(int)rint(val2[i]/step2);
         if (jh<imin2) {ERROR("Error 3");}
         if (jh>imax2) {ERROR("Error 4");}

         histo[ih][jh]++;
        }

    /* seach for maximal number */
    max = 0;
    for (i=imin1; i<=imax1; i++) 
        for (j=imin2; j<=imax2; j++) 
            {if (histo[i][j]>max) max = histo[i][j];
            }

    /* print result */
    for (i=imin1; i<=imax1; i++) 
        {for (j=imin2; j<=imax2; j++) 
             {if (norm==0)
                 fprintf(stdout,"%e %e %d\n",i*step1,j*step2,histo[i][j]);
              else if (norm==1)
                 fprintf(stdout,"%e %e %e\n",i*step1,j*step2,
                     (double)((double)histo[i][j]/(double)max));
              else if (norm==2)
                 fprintf(stdout,"%e %e %e\n",i*step1,j*step2,
                     (double)((double)histo[i][j]/(double)n));
              else ERROR("unknown norm-flag");
             }
        }

    return 0;
}
