/* G. Matthias Ullmann 						*/
/* Bayreuth, June 2009					        */
/*								*/
/* Program reads in a potential from an OpenDX file, calculates */
/* the potential at a given number of points on a spehere       */
/* around the origin and gives out the mean value and the       */
/* standard deviation.                                          */
/*								*/
/* ............................................................ */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include "memutil.h"
#include "util.h"
#include "random.h"
#include "mathematik.h"
#include "grid.h"
#include "dock-help.h"

/* ............................................................ */
int main(int argc, char *argv[])
{
    /* Varialbles */
    time_t tim;
    dGrid phi;
    FILE *fp;
    int nrun, i, warn=0;
    double rad, vec[4], pot;
    double meanpot, devpot;
    long seed;


    /* Usage */
    usage("usage: pot-test [potential.dx] [radius] [#points] \n\n", argc, 4);

    /* read the potential file */
    fp=file_open(argv[1],"r");
    ReadDX(&phi,fp);
    fclose(fp);
    dGrid_info(stdout, &phi);

    /* place random point on a surface of a sphere with the radius r */
    seed = -time(&tim);
    rad  = atof(argv[2]);
    nrun = atoi(argv[3]);
    meanpot = 0.0; devpot = 0.0;
    for (i=1; i<=nrun; i++)
        {vec[1]=(2*ran2(&seed)-1.0);
         vec[2]=(2*ran2(&seed)-1.0);
         vec[3]=(2*ran2(&seed)-1.0);
         normalize(vec,3);
         vec[1]=rad*vec[1];
         vec[2]=rad*vec[2];
         vec[3]=rad*vec[3];
         pot = interpolate(&phi, vec, &warn);
         if (warn!=0)
            {muwarn("Some Atoms outside the grid!");
             fprintf(stderr,"Number of atoms: %d\n", warn);
            }
         meanpot = meanpot + pot;
         devpot  = devpot  + pot*pot;
         fprint_DUMMYatomPQR(stdout,1,"DUMM",vec, pot);
        }
    meanpot = meanpot/nrun;
    devpot  = sqrt(devpot)/nrun;
    fprintf(stderr,"Average Potential at r=%8.2f: (%e +/- %e) kT\n", 
            rad, meanpot, devpot);
    fprintf(stdout,"Average Potential at r=%8.2f: (%e +/- %e) kT\n", 
            rad, meanpot, devpot);

    return 0;
}
