/* Matthias Ullmann 					*/
/* Berlin, 08.11.1995					*/
/*							*/
/* .....................................................*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <math.h>
#include "mol.h"
#include "properties.h"
#include "memutil.h"
#include "util.h"
#include "grid.h"



/**********************************************************/

int main(int argc, char *argv[])
{
    /* Varialbles */
    Mol p;
    double dummy[4];
    double max[4], min[4];
    double cm[4], dipolvec[4];
    double *dist_cm;
    double tot_mass, tot_chrg, dipl, gyrrad, max_cm_dist;
    
    usage("pqrm-info [pqrm] ",argc,2);

    read_PQRM(argv[1], &p);
    dist_cm = dvector(1, p.natoms);

    /* calculate some main properties of the molecule */

    /* calculate some main properties of the molecule */
    tot_mass    = center_of_mass(p.xyz, p.mass, p.natoms, cm);
    tot_chrg    = charge(p.charge, p.natoms);
    dipl        = dipol(p.xyz, p.charge, p.natoms, cm, dipolvec);
    gyrrad      = gyrationradius(p.xyz, p.natoms, cm);
    max_cm_dist = max_dist(p.xyz, p.natoms, cm, dist_cm);

    fprintf(stdout,"Number of atoms:         %d\n",p.natoms);
    fprintf(stdout,"Number of bonds:         %d\n",p.nbonds);
    fprintf(stdout,"Number of angles:        %d\n",p.nangles);
    fprintf(stdout,"Number of torsions:      %d\n",p.ntorsions);
    fprintf(stdout,"Number of impropers:     %d\n",p.nimprop);
    fprintf(stdout,"Mass:            %8.3f\n",tot_mass);
    fprintf(stdout,"Center of Mass:  %8.3f  %8.3f  %8.3f\n",cm[1],cm[2],cm[3]);
    fprintf(stdout,"Radius of Gyration:            %8.3f\n",gyrrad);
    //fprintf(stdout,"Surface:         %8.3f\n",p.area);
    //fprintf(stdout,"Volume:          %8.3f\n",p.volume);
    fprintf(stdout,"Charge:          %8.3f\n",tot_chrg);
    fprintf(stdout,"Dipol:           %8.3f  = %8.3f Debye\n",dipl,dipl*4.802);
    fprintf(stdout,"Dipolvector-KM:  %8.3f  %8.3f  %8.3f\n", dipolvec[1],dipolvec[2],dipolvec[3]);
    fprintf(stdout,"Maximal Distance from the center of mass:   %8.3f\n",max_cm_dist);

    /* print a dummy for the center of mass */
    fprint_DUMMYatom(stdout, 1, " CM ", "DUM", cm, 0.0, tot_mass);

    /* print two  dummies for the dipol */
    dummy[1]    =   dipolvec[1]*0.5 + cm[1];
    dummy[2]    =   dipolvec[2]*0.5 + cm[2];
    dummy[3]    =   dipolvec[3]*0.5 + cm[3];
    /* positive pole */
    fprint_DUMMYatom(stdout, 1, "DIP1", "DUM", dummy, 0.0, 0.0);
    dummy[1]    =  -dipolvec[1]*0.5 + cm[1];
    dummy[2]    =  -dipolvec[2]*0.5 + cm[2];
    dummy[3]    =  -dipolvec[3]*0.5 + cm[3];
    /* negative pole */
    fprint_DUMMYatom(stdout, 2, "DIP2", "DUM", dummy, 0.0, 0.0);
    /* define a outer frame */
    min_max_coord(min, max, p.xyz, p.natoms);
    fprintf(stdout,"Extension of the protein:\n");
    fprintf(stdout,"Minimal coordinates: %f %f %f\n",min[1],min[2],min[3]);
    fprintf(stdout,"Maximal coordinates: %f %f %f\n",max[1],max[2],max[3]);


    return 0;
}
