/* ...................................................................*/
/*								      */
/* ...................................................................*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mol.h"
#include "crd_atom.h"
#include "constants.h"
#include "mathematik.h"
#include "util.h"
#include "memutil.h"

#define USAGE \
"usage: psfcrd2pqr [psf] [crd] [output format]\n\
 Possible formats: \n\
     [PQR]  -- plain PQR        \n\
     [PQRC] -- PQR with chainid \n\
     [PQRM] -- PQRM, i.e. with mass \n\
     [PQRX] -- extended PQR for extended MEAD \n\
"

#define CONECT 0


/* .................................................... */
/* recounts the aminoacids and the atoms in a MUS-file  */
/* ATTENTION                                            */
/* Care must be taken, that two neightboured amino-     */
/* acids does not have the same id number!!!!           */
/*                                                      */
void recount_aa_with_chain1000(Mol *protein)
{
int i,aa, ihelp, save;
char save_chain, save_insert,save_name[5];

/* Assignment */
ihelp=0;
aa=(*protein).res_no[1];
i=1;
save=(*protein).res_no[i];
save_insert=(*protein).insert[i];
save_chain=(*protein).chain[i];
strcpy(save_name,(*protein).residue[i]);
(*protein).res_no[i]=aa;
(*protein).atomno[i]=i;
for (i=2; i<=(*protein).natoms; i++) 
    {(*protein).atomno[i]=i;
     if (  ((*protein).res_no[i] != save)
         ||(0!=strncmp((*protein).residue[i],save_name,3))
         ||(save_insert!=(*protein).insert[i])
         ||(save_chain!=(*protein).chain[i]))
        {if (save_chain!=(*protein).chain[i]) 
            {ihelp++;
             aa=(*protein).res_no[i]+ihelp*1000;
            }
         else {aa=(*protein).res_no[i]+ihelp*1000;
              } 
         save_insert=(*protein).insert[i];
         save=(*protein).res_no[i];
         strcpy(save_name,(*protein).residue[i]);
         save_chain=(*protein).chain[i];
         (*protein).res_no[i]=aa;
        }
     else (*protein).res_no[i]=aa;
    }
}
/**********************************************************/
int main(int argc, char *argv[])
{
    /* Varialbles */
    PSF_atom proteinPSF;
    CRD_atom proteinCRD;
    Mol proteinMUS;
    double chrg=0.0;
    char form[8];
    int i;
    //int **conect;
    
    /* Code */
    usage(USAGE,argc,4);

    strcpy(form,argv[3]);

    fprintf(stderr,"Read PSF File\n");
    read_PSF(argv[1], &proteinPSF);
    fprintf(stderr,"Read CRD File\n");
    read_CRD(argv[2], &proteinCRD);

    if (proteinCRD.natoms!=proteinPSF.natoms)
        {ERROR("Number of atoms in PSF and CRD must be identical.");
        } 
    psfcrd2mol(&proteinMUS, &proteinPSF, &proteinCRD);
    assign_chains(&proteinMUS,&proteinCRD);

    chrg=0.0;
    for (i=1; i<=proteinMUS.natoms; i++)
        {chrg+=proteinMUS.charge[i];
         proteinMUS.radius[i] = proteinCRD.variable[i];
        }
    fprintf(stderr,"Charge:%f\n",chrg);

    if (0==strcmp(form,"PQR"))
       {recount_aa_with_chain1000(&proteinMUS);
        fprint_PQR(stdout,&proteinMUS);
       }
    else if (0==strcmp(form,"PQRC"))
       fprint_PQRChain(stdout,&proteinMUS);
    else if (0==strcmp(form,"PQRX"))
       {recount_aa_with_chain1000(&proteinMUS);
        fprint_PQRX(stdout,&proteinMUS);
       }
    else if (0==strcmp(form,"PQRM"))
      {//recount_aa_with_chain1000(&proteinMUS);
        fprint_PQRM(stdout,&proteinMUS);
       }
    else muerror("Unknown output format");

    /*
    if (CONECT==1)
       {conect=imatrix(1,proteinMUS.natoms,1, MAX_CONECT);
        create_conect(conect,&proteinMUS);
        fprint_conect2(stdout,conect,proteinMUS.natoms);
        //fprint_conect1(stdout,conect,proteinMUS.number_of_atoms);
       }
    */
    fprintf(stdout,"END\n");
    return 0;
}
